// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Text;

namespace OpenNETCF.Telephony
{
	/// <summary>
	/// Provides functionality for handling incoming calls. 
	/// </summary>
	public class Call : IDisposable
	{
		private IntPtr m_hCall;
		private CallState m_state;
		private CallInformation m_info;
		private string m_callerID = "";
		internal int m_addressID;
		internal Line m_line;
        /// <summary>
        /// The current lineMessage associated with the call
        /// </summary>
        private LineMessage m_lineMessage;

        /// <summary>
        /// New in v1.1. Gets the current parameter values specifying a change in status of the line the application currently has open.
        /// </summary>
        /// <remarks>LineMessage dwParams may be different depending on the state.  Please see http://msdn2.microsoft.com/en-us/library/ms899557.aspx for more informatio. </remarks>
        /// <version>1.1</version>
        //TODO nned to figure out how linemessage works.
        public LineMessage LineMessage
        {
            get
            {
                return m_lineMessage;
            }
            internal set
            {
                m_lineMessage = value;
            }
        }
        /// <summary>
        /// Returns a handle to the Call object so that it may be used through
        /// Platform Invocation
        /// </summary>
        /// <bugs>
        /// <bug id="39">Pointer to Call object should be public readonly</bug>
        /// </bugs>
	    public IntPtr Handle
	    {
	        get { return m_hCall; }
	    }

		internal Call(IntPtr hCall)
		{
			m_hCall = hCall;
			m_info = new CallInformation(1024);
		}

        /// <summary>
        /// Terminates the current call
        /// </summary>
        /// <returns></returns>
		public bool Hangup()
		{
            if (m_hCall == IntPtr.Zero)
                return false;

            //MA: Should this be >= 0 http://msdn2.microsoft.com/en-us/library/ms927994.aspx
            /*From MSDN:
             * Returns a positive request identifier if the function is completed asynchronously, 
             * or a negative error number if an error occurs. The dwParam2 parameter of the 
             * corresponding LINE_REPLY message is zero if the function succeeds or it is a 
             * negative error number if an error occurs. The following table shows the return values for this function.
             */
            return NativeMethods.lineDrop(m_hCall, null, 0) >= 0;
		}

	    /// <summary>
	    /// Answer the call
	    /// </summary>
	    /// <returns></returns>
		public bool Answer()
		{
			return Answer(null);
		}

	    /// <summary>
	    /// Answer the call and send user-user to the caller.
	    /// </summary>
	    /// <param name="UserUserInfo"></param>
	    /// <returns></returns>
		public bool Answer(string UserUserInfo)
		{
			return NativeMethods.lineAnswer(m_hCall, UserUserInfo, UserUserInfo == null? 0: UserUserInfo.Length * Marshal.SystemDefaultCharSize) == 0;
		}

	    /// <summary>
	    /// Gets the state of the call
	    /// </summary>
		public CallState State { get { return m_state; } }
	    
	    /// <summary>
	    /// Retrieves information about the call
	    /// </summary>
		public CallInformation Info{ get { return m_info; } }
	    
	    /// <summary>
	    /// Returns the Caller ID for the call
	    /// </summary>
		public string CallerID { get { return m_callerID; } }
	    
	    /// <summary>
	    /// Returns a reference to the Line on which the call is being made
	    /// </summary>
		public Line Line { get { return m_line; } }
	    
	    /// <summary>
	    /// Returns the AddressID for the current call
	    /// </summary>
		internal int AddressID { get { return m_addressID; } }

		#region Delegate definitions
	    
	    /// <summary>
	    /// Delegate for handling call state events
	    /// </summary>
	    /// <param name="call"></param>
	    /// <param name="state"></param>
		public delegate void CallStateHandler(Call call, CallState state);
	    
	    /// <summary>
	    /// Delegate for handling call info events
	    /// </summary>
	    /// <param name="call"></param>
	    /// <param name="infoState"></param>
	    /// <param name="info"></param>
		public delegate void CallInfoHandler(Call call, CallInfoState infoState, CallInformation info);

        /// <summary>
        /// New v1.1 Delegate for handling call disconnect state
        /// </summary>
        /// <param name="call">current Call object</param>
        /// <param name="state">the current state of the call</param>
        /// <param name="disconnectMode">reason why the call was disconnected</param>
        /// <version>1.1</version>
        public delegate void CallDisconnectHandler(Call call, CallState state, DisconnectMode disconnectMode);
		#endregion

		#region Events
	    
	    /// <summary>
	    /// CallInfo Event
	    /// </summary>
		public event CallInfoHandler CallInfo;
	    
	    /// <summary>
	    /// CallState Event
	    /// </summary>
		public event CallStateHandler CallState;

        /// <summary>
        /// New v1.1. Occurs when a call is disconnected.
        /// </summary>
        /// <remarks>
        /// This event notifies listeners when a call has been disconnected.  This event should be registered for during a
        /// <see cref="OpenNETCF.Telephony.Line.NewCall"/> event.
        /// </remarks>
        /// <version>1.1</version>
        public event CallDisconnectHandler Disconnected;

        /// <summary>
        /// New in v1.1. Occurs when a call is connected. This event should be registered for during a
        /// <see cref="OpenNETCF.Telephony.Line.NewCall"/> event.
        /// </summary>
        public event CallStateHandler Connected;
		#endregion
		
		#region Event handlers
	    
	    /// <summary>
	    /// (internal) Handles Call State events
	    /// </summary>
	    /// <param name="msg"></param>
		protected internal void OnLineCallState(LineMessage msg)
		{
			m_state = (CallState)msg.Param1.ToInt32();
            //New for v1.1 - Added a discconect event
            if (m_state == OpenNETCF.Telephony.CallState.Disconnected && this.Disconnected!=null)
            {
                //If it's disconnect raise a disconnect event
                this.Disconnected(this, m_state, (DisconnectMode)msg.Param2.ToInt32());
            }
            else if (m_state == OpenNETCF.Telephony.CallState.Connected && this.Connected != null)
            {
                //If it's connected raise the connected event
                this.Connected(this, m_state);
            }
            else
            {
                if (CallState != null)
                    CallState(this, m_state);
                //			if ( m_state == LINECALLSTATE.LINECALLSTATE_IDLE )
                //				Dispose();
            }
		}

	    /// <summary>
	    /// (internal) Handles Call Info Events
	    /// </summary>
	    /// <param name="msg"></param>
		protected internal void OnLineCallInfo(LineMessage msg)
		{
			CallInfoState infoState = (CallInfoState)msg.Param1.ToInt32();
			LoadCallInfo();
			if ( (infoState & CallInfoState.CallerId) == CallInfoState.CallerId )
			{
				if (m_info.CallerIDSize != 0)
					m_callerID = Encoding.Unicode.GetString(m_info.Data, m_info.CallerIDOffset, m_info.CallerIDSize - 1);
			}
			if ( CallInfo != null )
				CallInfo(this, (CallInfoState)msg.Param1.ToInt32(), m_info);
		}

	    /// <summary>
	    /// (internal) 
	    /// </summary>
	    /// <param name="msg"></param>
		protected internal void OnLineMonitorTones(LineMessage msg)
		{
		}

		#endregion

	    /// <summary>
	    /// Gets the current call information
	    /// </summary>
		protected internal void LoadCallInfo()
		{
			m_info = new CallInformation(1024); 
			m_info.Store(); 
			int ret = NativeMethods.lineGetCallInfo(m_hCall, m_info.Data); m_info.Load();
			if ( ret < 0 && (ErrorCode)ret != ErrorCode.InvalidCallHandle )
				throw new TelephonyException(ret);
		}

	    /// <summary>
	    /// Retrieves the current call state
	    /// </summary>
		protected internal void LoadCallState()
		{
			CallStatus status = new CallStatus(1024); 
			status.Store(); 
			int ret = NativeMethods.lineGetCallStatus(m_hCall, status.Data); 
			status.Load();
			if ( ret < 0 )
				throw new TelephonyException(ret);
			m_state = status.CallState;
		}

		#region IDisposable Members

	    /// <summary>
	    /// Disposes of the Call instance
	    /// </summary>
		public void Dispose()
		{
			if ( m_hCall != IntPtr.Zero)
			{
				NativeMethods.lineDeallocateCall(m_hCall);
				m_hCall = IntPtr.Zero;
			}
		}

		#endregion

	}
}
