// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// This structure contains call data. This data remains fixed during the call and is obtained with the lineGetCallInfo function.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class CallInformation : PhoneBase
    {
        /// <summary>
        /// Creates an instance of CallInformation
        /// </summary>
        public CallInformation(int nSize)
            : base(nSize)
        {
            TotalSize = nSize;
        }

        /// <summary>
        /// The minimum size for internal data storage
        /// </summary>
        protected override int MinimumSize { get { return 1024; } }
        /// <summary>
        /// 
        /// </summary>
        public string DisplayableAddress { get { return GetString(DisplayableAddressOffset, DisplayableAddressSize); } }
        /// <summary>
        /// 
        /// </summary>
        public string CalledParty { get { return GetString(CalledPartyOffset, CalledPartySize); } }
        /// <summary>
        /// 
        /// </summary>
        public string CalledID { get { return GetString(CalledIDOffset, CalledIDSize); } }
        /// <summary>
        /// 
        /// </summary>
        public string CallerID { get { return GetString(CallerIDOffset, CallerIDSize); } }
        /// <summary>
        /// 
        /// </summary>
        public string CallerIDName { get { return GetString(CallerIDNameOffset, CallerIDNameSize); } }

        internal int TotalSize;
        internal int NeededSize;
        /// <summary>
        /// Size, in bytes, of the portion of this data structure that contains useful data.
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Handle for the line device with which this call is associated.
        /// </summary>
        public IntPtr LinePtr;
        /// <summary>
        /// Device identifier of the line device with which this call is associated.
        /// </summary>
        public int LineDeviceID;
        /// <summary>
        /// Address identifier of the address on the line on which this call exists.
        /// </summary>
        public int AddressID;
        /// <summary>
        /// Value that specifies the current bearer mode of the call. It uses BearerMode.
        /// </summary>
        public BearerMode BearerMode;
        /// <summary>
        /// Rate of the call data stream, in bits per second (bps).
        /// </summary>
        public int Rate;
        /// <summary>
        /// Value that specifies the media mode of the data stream currently on the call.
        /// </summary>
        public MediaMode MediaMode;
        /// <summary>
        /// Uninterpreted by the API implementation and service provider.
        /// </summary>
        public int AppSpecific;
        /// <summary>
        /// In some telephony environments, the switch, or service provider can assign a unique identifier to each call. This enables the call to be tracked across transfers, forwards, or other events. The domain of these call identifiers and their scope is service provider-defined.
        /// </summary>
        public int CallID;
        /// <summary>
        /// Telephony environments that use the call identifier often may find it necessary to relate one call to another. 
        /// </summary>
        public int RelatedCallID;
        /// <summary>
        /// Specifies a collection of call-related parameters when the call is outgoing. 
        /// </summary>
        public CallParameterFlags CallParamFlags;
        /// <summary>
        /// alue that specifies the call states of type CallState for which the application can be notified on this call. 
        /// </summary>
        public CallState CallStates;
        /// <summary>
        /// Value that specifies the various digit modes using LINEDIGITMODE.
        /// </summary>
        public DigitMode MonitorDigitModes;
        /// <summary>
        /// Specifies the various media modes for which monitoring is currently enabled using LINEMEDIAMODE.
        /// </summary>
        public MediaMode MonitorMediaModes;
        /// <summary>
        /// The dialing parameters currently in effect on the call, of type LINEDIALPARAMS.
        /// </summary>
        public DialParameters DialParams;
        /// <summary>
        /// Value that identifies where the call originated.
        /// </summary>
        public CallOrigin Origin;
        /// <summary>
        /// Value that identifies where the call originated.
        /// </summary>
        public CallReason Reason;
        /// <summary>
        /// Value that identifies where the call originated.
        /// </summary>
        public int CompletionID;
        /// <summary>
        /// Number of application modules with different call handles with owner privilege for the call.
        /// </summary>
        public int OwnersCount;
        /// <summary>
        /// Number of application modules with different call handles with owner privilege for the call.
        /// </summary>
        public int MonitorsCount;
        /// <summary>
        /// Country/region code of the destination party. Zero if unknown.
        /// </summary>
        public int CountryCode;
        /// <summary>
        /// Number of the trunk over which the call is routed. This member is used for both incoming and outgoing calls. The Trunk member should be set to 0xFFFFFFFF if it is unknown.
        /// </summary>
        public int Trunk;
        /// <summary>
        /// Value that determines the validity and content of the caller party identifier data. The caller is the originator of the call.
        /// </summary>
        public int CallerIDFlags;
        /// <summary>
        /// Size, in bytes, of the field that contains the data identifying the caller.
        /// </summary>
        public int CallerIDSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CallerIDOffset;
        /// <summary>
        /// Size, in bytes, of the field that contains the data identifying the name of the calling party.
        /// </summary>
        public int CallerIDNameSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CallerIDNameOffset;
        /// <summary>
        /// Value that determines the validity and content of the called-party identifier data. The called party corresponds to the originally addressed party.
        /// </summary>
        public int CalledIDFlags;
        /// <summary>
        /// Size, in bytes, of the field that contains the called-party identifier number data.
        /// </summary>
        public int CalledIDSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CalledIDOffset;
        /// <summary>
        /// Size, in bytes, of the field that contains the called-party identifier name data.
        /// </summary>
        public int CalledIDNameSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CalledIDNameOffset;
        /// <summary>
        /// Value that determines the validity and content of the connected party identifier data. The connected party is the party that was actually connected to. This may be different from the called-party identifier if the call was diverted.
        /// </summary>
        public int ConnectedIDFlags;
        /// <summary>
        /// Size, in bytes, of the field that contains the connected party identifier number data
        /// </summary>
        public int ConnectedIDSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int ConnectedIDOffset;
        /// <summary>
        /// Size, in bytes, of the field that contains the connected party identifier name data.
        /// </summary>
        public int ConnectedIDNameSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int ConnectedIDNameOffset;
        /// <summary>
        /// Value that determines the validity and content of the redirection party identifier data. The redirection party identifies to the calling user the number toward which diversion was invoked.
        /// </summary>
        public int RedirectionIDFlags;
        /// <summary>
        /// Size, in bytes, of the field that contains the redirection party identifier number data.
        /// </summary>
        public int RedirectionIDSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int RedirectionIDOffset;
        /// <summary>
        /// Size, in bytes, of the field that contains the redirection party identifier name data.
        /// </summary>
        public int RedirectionIDNameSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int RedirectionIDNameOffset;
        /// <summary>
        /// Value that determines the validity and content of the redirecting party identifier data. The party that received the call identifies the new destination number or whatever data is detected to the call originator. 
        /// </summary>
        public int RedirectingIDFlags;
        /// <summary>
        /// Size, in bytes, of the field that contains the redirecting party identifier number data
        /// </summary>
        public int RedirectingIDSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int RedirectingIDOffset;
        /// <summary>
        /// Size, in bytes, of the field that contains the redirecting party identifier name data
        /// </summary>
        public int RedirectingIDNameSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int RedirectingIDNameOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds the application name of the application that first originated, accepted, or answered the call.
        /// </summary>
        public int AppNameSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. This is the name that an application can specify in the lineInitializeEx function. If the application specifies no such name, then the application's module file name is used instead.
        /// </summary>
        public int AppNameOffset;
        /// <summary>
        /// Specify the displayable string is used for logging purposes. 
        /// </summary>
        public int DisplayableAddressSize;
        /// <summary>
        /// Specify the displayable string is used for logging purposes. 
        /// </summary>
        public int DisplayableAddressOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds a user-friendly description of the called party
        /// </summary>
        public int CalledPartySize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int CalledPartyOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds a comment about the call provided by the application that originated the call using lineMakeCall.
        /// </summary>
        public int CommentSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CommentOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds raw display data.
        /// </summary>
        public int DisplaySize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int DisplayOffset;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int UserUserInfoSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int UserUserInfoOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds high-level compatibility data.
        /// </summary>
        public int HighLevelCompSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. The format of this data is specified by other standards (ISDN Q.931).
        /// </summary>
        public int HighLevelCompOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds low-level compatibility data.
        /// </summary>
        public int LowLevelCompSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. The format of this data is specified by other standards (ISDN Q.931).
        /// </summary>
        public int LowLevelCompOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds charging data.
        /// </summary>
        public int ChargingInfoSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. The format of this data is specified by other standards (ISDN Q.931).
        /// </summary>
        public int ChargingInfoOffset;
        /// <summary>
        /// Size, in bytes, of the variably sized device field that contains an array with DWORD-sized entries.
        /// </summary>
        public int TerminalModesSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int TerminalModesOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds device-specific data.
        /// </summary>
        public int DeviceSpecificSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int DeviceSpecificOffset;
        /// <summary>
        /// 
        /// </summary>
        public int CallTreatment;                                // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        public int CallDataSize;                                 // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        public int CallDataOffset;                               // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        public int SendingFlowspecSize;                          // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        public int SendingFlowspecOffset;                        // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        public int ReceivingFlowspecSize;                        // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        public int ReceivingFlowspecOffset;                      // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        public int AddressType;                                  // TAPI v3.0
    }
}
