// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Text;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// This structure contains bearer information that is used for setting different bearer-related information (specifically to enable functionality such as the Global System for Mobile Communications [Gsm] "quick connect" feature).
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct BearerInformation
    {
        /// <summary>
        /// Defines protocol dependent data rates and may be one of the values listed in the following table.  Automatic is recommended.
        /// </summary>
        public DataRate Speed;              /* One of the CELLDEVCONFIG_SPEED_* constants */
        /// <summary>
        /// Defines service type for data access only. These flags only apply during data calls; this flag is ignored when a voice call is underway. May be one of the values listed in the following table. CELLDEVCONFIG_SERVICE_MODEM_ASYNC is recommended. 
        /// </summary>
        public int Service;            /* One of the CELLDEVCONFIG_SERVICE_* constants */
        /// <summary>
        /// Defines bearer service connection elements. May be one of the values listed in the following table. CELLDEVCONFIG_CONNELEM_NONTRANSPARENT is recommended.
        /// </summary>
        public int ConnectionElement;  /* One of the CELLDEVCONFIG_CONNELEM_* constants */
    }

    /// <summary>
    /// This structure contains data compression information that is used for v.42bis modem protocol compression settings.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct DataCompressionInfo
    {
        /// <summary>
        /// Data compression directions.
        /// </summary>
        public int Direction;        /* One of the CELLDEVCONFIG_DATACOMPDIR_* constants */        
        /// <summary>
        /// If TRUE, data compression is required. If FALSE, data compression is not required. False is the default setting.
        /// </summary>
        public int Required;         /* Data compression required */
        /// <summary>
        /// Maximum number of dictionary entries. The range for this is 512 - 65535; 512 is the default setting.
        /// </summary>
        public int MaxDictionaryEntries;   /* Maximum number of dictionary entries */
        /// <summary>
        /// Maximum string length. The range for this is 6 - 250; 6 is the default setting.
        /// </summary>
        public int MaxStringLength;  /* Maximum string length */
    }

    /// <summary>
    /// This structure contains radio link protocol settings. Radio link protocol is a link layer correction protocol 
    /// that increases the perceived reliability of the air link.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct RadioLinkProtocolSettings
    {
        /// <summary>
        /// Version number. The recommended range is 0 - 2; 2 is the default setting.
        /// </summary>
        public int Version;             /* Version number */
        /// <summary>
        /// Inter-working function (IWF) to the device window size. 
        /// The recommended range for versions 0 and 1 is 0 - 61; 61 is the default setting.
        /// </summary>
        public int Iws;                 /* IWF to MS window size  */
        /// <summary>
        /// MS to IWF window size. The recommended range for versions 0 and 1 is 0 - 61; 61 is the default setting. 
        /// The range for version 2 is 0 - 480; 240 is the default setting.
        /// </summary>
        public int Mws;                 /* MS to IWF window size  */
        /// <summary>
        /// Acknowledgement timer [T1] in milliseconds. The recommended range is 380+; 500 is the default setting.
        /// </summary>
        public int AckTimer;            /* Acknowledgement timer [T1] (milliseconds) */
        /// <summary>
        /// Retransmit attempts [N2]. The recommended range is >0; 6 is the default setting.
        /// </summary>
        public int RetransmitAttempts;  /* Retransmit attempts [N2] */
        /// <summary>
        /// Resequence timer [T4] in milliseconds. The recommended range is >25; 
        /// if dwVersion is 1 then is 30 is the default setting for 14.5 kilobit full rate, 
        /// if dwVersion is 2 then 50 is the default setting for 14.5 kilobit full rate.
        /// </summary>
        public int ResequenceTimer;     /* Resequence timer [T4] (milliseconds) */
    }

    /// <summary>
    /// GPRS Connection Information
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct GPRSConnectionInfo
    {
        const int CELLDEVCONFIG_MAXLENGTH_GPRSACCESSPOINTNAME = 64;
        const int CELLDEVCONFIG_MAXLENGTH_GPRSADDRESS = 64;
        const int CELLDEVCONFIG_MAXLENGTH_GPRSPARAMETERS = 32;

        /// <summary>
        /// Identifies the protocol type for the GPRS connection.
        /// </summary>
        public int ProtocolType;                                                    /* One of the CELLDEVCONFIG_GPRSPROTOCOL_* constants */
        /// <summary>
        /// Identifies the level-2 protocol type for the GPRS connection.
        /// </summary>
        public int L2ProtocolType;                                                  /* One of the CELLDEVCONFIG_GPRSL2PROTOCOL_* constants */
        /// <summary>
        /// Specifies the logical name to select the GPRS gateway.
        /// </summary>
        [Array(CELLDEVCONFIG_MAXLENGTH_GPRSACCESSPOINTNAME * 2)]
        public byte[] AccessPointName;   /* Logical name to select the GPRS gateway  */
        /// <summary>
        /// Specifies the packet address to use for the connection. 
        /// </summary>
        [Array(CELLDEVCONFIG_MAXLENGTH_GPRSADDRESS * 2)]
        public byte[] Address;                   /* The packet address to use (if empty, then a dynamic address will be requested) */
        /// <summary>
        /// Identifies the data compression setting for messages that are sent and received.
        /// </summary>
        public int DataCompression;                                                 /* One of the CELLDEVCONFIG_GPRSCOMPRESSION_* constants */
        /// <summary>
        /// Identifies the data compression setting for messages that are sent and received.
        /// </summary>
        public int HeaderCompression;                                               /* One of the CELLDEVCONFIG_GPRSCOMPRESSION_* constants */
        /// <summary>
        /// Identifies protocol-specific parameters. Must be null terminated.
        /// </summary>
        [Array(CELLDEVCONFIG_MAXLENGTH_GPRSPARAMETERS * 2)]
        public byte[] Parameters;              /* Prococol-specific parameters (NULL terminated) */
        /// <summary>
        /// TRUE if and only if RequestedQOSSettings is valid.
        /// </summary>
        public int RequestedQOSSettingsValid;                                        /* TRUE iff sgqsRequestedQOSSettings is valid */
        /// <summary>
        /// GPRS Quality of Service (QOS) settings structure. 
        /// </summary>
        public QualityOfService RequestedQOSSettings;                                          /* GPRS QOS settings structure */
        /// <summary>
        /// TRUE if and only if MinimumQOSSettings is valid.
        /// </summary>
        public int MinimumQOSSettingsValid;                                          /* TRUE iff sgqsMinimumQOSSettings is valid */
        /// <summary>
        /// GPRS QOS settings structure. 
        /// </summary>
        public QualityOfService MinimumQOSSettings;                                            /* GPRS QOS settings structure */
    }

    /// <summary>
    /// This structure contains General Packet Radio Service (GPRS) Quality Of Service (QOS) 
    /// settings that are used to specify minimum and requested QOS settings. 
    /// The QOS defines the quality of service expected for a GPRS packet.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct QualityOfService
    {
        /// <summary>
        /// Identifies the GPRS precedence class, which indicates packet priority. The network uses this setting to prioritize service under abnormal conditions.
        /// </summary>
        public GPRSPrecedence Precedence;                               /* One of the CELLDEVCONFIG_GPRSPRECEDENCECLASS_* constants */
        /// <summary>
        /// Identifies the delay class. The delay class identifies the maximum delay intervals for a packet during transmission on an internal network. This information is required for real-time traffic.
        /// </summary>
        public GPRSDelay Delay;                                    /* One of the CELLDEVCONFIG_GPRSDELAYCLASS_* constants */
        /// <summary>
        /// Identifies the reliability class of a packet.
        /// </summary>
        public GPRSReliability Reliability;                              /* One of the CELLDEVCONFIG_GPRSRELIABILITYCLASS_* constants */
        /// <summary>
        /// Identifies the peak throughput class, which specifies the expected data transfer rate across the network in bits per second. This setting is independent of the delay class.
        /// </summary>
        public PeakThroughput PeakThroughput;                                 /* One of the CELLDEVCONFIG_GPRSPEAKTHRUCLASS_* constants */
        /// <summary>
        /// Identifies the mean throughput class, which specifies the average data transfer rate across the GPRS network in octets per hour.
        /// </summary>
        public MeanThroughput MeanThroughput;                                 /* One of the CELLDEVCONFIG_GPRSMEANTHRUCLASS_* constants */
    }
    
    /// <summary>
    /// Line barring modes
    /// </summary>
    [Flags]
    public enum LineBarMode
    {
        /// <summary>
        /// Bar all outgoing calls.
        /// </summary>
        AllOutgoingCalls = 0x00000001,
        /// <summary>
        /// Bar all international outgoing calls.
        /// </summary>
        AllInternationalOutgoingCalls = 0x00000002,
        /// <summary>
        /// Bar all international outgoing calls except to home.
        /// </summary>
        AllInternationalOutgoingCallsExceptToHome = 0x00000004,
        /// <summary>
        /// Bar all incoming calls.
        /// </summary>
        AllIncomingCalls = 0x00000008,
        /// <summary>
        /// Bar incoming calls when roaming.
        /// </summary>
        AllIncomingCallsWhenRoaming = 0x00000010,
        /// <summary>
        /// Bar incoming calls not in Subscriber Identity Module (SIM) phone book.
        /// </summary>
        AllIncomingCallsNotInSimPhoneBook = 0x00000020,
        /// <summary>
        /// All barring services.
        /// </summary>
        All = 0x00000040,
        /// <summary>
        /// All outgoing barring services.
        /// </summary>
        AllOutgoingServices = 0x00000080,
        /// <summary>
        /// All incoming barring services.
        /// </summary>
        AllIncomingServices = 0x00000100,
    }

    /// <summary>
    /// Line GPRS class types
    /// </summary>
    public enum LineGprsClass
    {
        /// <summary>
        /// Simultaneous voice and GPRS data.
        /// </summary>
        GsmAndGprs = 0x00000001,
        /// <summary>
        /// Simultaneous voice and GPRS traffic channel, one or other data.
        /// </summary>
        GsmOrGprs = 0x00000002,
        /// <summary>
        /// Either all voice or all GPRS, both traffic channels unmonitored.
        /// </summary>
        GsmOrGprsExclusive = 0x00000003,
        /// <summary>
        /// Only GPRS.
        /// </summary>
        GprsOnly = 0x00000004,
        /// <summary>
        /// Only circuit switched voice and data.
        /// </summary>
        GsmOnly = 0x00000005,
    }

    /// <summary>
    /// Line GPRS class changed types
    /// </summary>
    [Flags]
    public enum LineGprsClassChanged
    {
        /// <summary>
        /// Network indicated a change.
        /// </summary>
        Network = 0x00000001,
        /// <summary>
        /// Radio indicated a change.
        /// </summary>
        Radio = 0x00000002,
    }

    /// <summary>
    /// Line HSCSD air-rates
    /// </summary>
    public enum LineHscsdAirRate
    {
        /// <summary>
        /// 9,600 baud.
        /// </summary>
        Rate9600 = 0x00000001,
        /// <summary>
        /// 14,400 baud
        /// </summary>
        Rate14400 = 0x00000002,
        /// <summary>
        /// 19,200 baud.
        /// </summary>
        Rate19200 = 0x00000003,
        /// <summary>
        /// 28,800 baud.
        /// </summary>
        Rate28800 = 0x00000004,
        /// <summary>
        /// 38,400 baud.
        /// </summary>
        Rate38400 = 0x00000005,
        /// <summary>
        /// 43,200 baud.
        /// </summary>
        Rate43200 = 0x00000006,
        /// <summary>
        /// 57,600 baud.
        /// </summary>
        Rate57600 = 0x00000007,
    }
    
    /// <summary>
    /// Line HSCSD codings
    /// </summary>
    public enum LineHscsdCoding
    {
        /// <summary>
        /// 4,800 baud.
        /// </summary>
        Coding4800 = 0x00000001,
        /// <summary>
        /// 9,600 baud.
        /// </summary>
        Coding9600 = 0x00000002,
        /// <summary>
        /// 14,400 baud.
        /// </summary>
        Coding14400 = 0x00000004,
    }

    /// <summary>
    /// Line operator statuses
    /// </summary>
    public enum LineOpStatus
    {
        /// <summary>
        /// Operator status is unknown.
        /// </summary>
        Unknown = 0x00000000,
        /// <summary>
        /// Operator is available.
        /// </summary>
        Available = 0x00000001,
        /// <summary>
        /// Current operator being used.
        /// </summary>
        Current = 0x00000002,
        /// <summary>
        /// Operator is forbidden.
        /// </summary>
        Forbidden = 0x00000003,
    }

    /// <summary>
    /// Line register modes
    /// </summary>
    public enum LineRegMode
    {
        /// <summary>
        /// Automatically register with available operator.
        /// </summary>
        Automatic = 0x00000001,
        /// <summary>
        /// Explicitly register with a particular operator.
        /// </summary>
        Manual = 0x00000002,
        /// <summary>
        /// If manual registration fails, register automatically.
        /// </summary>
        Either = 0x00000003,
    }

    /// <summary>
    /// This enum describes the CDMA line system types and their values.
    /// </summary>
    [Flags]
    public enum LineSystemType
    {
        // Line system types, CDMA
        /// <summary>
        /// 
        /// </summary>
        None = 0x00000000,
        /// <summary>
        /// 
        /// </summary>
        IS95A = 0x00000001,
        /// <summary>
        /// 
        /// </summary>
        IS95B = 0x00000002,
        /// <summary>
        /// 
        /// </summary>
        OneXrttPacket = 0x00000004,
        // Line system types, Gsm
        /// <summary>
        /// 
        /// </summary>
        Gsm = 0x00000008,
        /// <summary>
        /// 
        /// </summary>
        Gprs = 0x00000010,
    }

    /// <summary>
    /// Line USSD flags
    /// </summary>
    [Flags]
    public enum LineUssdFlags
    {
        /// <summary>
        /// Network needs more USSD information sent back.
        /// </summary>
        ActionRequired = 0x00000001,
        /// <summary>
        /// Network does not need more USSD info.
        /// </summary>
        ActionNotNeeded = 0x00000002,
        /// <summary>
        /// Network has terminated USSD session.
        /// </summary>
        Terminated = 0x00000004,
        /// <summary>
        /// Network has terminated the USSD session because another client responded.
        /// </summary>
        OtherClientResponded = 0x00000008,
        /// <summary>
        /// Network does not support USSD.
        /// </summary>
        Unsupported = 0x00000010,
        /// <summary>
        /// USSD session timed out.
        /// </summary>
        Timeout = 0x00000020,
        /// <summary>
        /// Network is terminating USSD session.
        /// </summary>
        EndSession = 0x00000040,
    }

    /// <summary>
    /// Special LineOperator index values
    /// </summary>
    public enum LineOperatorOptions
    {
        /// <summary>
        /// 
        /// </summary>
        UseFirstAvailableIndex = (-1),
    }


    /// <summary>
    /// 
    /// </summary>
    public enum LineDeviceSpecific
    {
        // LINE_DEVSPECIFIC message types
        /// <summary>
        /// dwParam1 = EquipmentStateChange
        /// dwParam2 = One of the * constants
        /// dwParam3 = One of the * constants
        /// </summary>
        EquipmentStateChange = 0x00000100,

        /// <summary>
        /// dwParam1 = GprsClass
        /// dwParam2 = One of the * constants
        /// dwParam3 = One of the * constants
        /// </summary>
        GprsClass = 0x00000101,

        /// <summary>
        /// dwParam1 = GprsRegisterState
        /// dwParam2 = One of the * constants
        /// dwParam3 Unused
        /// </summary>
        GprsRegisterState = 0x00000102,

        /// <summary>
        /// dwParam1 = RadioPresence
        /// dwParam2 = One of the * constants
        /// dwParam3 Unused
        /// </summary>
        RadioPresence = 0x00000103,

        /// <summary>
        /// dwParam1 = RegisterState
        /// dwParam2 = One of the * constants
        /// dwParam3 Unused
        /// </summary>
        RegisterState = 0x00000104,

        /// <summary>
        /// dwParam1 = Ussd
        /// dwParam2 = Message identifier
        /// dwParam3 = Size in bytes of message
        /// </summary>
        Ussd = 0x00000105,

        /// <summary>
        /// dwParam1 = CurrentLineChange
        /// dwParam2 = New line identifier
        /// dwParam3 = New address ID
        /// </summary>
        CurrentLineChange = 0x00000106,

        /// <summary>
        /// dwParam1 = CurrentSystemChange
        /// dwParam2 = New system coverage (*)
        /// dwParam3 = unused
        /// </summary>
        CurrentSystemChange = 0x00000107,
    }

    /// <summary>
    /// 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class LineOperator
    {
        /// <summary>
        /// 
        /// </summary>
        public LineOperator()
        {
            LongNameBytes = new byte[NativeMethods.MAX_LENGTH_OPERATOR_LONG * 2];
            ShortNameBytes = new byte[NativeMethods.MAX_LENGTH_OPERATOR_SHORT * 2];
            NumericNameBytes = new byte[NativeMethods.MAX_LENGTH_OPERATOR_NUMERIC * 2];
        }
        /// <summary>
        /// 
        /// </summary>
        public int Index;
        /// <summary>
        /// 
        /// </summary>
        public int ValidFields;
        /// <summary>
        /// New in v1.1. Indicates which fields are valid. Possible values are from <see cref="OperatorFormat"/>. 
        /// </summary>
        public OperatorFormat LineOperatorFormatValidFields
        {
            get
            {
                return (OperatorFormat)ValidFields;
            }
        }
        /// <summary>
        /// 
        /// </summary>
        public int Status;
        /// <summary>
        /// New in v1.1. Indicates status of the given operator. Possible values are <see cref="LineOperatorStatus"/>
        /// </summary>
        public LineOpStatus LineOperatorStatus
        {
            get
            {
                return (LineOpStatus)Status;
            }
        }
        /// <summary>
        /// 
        /// </summary>
        [Array(NativeMethods.MAX_LENGTH_OPERATOR_LONG * 2)]
        public byte[] LongNameBytes;
        /// <summary>
        /// 
        /// </summary>
        [Array(NativeMethods.MAX_LENGTH_OPERATOR_SHORT * 2)]
        public byte[] ShortNameBytes;
        /// <summary>
        /// 
        /// </summary>
        [Array(NativeMethods.MAX_LENGTH_OPERATOR_NUMERIC * 2)]
        public byte[] NumericNameBytes;
        /// <summary>
        /// 
        /// </summary>
        public int SizeOf { get { return 3 * Marshal.SizeOf(typeof(int)) + (NativeMethods.MAX_LENGTH_OPERATOR_LONG + NativeMethods.MAX_LENGTH_OPERATOR_SHORT + NativeMethods.MAX_LENGTH_OPERATOR_NUMERIC) * 2; } }
        /// <summary>
        /// Long text description.
        /// </summary>
        public string LongName { get { return Encoding.Unicode.GetString(LongNameBytes, 0, LongNameBytes.Length).TrimEnd('\0'); } }
        /// <summary>
        /// Short text description.
        /// </summary>
        public string ShortName { get { return Encoding.Unicode.GetString(ShortNameBytes, 0, ShortNameBytes.Length).TrimEnd('\0'); } }
        /// <summary>
        /// 	Numeric description.
        /// </summary>
        public string NumericName { get { return Encoding.Unicode.GetString(NumericNameBytes, 0, NumericNameBytes.Length).TrimEnd('\0'); } }

        /// <summary>
        /// New in v1.1. Converts a byte array to a LineOperator structure.
        /// </summary>
        /// <param name="data">The byte array data to use.</param>
        /// <returns>A valid LineOperator structure</returns>
        public static implicit operator LineOperator(byte[] data)
        {
            LineOperator lineOperator = new LineOperator();
            lineOperator.Index = BitConverter.ToInt32(data, 0);
            lineOperator.ValidFields = BitConverter.ToInt32(data, 4);
            lineOperator.Status = BitConverter.ToInt32(data, 4 + 4);
            Array.Copy(data,12,lineOperator.LongNameBytes,0,32);
            Array.Copy(data,12 + (32 * 2),lineOperator.ShortNameBytes,0,16);
            Array.Copy(data, 12 + (32 * 2) + (16*2), lineOperator.NumericNameBytes, 0, 16);
            return lineOperator;
        }
    }

    /// <summary>
    /// The LineOperatorStatus structure holds the set of preferred and available operators. 
    /// </summary>
    public class LineOperatorStatus : PhoneBase
    {
        /// <summary>
        /// Creates an instance of LineOperatorStatus
        /// </summary>
        public LineOperatorStatus(int size)
            : base(size)
        {
            TotalSize = size;
        }
        /// <summary>
        /// The total size in bytes allocated to this data structure.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// The size, in bytes, of this data structure, which is required to hold all of the returned information. 
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// The size, in bytes, of the portion of this data structure that contains useful information.
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Number of entries in the preferred operator list.
        /// </summary>
        public int PreferredCount;
        /// <summary>
        /// The size, in bytes, of the variably sized field that describes the preferred operators.
        /// </summary>
        public int PreferredSize;
        /// <summary>
        /// The offset, in bytes, of the variably sized field that describes the preferred operators from the beginning of this data structure. 
        /// </summary>
        public int PreferredOffset;
        /// <summary>
        /// Number of operators currently available.
        /// </summary>
        public int AvailableCount;
        /// <summary>
        /// The size, in bytes, of the variably sized field that describes the available operators.
        /// </summary>
        public int AvailableSize;
        /// <summary>
        /// The offset, in bytes, of the variably sized field that describes the available operators from the beginning of this data structure. 
        /// </summary>
        public int AvailableOffset;
    }

    /// <summary>
    /// The LineGeneralInfo structure holds miscellaneous information about the device and its software.
    /// </summary>
    public class LineGeneralInfo : PhoneBase
    {
        /// <summary>
        /// Creates an instance of LineGeneralInfo
        /// </summary>
        public LineGeneralInfo(int size)
            : base(size)
        {
            TotalSize = size;
        }
        /// <summary>
        /// The total size in bytes allocated to this data structure.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// The size in bytes for this data structure that is needed to hold all of the returned information.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// The size in bytes of the portion of this data structure that contains useful information.
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// The size in bytes of the Unicode manufacturer name.
        /// </summary>
        public int ManufacturerSize;
        /// <summary>
        /// The offset from the beginning of the structure of the Unicode manufacturer name. 
        /// </summary>
        public int ManufacturerOffset;
        /// <summary>
        /// The size in bytes of the Unicode model name.
        /// </summary>
        public int ModelSize;
        /// <summary>
        /// The offset from the beginning of the structure of the Unicode model name. This value indicates the model of the underlying radio.
        /// </summary>
        public int ModelOffset;
        /// <summary>
        /// The size in bytes of the Unicode revision name.
        /// </summary>
        public int RevisionSize;
        /// <summary>
        /// The offset from the beginning of the structure of the Unicode revision name. This value indicates the software version of the radio stack.
        /// </summary>
        public int RevisionOffset;
        /// <summary>
        /// The size in bytes of the Unicode serial number.
        /// </summary>
        public int SerialNumberSize;
        /// <summary>
        /// The offset from the beginning of the structure of the Unicode serial number. This value indicates the equipment identity International Mobile Equipment Identity (IMEI) code.
        /// </summary>
        public int SerialNumberOffset;
        /// <summary>
        /// The size in bytes of the Unicode serial number.
        /// </summary>
        public int SubscriberNumberSize;
        /// <summary>
        /// The offset from the beginning of the structure of the Unicode serial number. This value indicates the Unicode subscriber number.
        /// </summary>
        public int SubscriberNumberOffset;

        /// <summary>
        /// Manufacturer's Name
        /// </summary>
        public string Manufacturer { get { return ManufacturerSize == 0 ? "" : Encoding.Unicode.GetString(Data, ManufacturerOffset, ManufacturerSize).TrimEnd('\0'); } }
        /// <summary>
        /// Model name
        /// </summary>
        public string Model { get { return ModelSize == 0 ? "" : Encoding.Unicode.GetString(Data, ModelOffset, ModelSize).TrimEnd('\0') ; } }
        /// <summary>
        /// Model Revision 
        /// </summary>
        public string Revision { get { return RevisionSize == 0 ? "" : Encoding.Unicode.GetString(Data, RevisionOffset, RevisionSize).TrimEnd('\0'); } }
        /// <summary>
        /// Serial Number
        /// </summary>
        public string SerialNumber { get { return SerialNumberSize == 0 ? "" : Encoding.Unicode.GetString(Data, SerialNumberOffset, SerialNumberSize).TrimEnd('\0'); } }
        /// <summary>
        /// IMEI
        /// </summary>
        public string SubscriberNumber { get { return SubscriberNumberSize == 0 ? "" : Encoding.Unicode.GetString(Data, SubscriberNumberOffset, SubscriberNumberSize).TrimEnd('\0'); } }

        ///// <summary>
        ///// New in v1.1. Converts a byte array to a LineOperator structure.
        ///// </summary>
        ///// <param name="data">The byte array data to use.</param>
        ///// <returns>A valid LineOperator structure</returns>
        //public static implicit operator LineGeneralInfo(byte[] data)
        //{
        //    LineGeneralInfo item = new LineGeneralInfo(0);
        //    item.TotalSize = BitConverter.ToInt32(data, 0);
        //    item.NeededSize = BitConverter.ToInt32(data, 4);
        //    item.UsedSize = BitConverter.ToInt32(data, 4 *2);
        //    item.ManufacturerSize= BitConverter.ToInt32(data, 4 *3);
        //    item.ManufacturerOffset= BitConverter.ToInt32(data, 4 *4);
        //    item.ModelSize= BitConverter.ToInt32(data, 4 *5);
        //    item.ModelOffset= BitConverter.ToInt32(data, 4 *6);
        //    item.RevisionSize= BitConverter.ToInt32(data, 4 *7);
        //    item.RevisionOffset= BitConverter.ToInt32(data, 4 *8);
        //    item.SerialNumberSize= BitConverter.ToInt32(data, 4 *9);
        //    item.SerialNumberOffset= BitConverter.ToInt32(data, 4 *10);
        //    item.SubscriberNumberSize= BitConverter.ToInt32(data, 4 *11);
        //    item.SubscriberNumberOffset= BitConverter.ToInt32(data, 4 *12);
        //    return item;
        //}
    }


}