// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Runtime.InteropServices;
using System.IO;
using System.Windows.Forms;

namespace OpenNETCF.Telephony
{
    //TODO LineGetUSSD, lineSendUSSD, lineGetCurrentHSCSDStatus, lineGetHSCSDCaps, lineGetHSCSDState, lineSetHSCSDState, lineGetCurrentAddressID, lineSetCUrrentAddressID
    //TODO lineGetOperatorStatus, lineRegister, lineUnRegister, lineSetPereferredOperator
    //TODO TapiEx specific messages
    /// <summary>
    /// Represents a cellular line available on a Windows Mobile or Windows CE device.  This class is an extension of the <see cref="Line"/> object and adds more
    /// cellular line related functionality for example disabling the radio.
    /// </summary>
    /// <remarks>
    /// To use this class first create a new <see cref="Telephony"/> object.  Once the object is created use the <see cref="Telephony.CellularLine"/> method to retreive the current
    /// cellular line associated with the device.  If the device is not capable of being a cellular device a <see cref="NotSupportedException"/> will be thrown.
    /// 
    /// <para>To use this class within a <see cref="System.Windows.Forms.Form"/> use the <see cref="CellularLineMonitor"/> component and use the <see cref="CellularLineMonitor.Line"/>
    /// property to get access to the <see cref="CellularLine"/> object.</para>
    /// 
    /// </remarks>
    public class CellularLine : Line
    {
        internal CellularLine(Telephony telephony, IntPtr hLine)
            : base(telephony, hLine)
        {
            ValidateCellCore();
            telephonyObject.LineMessage += new Telephony.MessageHandler(telephonyObject_LineMessage);
        }

        /// <summary>
        /// Handles specific line messages for ExTAPI
        /// </summary>
        /// <param name="msg"></param>
        private void telephonyObject_LineMessage(LineMessage msg)
        {
            if (msg.MessageID == LineMessages.LINE_DEVSPECIFIC)
            {
                switch ((LineMessages)msg.Param1)
                {
                    case LineMessages.LINE_CURRENTLINECHANGE:
                        OnCurrentLineChange(msg);
                        break;
                    case LineMessages.LINE_CURRENTSYSTEMCHANGE:
                        OnCurrentSystemChange(msg);
                        break;
                    case LineMessages.LINE_EQUIPSTATECHANGE:
                        OnEquipmentStateChange(msg);
                        break;
                    case LineMessages.LINE_GPRSCLASS:
                        OnGprsClassChange(msg);
                        break;
                    case LineMessages.LINE_GPRSREGISTERSTATE:
                        OnGprsRegisterStateChange(msg);
                        break;
                    case LineMessages.LINE_RADIOPRESENCE:
                        OnRadioPresenceChange(msg);
                        break;
                    case LineMessages.LINE_REGISTERSTATE:
                        OnRegisterStateChange(msg);
                        break;
                    //case LineMessages.LINE_USSD:
                    //    break;
                }
            }
        }

        /// <summary>
        /// Represents the method that will handle the <see cref="CellularLine.CurrentLineChange"/> event.
        /// </summary>
        /// <param name="lineID">The new line identifier.</param>
        /// <param name="addressID">New address ID</param>
        public delegate void CurrentLineChangeEventHandler(int lineID, int addressID);
        /// <summary>
        /// Occurs when the current line of the device has changed.
        /// </summary>
        public event CurrentLineChangeEventHandler CurrentLineChange;
        private void OnCurrentLineChange(LineMessage msg)
        {
            if (CurrentLineChange != null)
                CurrentLineChange(msg.Param2.ToInt32(), msg.Param3.ToInt32());
        }

        /// <summary>
        /// Represents the method that will handle the <see cref="CellularLine.CurrentSystemChange"/> event.
        /// </summary>
        /// <param name="lineSystemType">New system coverage. See <see cref="LineSystemType"/></param>
        public delegate void CurrentSystemChangeEventHandler(LineSystemType lineSystemType);
        /// <summary>
        /// Occurs when the current system changes.   See <see cref="CellularLine.CurrentSystemChangeEventHandler"/>.
        /// </summary>
        public event CurrentSystemChangeEventHandler CurrentSystemChange;
        private void OnCurrentSystemChange(LineMessage msg)
        {
            if (CurrentSystemChange != null)
                CurrentSystemChange((LineSystemType)msg.Param2.ToInt32());
        }

        /// <summary>
        /// Represents the method that will handle the <see cref="CellularLine.EquipmentStateChange"/> event.
        /// </summary>
        /// <param name="equipmentState">Indicates the new equipment state. See the <see cref="EquipmentState"/>  for possible values.</param>
        /// <param name="radioState">Indicates whether the major radio functions are on or off. See the <see cref="RadioState"/> constants for possible values.</param>
        public delegate void EquipmentStateChangeEventHandler(EquipmentState equipmentState, RadioState radioState);
        /// <summary>
        /// Occurs when the equipment state has changed.  See <see cref="CellularLine.EquipmentStateChangeEventHandler"/>.
        /// </summary>
        public event EquipmentStateChangeEventHandler EquipmentStateChange;
        private void OnEquipmentStateChange(LineMessage msg)
        {
            if(EquipmentStateChange != null)
                EquipmentStateChange((EquipmentState)msg.Param2.ToInt32(),(RadioState)msg.Param3.ToInt32());
        }

        /// <summary>
        /// Represents the method that will handle the <see cref="CellularLine.GprsClassChangeEventHandler"/> event.  
        /// </summary>
        /// <param name="lineGprsClass">The class of the mobile and network.  See <see cref="LineGprsClass"/> for possible values.</param>
        /// <param name="lineGprsClassChanged">Indicates the change that occured.  See <see cref="LineGprsClassChanged"/> for possible values.</param>
        /// <remarks>Not implemented for CDMA devices.</remarks>
        public delegate void GprsClassChangeEventHandler(LineGprsClass lineGprsClass, LineGprsClassChanged lineGprsClassChanged);
        /// <summary>
        /// Occurs when the <see cref="LineGprsClass"/> has changed.  See <see cref="CellularLine.GprsClassChangeEventHandler"/>.
        /// </summary>
        public event GprsClassChangeEventHandler GprsClassChange;
        private void OnGprsClassChange(LineMessage msg)
        {
            if (GprsClassChange != null)
                GprsClassChange((LineGprsClass)msg.Param2.ToInt32(), (LineGprsClassChanged)msg.Param3.ToInt32());
        }
        /// <summary>
        /// Represents the method that will handle the <see cref="CellularLine.GprsRegisterStateChangeEventHandler"/> event.  
        /// </summary>
        /// <param name="registrationStatus">The registration state. See the <see cref="RegistrationStatus"/> for possible values.</param>
        public delegate void GprsRegisterStateChangeEventHandler(RegistrationStatus registrationStatus);
        /// <summary>
        /// Occurs when the <see cref="RegistrationStatus"/> has changed.  See <see cref="CellularLine.GprsRegisterStateChangeEventHandler"/>.
        /// </summary>
        public event GprsRegisterStateChangeEventHandler GprsRegisterStateChange;
        private void OnGprsRegisterStateChange(LineMessage msg)
        {
            if(GprsRegisterStateChange!=null)
                GprsRegisterStateChange((RegistrationStatus)msg.Param2.ToInt32()); 
        }

        /// <summary>
        /// Represents the method that will handle the <see cref="CellularLine.RadioPresenceChange"/> event.
        /// </summary>
        /// <param name="radioPresence">The new equipment state. See the <see cref="RadioPresence"/> for possible values.</param>
        public delegate void RadioPresenceChangeEventHandler(RadioPresence radioPresence);
        /// <summary>
        ///  Occurs when the <see cref="RadioPresence"/> has changed.  See <see cref="CellularLine.RadioPresenceChangeEventHandler"/>.
        /// </summary>
        public event RadioPresenceChangeEventHandler RadioPresenceChange;
        private void OnRadioPresenceChange(LineMessage msg)
        {
            if (RadioPresenceChange != null)
                RadioPresenceChange((RadioPresence)msg.Param2.ToInt32());
        }

        /// <summary>
        /// Represents the method that will handle the <see cref="CellularLine.RegisterStateChange"/> event.  
        /// </summary>
        /// <param name="registrationStatus">The registration state. See the <see cref="RegistrationStatus"/> for possible values.</param>
        public delegate void RegisterStateChangeEventHandler(RegistrationStatus registrationStatus);
        /// <summary>
        /// Occurs when the <see cref="RegistrationStatus"/> has changed.  See <see cref="CellularLine.RegisterStateChangeEventHandler"/>.
        /// </summary>
        public event RegisterStateChangeEventHandler RegisterStateChange;
        private void OnRegisterStateChange(LineMessage msg)
        {
            if (RegisterStateChange != null)
                RegisterStateChange((RegistrationStatus)msg.Param2.ToInt32());
        }


        /// <summary>
        /// Dispose of any unmanaged objects.
        /// </summary>
        public override void Dispose()
        {
            base.Dispose();
        }

        /// <summary>
        /// Retrieve the <see cref="E:LineBarringModes">barring modes</see> and <see cref="LineCapabilityClass">line capability classes</see> supported by the radio hardware.
        /// </summary>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The lineBarMode will contain all supported barring modes.  The lineCapabilityClass parameter will contain all supported line capability types.</remarks>
        public void CallBarringCapabilities(out LineBarMode lineBarMode, out LineCapabilityClass lineCapabilityClass)
        {
            int errorCode = NativeMethods.lineGetCallBarringCaps(hLine, out lineBarMode, out lineCapabilityClass);
            if (errorCode != 0)
                throw new TelephonyException("Error getting Barring Capabilities.", errorCode);
        }

        /// <summary>
        /// Retreives the current barring state for a given <see cref="LineBarMode"/>.
        /// </summary>
        /// <param name="mode">Indicates which barring mode the application needs status on.</param>
        /// <returns>a <see cref="LineCapabilityClass"/> indicating which modes are currently barred.  <see cref="LineCapabilityClass"/> may contain bit masked values.</returns>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        public LineCapabilityClass GetCallBarringState(LineBarMode mode)
        {
            return this.GetCallBarringState(mode, "");
        }

        /// <summary>
        /// Retreives the current barring state for a given <see cref="LineBarMode"/>.
        /// </summary>
        /// <param name="mode">Indicates which barring mode the application needs status on.</param>
        /// <param name="password">A password may be necessary to read the current barring status.</param>
        /// <returns>a <see cref="LineCapabilityClass"/> indicating which modes are currently barred.  <see cref="LineCapabilityClass"/> may contain bit masked values.</returns>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public LineCapabilityClass GetCallBarringState(LineBarMode mode, string password)
        {
            ValidateGSM();
            LineCapabilityClass lpdwClasses;
            int errorCode = NativeMethods.lineGetCallBarringState(hLine, mode, out lpdwClasses, (password.Length == 0 ? null : password));
            if (errorCode != 0)
                throw new TelephonyException("Error getting Call Barring state for " + mode.ToString() + ".", errorCode);
            return lpdwClasses;
        }

        /// <summary>
        /// Sets the barring state of the <see cref="LineBarMode"/> to on or off.
        /// </summary>
        /// <param name="mode">Indicates which <see cref="LineBarMode">barring mode</see> the application needs to set.</param>
        /// <param name="password">A password may be necessary to set the current barring status. See <see cref="LineBarMode"/> for possible values.</param>
        /// <param name="lineCapabilityClass">Indicates which line capability classes to bar. See <see cref="LineCapabilityClass"/> for possible values. </param>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public void SetCallBarringState(LineBarMode mode, LineCapabilityClass lineCapabilityClass, string password)
        {
           
            int errorCode = NativeMethods.lineSetCallBarringState(hLine, mode, lineCapabilityClass, password);
            if (errorCode != 0)
                throw new TelephonyException("Error setting Call Barring state for " + mode.ToString() + ".", errorCode);
        }

        /// <summary>
        /// Changes the call-barring password for the <see cref="Line"/>.
        /// </summary>
        /// <param name="mode">Indicates the barring mode for which the application needs to change the password.</param>
        /// <param name="oldPassword">Points to the old password, which may be necessary to set a new password.</param>
        /// <param name="newPassword">Points to the new password.</param>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public void SetCallBarringPassword(LineBarMode mode, string oldPassword, string newPassword)
        {
            CellCoreAsyncResult result = (CellCoreAsyncResult)BeginCallBarringPassword(mode,oldPassword,newPassword,null,null);
            WaitAsyncResult(result);
            EndCallBarringPassword(result);
        }

        /// <summary>
        /// Begins an asynchronous request to set call barring password for a given <see cref="Line"/>.
        /// </summary>
        /// <param name="mode">Indicates the barring mode for which the application needs to change the password.</param>
        /// <param name="oldPassword">The old password, which may be necessary to set a new password.</param>
        /// <param name="newPassword">The new password.</param>
        /// <param name="Callback">Asynchronous callback delegate.</param>
        /// <param name="State">State of the object for the request.</param>
        /// <returns>An <see cref="IAsyncResult"/> that references the asynchronous request for a response.</returns>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The BeginCallBarringPassword method starts an asynchronous request to set the call barring password.
        /// 
        /// The asynchronous callback method should use the <see cref="EndCallBarringPassword"/> to validate the change. 
        /// 
        /// <para>
        /// If an exception is encountered a <see cref="TelephonyException"/> will be thrown.
        /// </para>
        /// </remarks>
        public IAsyncResult BeginCallBarringPassword(LineBarMode mode, string oldPassword, string newPassword, AsyncCallback Callback, object State)
        {
            ValidateGSM();
            CellCoreAsyncResult result = new CellCoreAsyncResult(Callback, State);

            lock (Telephony.PendingRequests.SyncRoot)
            {
                result.ReplyID = NativeMethods.lineSetCallBarringPassword(hLine, mode, oldPassword, newPassword);
                Telephony.PendingRequests.Add(result.ReplyID, result);
            }

            return result;
        }

        /// <summary>
        /// Ends an asynchronous request to set the call barring password.
        /// </summary>
        /// <param name="ar">The pending request for a response. </param>
        /// <exception cref="C:TelphonyException">If the asynchronous call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The EndCallBarringPassword method completes an asynchronous request to set call waiting state that was started by calling BeginCallBarringPassword.  
        /// A <see cref="TelephonyException"/> will be thrown if there is an exception in setting the password.  If everything is successfull no exceptions should be thrown.</remarks>
        public void EndCallBarringPassword(IAsyncResult ar)
        {
            CellCoreAsyncResult result = (CellCoreAsyncResult)ar;
            if (result == null)
                throw new ArgumentException("Invalid parameter", "ar");
            if (!result.IsCompleted)
                result.AsyncWaitHandle.WaitOne();
            int response = ((TapiAsyncResult)ar).Response;

            if (response != 0)
                throw new TelephonyException("Error setting call barring password.", response);

        }

        /// <summary>
        /// Retrieve the set of media modes for which call waiting is supported by the radio hardware.
        /// </summary>
        /// <param name="lineCapabilitiesClass"><see cref="LineCapabilityClass"/> that support call wating.</param>
        public void CallWaitingCapabilities(out LineCapabilityClass lineCapabilitiesClass)
        {
            int errorCode = NativeMethods.lineGetCallWaitingCaps(hLine, out lineCapabilitiesClass);
            if (errorCode != 0)
                throw new TelephonyException("Error getting call waiting capabilities.", errorCode);
        }

        /// <summary>
        /// Gets a <see cref="LineCapabilityClass"/> indicating which <see cref="LineCapabilityClass"/> currently have call waiting enabled.
        /// </summary>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        /// <remarks>The returned <see cref="LineCapabilityClass"/> may contain multiple values as it is bit masked.</remarks>
        public LineCapabilityClass CallWaitingState
        {
            get
            {
                ValidateGSM();
                LineCapabilityClass lineCapabilitiesClass;
                int errorCode = NativeMethods.lineGetCallWaitingState(hLine, out lineCapabilitiesClass);
                if (errorCode != 0)
                    throw new TelephonyException("Error getting call waiting state.", errorCode);
                return lineCapabilitiesClass;
            }
        }

        /// <summary>
        /// Sets <see cref="CallWaitingState">call waiting state</see> for a given set of <see cref="LineCapabilityClass">line capability classes</see>.
        /// </summary>
        /// <param name="lineCapabilitiesClass">Indicates which <see cref="LineCapabilityClass">line capability classes</see> to enable or disable.</param>
        /// <param name="enabled">Indicates if the <see cref="CallWaitingState">call waiting state</see> should be enabled or not.</param>
        /// <returns>The state of the call waiting call represented by <see cref="CallWaitingState"/> value.</returns>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        public CallWaitingState SetCallWaitingState(LineCapabilityClass lineCapabilitiesClass, bool enabled)
        {
            SetCallWaitingStateAsyncResult result = (SetCallWaitingStateAsyncResult)BeginSetCallWaitingState(lineCapabilitiesClass, enabled, null, null);
            WaitAsyncResult(result);
            return EndSetCallWaitingState(result);
        }

        /// <summary>
        /// Begins an asynchronous request to set <see cref="CallWaitingState">call waiting state</see> for a given set of <see cref="LineCapabilityClass">line capability classes</see>.
        /// </summary>
        /// <param name="lineCapabilitiesClass">Indicates which <see cref="LineCapabilityClass">line capability classes</see> to enable or disable.</param>
        /// <param name="enabled">Indicates if the <see cref="CallWaitingState">call waiting state</see> should be enabled or not.</param>
        /// <param name="Callback">Asynchronous callback delegate.</param>
        /// <param name="State">State of the object for the request.</param>
        /// <returns>An <see cref="IAsyncResult"/> that references the asynchronous request for a response.</returns>
        /// <remarks>The BeginSetCallWaitingState method starts an asynchronous request to set the call waiting state.
        /// 
        /// The asynchronous callback method should use the <see cref="EndSetCallWaitingState"/> to validate the change. 
        /// 
        /// <para>
        /// If an exception is encountered a <see cref="TelephonyException"/> will be thrown.
        /// </para>
        /// </remarks>
        public IAsyncResult BeginSetCallWaitingState(LineCapabilityClass lineCapabilitiesClass, bool enabled, AsyncCallback Callback, object State)
        {
            SetCallWaitingStateAsyncResult result = new SetCallWaitingStateAsyncResult(Callback, State, enabled);

            lock (Telephony.PendingRequests.SyncRoot)
            {
                result.ReplyID = NativeMethods.lineSetCallWaitingState(hLine, lineCapabilitiesClass, enabled ? OpenNETCF.Telephony.CallWaitingState.Enabled : OpenNETCF.Telephony.CallWaitingState.Disabled);
                Telephony.PendingRequests.Add(result.ReplyID, result);
            }

            return result;
        }

        /// <summary>
        /// Ends an asynchronous request to set the call waiting state.
        /// </summary>
        /// <param name="ar">The pending request for a response. </param>
        /// <returns>The state of the call waiting call represented by <see cref="CallWaitingState"/> value.</returns>
        /// <exception cref="C:TelphonyException">If the asynchronous call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The EndSetCallWaitingState method completes an asynchronous request to set call waiting state that was started by calling BeginSetCallWaitingState</remarks>
        public CallWaitingState EndSetCallWaitingState(IAsyncResult ar)
        {
            SetCallWaitingStateAsyncResult result = (SetCallWaitingStateAsyncResult)ar;
            if (result == null)
                throw new ArgumentException("Invalid parameter", "ar");
            if (!result.IsCompleted)
                result.AsyncWaitHandle.WaitOne();
            int response = ((TapiAsyncResult)ar).Response;

            if (response != 0)
                throw new TelephonyException("Error setting call waiting state " + (result.CallWaitingOn ? "on." : "off."), response);

            return result.CallWaitingOn ? OpenNETCF.Telephony.CallWaitingState.Enabled : OpenNETCF.Telephony.CallWaitingState.Disabled;
        }

        /// <summary>
        /// Gets the <see cref="LineOperator">network operator</see> on which the device is currently registered. 
        /// </summary>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public LineOperator CurrentOperator
        {
            get
            {
                ValidateGSM();
                byte[] b = new byte[128];
                int errorCode = NativeMethods.lineGetCurrentOperator(hLine, b);
                if (errorCode != 0)
                    throw new TelephonyException("Error getting current line operator.", errorCode);
                return (LineOperator)b;
            }
        }

        /// <summary>
        /// Retrieve the lists of preferred and available operators. 
        /// The preferred operator list is stored locally on the device and automatic registration iterates 
        /// through this list when attempting to register. The available list shows which network operators are 
        /// currently in range of the device. To obtain the current operator, use <see cref="CurrentOperator"/>.
        /// </summary>
        /// <exception cref="NotSupportedException">NotSupportedException will be thrown if called from a non GSM phone device.</exception>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public LineOperatorStatus LineOperatorStatus
        {
            get
            {//TODO need to fix this as it just returns a bunch of offsets and sizes
                ValidateGSM();
                LineOperatorStatus los = new LineOperatorStatus(Marshal.SizeOf(typeof(LineOperatorStatus)));
                los.Store();
                int ret = NativeMethods.lineGetOperatorStatus(hLine, los.Data);
                if (ret != 0)
                    throw new TelephonyException("Error getting line opertator status.", ret);
                los.Load();
                los = new LineOperatorStatus(los.NeededSize);
                los.Store();
                ret = NativeMethods.lineGetGeneralInfo(hLine, los.Data);
                if (ret != 0)
                    throw new TelephonyException("Error getting line opertator status.", ret);
                los.Load();
                return los;
            }
        }

        /// <summary>
        /// Gets the current <see cref="LineSystemType">line system type</see>.
        /// </summary>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public LineSystemType LineSystemType
        {
            get
            {
                LineSystemType l;
                int errorCode = NativeMethods.lineGetCurrentSystemType(hLine, out l);
                if (errorCode != 0)
                    throw new TelephonyException("Error getting line system type.", errorCode);
                return l;
            }
        }

        /// <summary>
        /// Retrieve the state of the radio transmitter and receiver.
        /// </summary>
        /// <param name="equipmentState"></param>
        /// <param name="radioState"></param>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public void GetEquipmentState(out EquipmentState equipmentState, out RadioState radioState)
        {
            int errorCode = NativeMethods.lineGetEquipmentState(hLine, out equipmentState, out radioState);
            if (errorCode != 0)
                throw new TelephonyException("Error getting equipment state.", errorCode);
        }

        /// <summary>
        /// Sets the state of the radio transmitter and receiver.
        /// </summary>
        /// <param name="equipmentState">Indicates the state to set the line to.</param>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The SetEquipmentState starts a synchronous request to set the radio transmitter and receiver state.
        /// To set the radio state asynchronously use the <see cref="CellularLine.BeginSetEquipmentState"/> and <see cref="CellularLine.EndSetEquipmentState"/>.</remarks>
        public void SetEquipmentState(EquipmentState equipmentState)
        {
            SetEquipmentStateAsyncResult result =(SetEquipmentStateAsyncResult)BeginSetEquipmentState(equipmentState, null, null);
            WaitAsyncResult(result);
            EquipmentState es = EndSetEquipmentState(result);
        }

        /// <summary>
        /// Begins an asynchronous request to set the state of the radio transmitter and receiver using the <see cref="EquipmentState"/> parameter.
        /// </summary>
        /// <param name="equipmentState">The state to set the equipment to.</param>
        /// <param name="Callback">Asynchronous callback delegate.</param>
        /// <param name="State">State of the object for the request.</param>
        /// <returns>An <see cref="IAsyncResult"/> that references the asynchronous request for a response.</returns>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The BeginSetEquipmentState method starts an asynchronous request to set the state of the radio transmitter and receiver.
        /// 
        /// The asynchronous callback method should use the <see cref="EndSetEquipmentState"/> to validate the change. 
        /// 
        /// <para>
        /// If an exception is encountered a <see cref="TelephonyException"/> will be thrown.
        /// </para>
        /// </remarks>
        public IAsyncResult BeginSetEquipmentState(EquipmentState equipmentState, AsyncCallback Callback, object State)
        {
            SetEquipmentStateAsyncResult result = new SetEquipmentStateAsyncResult( Callback, State, equipmentState);

            lock (Telephony.PendingRequests.SyncRoot)
            {
                result.ReplyID = NativeMethods.lineSetEquipmentState(hLine, equipmentState);
                Telephony.PendingRequests.Add(result.ReplyID, result);
            }

            return result;
        }

        /// <summary>
        /// Ends an asynchronous request to set the equipment state.
        /// </summary>
        /// <param name="ar">The pending request for a response. </param>
        /// <returns>An <see cref="EquipmentState"/> value of the current state for the radio.</returns>
        /// <exception cref="C:TelphonyException">If the asynchronous call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The EndSetEquipmentState method completes an asynchronous request to set the radio transmitter and receiver state that 
        /// was started by calling BeginSetEquipmentState</remarks>
        public EquipmentState EndSetEquipmentState(IAsyncResult ar)
        {
            SetEquipmentStateAsyncResult result = (SetEquipmentStateAsyncResult)ar;
            if (result == null)
                throw new ArgumentException("Invalid parameter", "ar");
            if (!result.IsCompleted)
                result.AsyncWaitHandle.WaitOne();
            int response = ((TapiAsyncResult)ar).Response;

            if (response != 0)
                throw new TelephonyException("Error setting equipment state to  " + result.EquipmentState.ToString() + ".", response);

            return result.EquipmentState;
        }

        /// <summary>
        /// Gets general device information about the radio hardware, the radio software, the serial number, subscriber identity, and other information.
        /// </summary>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>
        /// The support for this function varies on CDMA devices. For this to complete successfully on a CDMA device, the device's RIL 
        /// driver must support the RIL_GetUserIdentity and RIL_GetEquipmentInfo functions.
        /// <para>
        /// This function can be used to retrieve the Electronic Serial Number (ESN) from a CDMA device. Typically, 
        /// the ESN will be in the <see cref="LineGeneralInfo.SerialNumber"/> property and is represented as an eight digit hexadecimal number.
        /// </para>
        /// </remarks>
        public LineGeneralInfo GeneralInfo
        {
            get
            {
                LineGeneralInfo lgi = new LineGeneralInfo(Marshal.SizeOf(typeof(LineGeneralInfo)));
                lgi.Store();
                int ret = NativeMethods.lineGetGeneralInfo(hLine, lgi.Data);
                if (ret != 0)
                    throw new TelephonyException("Error getting LineGeneralInfo.", ret);
                lgi.Load();
                lgi = new LineGeneralInfo(lgi.NeededSize);
                lgi.Store();
                ret = NativeMethods.lineGetGeneralInfo(hLine, lgi.Data);
                if (ret != 0)
                    throw new TelephonyException("Error getting LineGeneralInfo.", ret);
                lgi.Load();
                return lgi;
            }
        }

        /// <summary>
        /// Gets the <see cref="LineGprsClass"/> class of the Windows Mobile-based device and network.
        /// </summary>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public LineGprsClass GPRSClasses
        {//TODO lineSetGPRSClass
            get
            {
                LineGprsClass lgc;
                int  ret= NativeMethods.lineGetGPRSClass(hLine, out lgc);
                if (ret != 0)
                    throw new TelephonyException("Error getting GPRS Classes.", ret);
                return lgc;
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="MuteState"/> of the device.
        /// </summary>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public MuteState MuteState
        {
            get
            {
                MuteState ms;
                int ret = NativeMethods.lineGetMuteState(hLine, out ms);
                if (ret != 0)
                    throw new TelephonyException("Error getting mute state.", ret);
                return ms;
            }
            set
            {
                if (value != this.MuteState)
                {
                    SetMuteStateAsyncResult result = (SetMuteStateAsyncResult)BeginSetMuteState(value, null, null);
                    WaitAsyncResult(result);
                    MuteState ms = EndSetMuteState(result);
                }
            }
        }

        /// <summary>
        /// Begins an asynchronous request to set the mute state of the device using the <see cref="MuteState"/> parameter.
        /// </summary>
        /// <param name="muteState">The state to set the mute state.</param>
        /// <param name="Callback">Asynchronous callback delegate.</param>
        /// <param name="State">State of the object for the request.</param>
        /// <returns>An <see cref="IAsyncResult"/> that references the asynchronous request for a response.</returns>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The BeginSetMuteState method starts an asynchronous request to set the mute state of the device.
        /// 
        /// The asynchronous callback method should use the <see cref="EndSetMuteState"/> to validate and complete the change. 
        /// 
        /// <para>
        /// If an exception is encountered a <see cref="TelephonyException"/> will be thrown.
        /// </para>
        /// </remarks>
        public IAsyncResult BeginSetMuteState(MuteState muteState, AsyncCallback Callback, object State)
        {
            SetMuteStateAsyncResult result = new SetMuteStateAsyncResult(Callback, State, muteState);

            lock (Telephony.PendingRequests.SyncRoot)
            {
                result.ReplyID = NativeMethods.lineSetMuteState(hLine, (int)muteState);
                Telephony.PendingRequests.Add(result.ReplyID, result);
            }

            return result;
        }

        /// <summary>
        /// Ends an asynchronous request to set the mute state.
        /// </summary>
        /// <param name="ar">The pending request for a response. </param>
        /// <returns>A <see cref="MuteState"/> value of the current mute state.</returns>
        /// <exception cref="C:TelphonyException">If the asynchronous call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The EndSetMuteState method completes an asynchronous request to set mute state that was started by calling BeginSetMuteState</remarks>
        public MuteState EndSetMuteState(IAsyncResult ar)
        {
            SetMuteStateAsyncResult result = (SetMuteStateAsyncResult)ar;
            if (result == null)
                throw new ArgumentException("Invalid parameter", "ar");
            if (!result.IsCompleted)
                result.AsyncWaitHandle.WaitOne();
            int response = ((TapiAsyncResult)ar).Response;

            if (response != 0)
                throw new TelephonyException("Error setting mute state to  " + result.MuteState.ToString() + ".", response);

            return result.MuteState;
        }

        /// <summary>
        /// Gets the network registration status of the device
        /// </summary>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        public RegistrationStatus RegistrationStatus
        {
            get
            {
                RegistrationStatus r;
                int ret = NativeMethods.lineGetRegisterStatus(hLine, out r);
                if (ret != 0)
                    throw new TelephonyException("Error getting registration status.", ret);
                return r;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether or not to send the caller ID information when making a call.  
        /// </summary>
        /// <remarks>This setting may be overridden on a per-call basis.  See <see cref="M:Line.MakeCall"/>.
        /// <para>
        /// The setter of <b>SendCallerIDState</b> is a synchronous request.  To set the information asynchronously see <see cref="BeginSetSendCallerIDState"/> and <see cref="EndSetSendCallerIDState"/>.
        /// </para>
        ///         </remarks>
        public CallerIDState SendCallerIDState
        {
            get
            {
                CallerIDState cis;
                int ret = NativeMethods.lineGetSendCallerIDState(hLine, out cis);
                if (ret != 0)
                    throw new TelephonyException("Error getting Caller ID state.", ret);
                return cis;
            }
            set
            {
                SetCallerIDAsyncResult result = (SetCallerIDAsyncResult)BeginSetSendCallerIDState(value, null, null);
                WaitAsyncResult(result);
                EndSetSendCallerIDState(result);
            }
        }
        
        /// <summary>
        /// Begins an asynchronous request to set the send <see cref="CallerIDState">caller ID state</see>.
        /// </summary>
        /// <param name="callerIDState"></param>
        /// <param name="Callback">Asynchronous callback delegate.</param>
        /// <param name="State">State of the object for the request.</param>
        /// <returns>An <see cref="IAsyncResult"/> that references the asynchronous request for a response.</returns>
        /// <exception cref="C:TelphonyException">If the call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The BeginSetSendCallerIDState method starts an asynchronous request to set the send caller ID state of the phone device.
        /// 
        /// The asynchronous callback method should use the <see cref="EndSetSendCallerIDState"/> to validate the change. 
        /// 
        /// <para>
        /// If an exception is encountered a <see cref="TelephonyException"/> will be thrown.
        /// </para>
        /// </remarks>
        public IAsyncResult BeginSetSendCallerIDState(CallerIDState callerIDState, AsyncCallback Callback, object State)
        {
            SetCallerIDAsyncResult result = new SetCallerIDAsyncResult(Callback, State, callerIDState);

            lock (Telephony.PendingRequests.SyncRoot)
            {
                result.ReplyID = NativeMethods.lineSetSendCallerIDState(hLine, callerIDState);
                Telephony.PendingRequests.Add(result.ReplyID, result);
            }

            return result;
        }

        /// <summary>
        /// Ends an asynchronous request to set the send caller ID state.
        /// </summary>
        /// <param name="ar">The pending request for a response. </param>
        /// <returns>An <see cref="CallerIDState"/> value of the current state for sending caller ID information.</returns>
        /// <exception cref="C:TelphonyException">If the asynchronous call fails a TelephonyException with the <see cref="ErrorCode"/> will be thrown.</exception>
        /// <remarks>The EndSetSendCallerIDState method completes an asynchronous request to set the send state of the caller ID.</remarks>
        public CallerIDState EndSetSendCallerIDState(IAsyncResult ar)
        {
            SetCallerIDAsyncResult result = (SetCallerIDAsyncResult)ar;
            if (result == null)
                throw new ArgumentException("Invalid parameter", "ar");
            if (!result.IsCompleted)
                result.AsyncWaitHandle.WaitOne();
            int response = ((TapiAsyncResult)ar).Response;

            if (response != 0)
                throw new TelephonyException("Error setting caller ID state to  " + result.CallerIDState.ToString() + ".", response);

            return result.CallerIDState;
        }

        #region Extra properties using the APIs

        /// <summary>
        /// Gets the manufacturer name. This value indicates the maker of the underlying radio.
        /// </summary>
        public string Manufacturer
        {
            get
            {
                LineGeneralInfo lgi = GeneralInfo;
                return lgi.Manufacturer;
            }
        }

        /// <summary>
        /// Gets the model name.  This value indicates the model of the underlying radio. 
        /// </summary>
        public string Model
        {
            get
            {
                LineGeneralInfo lgi = GeneralInfo;
                return lgi.Model;
            }
        }

        /// <summary>
        /// Gets the revision name. This value indicates the software version of the radio stack. 
        /// </summary>
        public string Revision
        {
            get
            {
                LineGeneralInfo lgi = GeneralInfo;
                return lgi.Revision;
            }
        }

        /// <summary>
        /// Gets the serial number. For GSM devices, the serial number returned is the equipment identity International Mobile Equipment Identity (IMEI) code. For CDMA devices, the serial number returned is the Electronic Serial Number (ESN), represented as an eight digit hexadecimal string.
        /// </summary>
        public string SerialNumber
        {
            get
            {
                LineGeneralInfo lgi = GeneralInfo;
                return lgi.SerialNumber;
            }
        }

        /// <summary>
        /// Gets the subscriber number.
        /// </summary>
        public string SubscriberNumber
        {
            get
            {
                LineGeneralInfo lgi = GeneralInfo;
                return lgi.SubscriberNumber;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the mobile device has a phone. 
        /// </summary>
        public bool RadioPresent
        {
            get
            {
                bool bRet = false;
                if (File.Exists(Path.Combine(OpenNETCF.Environment2.GetFolderPath(OpenNETCF.SpecialFolder.Windows), "cellcore.dll")))
                {
                    RadioPresence rp;
                    int ret = NativeMethods.lineGetRadioPresence(hLine, out rp);
                    if (ret != 0)
                        throw new TelephonyException("Error getting registration status.", ret);
                    bRet = rp == RadioPresence.Present;
                }

                return bRet;
            }
        }
        /// <summary>
        /// Gets the name of the mobile operator (i.e., the mobile phone company, or carrier). 
        /// </summary>
        /// <remarks>An alternative method to get the same information use <see cref="CellularLine.CurrentOperator"/>.</remarks>
        public string OperatorName
        {
            get
            {
                return CurrentOperator.LongName;
            }
        }
        /// <summary>
        /// Gets or Sets the value indicating if the cellular radio equipment is off.
        /// </summary>
        /// <remarks>An alternate way to get the information is call GetEquipmentState or SetEquipmentState.</remarks>
        public bool RadioOff
        {
            get
            {
                EquipmentState es;
                RadioState rs;
                GetEquipmentState(out es, out rs);
                return es == EquipmentState.Minimum;
            }
            set
            {
                if (RadioOff == value)
                    SetEquipmentState(value ? EquipmentState.Minimum : EquipmentState.Full);
            }
        }

        /// <summary>
        /// Gets a value indicating whether the current line is CDMA.
        /// </summary>
        public bool IsCDMA
        {
            get
            {
                LineSystemType l = this.LineSystemType;
                return !(l == LineSystemType.Gsm || l == LineSystemType.Gprs);
            }
        }

        /// <summary>
        /// Gets a value indicating whether the current line is GSM type.
        /// </summary>
        public bool IsGSM
        {
            get
            {
                LineSystemType l = this.LineSystemType;
                return (l == LineSystemType.Gsm || l == LineSystemType.Gprs);
            }
        }

        
        #endregion

        /// <summary>
        /// Validates to make sure we have CellCore.dll onthe device which is just for PPC PE or SmartPhones.
        /// </summary>
        private void ValidateCellCore()
        {
            if (!File.Exists(Path.Combine(OpenNETCF.Environment2.GetFolderPath(OpenNETCF.SpecialFolder.Windows), "cellcore.dll")))
                throw new NotSupportedException("OpenNETCF Telephony API is only supported on Windows Mobile phone devices.");
        }

        private void ValidateGSM()
        {
            if (!IsGSM)
                throw new NotSupportedException("Functionality is not supported on CDMA devices");
        }

        private void WaitAsyncResult(CellCoreAsyncResult result)
        {
            bool isDone = false;
            while (!isDone)
            {
                Monitor.Enter(result);
                if (result.IsCompleted)
                    isDone = true;
                Monitor.Exit(result);
                Application.DoEvents();
            }
        }

    }
}
