// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.IO;
using OpenNETCF;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// Represents the method that will handle call event.
    /// </summary>
    public delegate void CallEventHandler(Call call, CallState callState, CallInfoState callInfoState);
    /// <summary>
    /// Represents the method that will handle line event.
    /// </summary>
    public delegate void LineEventHandler(Call call, CallState callState, CallInfoState callInfoState);
    /// <summary>
    /// Represents the method that will handle line busy event.
    /// </summary>
    public delegate void LineBusyEventHandler(Call call, CallState callState);
    /// <summary>
    /// Represents the method that will handle line disconnect event.
    /// </summary>
    public delegate void LineDisconnectEventHandler(Call call, CallState callState, DisconnectMode disconnectMode);
    /// <summary>
    /// Represents the method that will handle line connected event.
    /// </summary>
    public delegate void LineConnectedEventHandler(Call call, CallState callState);

    /// <summary>
    /// Monitors the line and gives various events on the line like incoming/outgoing call
    /// </summary>
    /// <remarks>
    /// This class is provided to make using the Telephony API (TAPI) easier to use and is intended for the developer not experienced with TAPI development.
    /// The main Telephony library is a thin wrapper to the Windows CE Telephony API and is intended for the developer familiar with TAPI.  For more information on 
    /// TAPI development please see <a href="http://msdn2.microsoft.com/en-us/library/ms927962.aspx">MSDN Documentation</a>.
    /// <para>
    /// To use this class you can either create a new object through code.
    /// <example>
    /// <code>CellularLineMonitor lineMonitor = new CellularLineMonitor();</code>
    /// </example>
    /// </para>
    /// <para>
    /// Once the new <see cref="CellularLineMonitor"/> object is created call the <see cref="CellularLineMonitor.Initialize"/> method to initialize the object.
    /// You <strong>must</strong> call <see cref="CellularLineMonitor.ShutDown"/> or <see cref="CellularLineMonitor.Dispose"/> to properly dispose of any internal
    /// objects not required anymore.
    /// </para>
    /// <para>
    /// The LineMonitor class inherits from <see cref="Component"/> and can therefore be used using the Visual Studio Designer.  You can simply add the LineMonitor to your Toolbox and add the
    /// LineMonitor <see cref="Component"/> to your form.  From there you can use the designer to set properties and register for LineMonitor events
    /// </para>
    /// </remarks>
    public partial class CellularLineMonitor : Component
    {
        /// <summary>
        /// Occurs when a new incoming call is detected on the current <see cref="Line"/>.
        /// </summary>
        /// <remarks>Use this event to monitor when a new incoming call is detected by the line.  To determine the source
        /// phone number use <see cref="P:Call.Info.CallerID"/>.</remarks>
        public event CallEventHandler NewIncomingCall;
        
        /// <summary>
        /// Occurs when a new outgoing call is detected on the current <see cref="Line"/>.
        /// </summary>
        /// <remarks>Use this event to monitor when a new outgoing call is detected by the line.  To determine the source
        /// phone number use <see cref="P:Call.Info.CalledID"/>.</remarks>
        public event CallEventHandler NewOutgoingCall;
        /// <summary>
        /// Occurs when a <see cref="Call"/> has been connected on the current <see cref="Line"/>.
        /// </summary>
        /// <remarks>Use this event to monitor when a <see cref="Call"/> has been connected on the <see cref="Line"/>.
        /// <see cref="P:LineMonitor.CurrentCall"/> will contain the current <see cref="Call"/> object.  The <see cref="Call.Info"/> parameter or
        /// <see cref="CallInformation"/> parameter will which will provide more specific details about the current call for example
        /// <see cref="CallInformation.CallerID"/>
        /// specific details about the call.
        /// </remarks>
        public event LineConnectedEventHandler LineConnected;
        /// <summary>
        /// Occurs when a <see cref="Call"/> has been disconnect on the current <see cref="Line"/>.
        /// </summary>
        /// <remarks>Use this event to monitor when a <see cref="Call"/> has been disconnected on the <see cref="Line"/>.
        /// </remarks>
        public event LineDisconnectEventHandler LineDisconnected;
        /// <summary>
        /// Occurs when an outgoing <see cref="Call"/> is not completed because a busy tone was returned from the destination.
        /// </summary>
        /// <remarks>When making an outgoing call a line busy status may be received from the destination.  Directly after this event
        /// the <see cref="LineDisconnected"/> event will be fired.  The developer can monitor either this event to 
        /// be notified of a busy signal on the destination or can use the <see cref="LineDisconnected"/> event in conjunction with
        /// the <see cref="DisconnectMode"/> parameter which will give the reason for the disconnection.</remarks>
        public event LineBusyEventHandler LineBusy;

        private Telephony m_telephony;
        private CellularLine m_line;
        private Call m_currentCall;

        internal DebugLogger m_logger;
        internal DebugLogger m_tapiLogger;

        /// <summary>
        /// Default constructor.
        /// </summary>
        public CellularLineMonitor()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Overloaded contructor.
        /// </summary>
        /// <param name="container"></param>
        public CellularLineMonitor(IContainer container)
        {
            container.Add(this);

            InitializeComponent();
        }

        /// <summary>
        /// Initializes the CellularLineMonitor class.  This method must be called to setup the monitoring.
        /// </summary>
        public virtual void Initialize()
        {
            string file = Path.Combine(
                Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase),
                "LineMonitor.log");

            m_logger = new OpenNETCF.DebugLogger(file,true,true);
            m_tapiLogger = new OpenNETCF.DebugLogger(Path.Combine(Path.GetDirectoryName(file),"TapiLog.log"), true, true);

            if (IsRunTime)
            {
                m_telephony = new Telephony();

                //Initialize the telephony class
                m_telephony.Initialize();

                //Get the cellular line with Monitor and Owner privileges
                m_line = m_telephony.CellularLine(MediaMode.InteractiveVoice,
                    CallPrivilege.Monitor | CallPrivilege.Owner);

                //Add a new call event
                m_line.NewCall += new Line.NewCallHandler(OnLineNewCall);
                m_telephony.LineMessage += new Telephony.MessageHandler(m_telephony_LineMessage);
            }
        }

        /// <summary>
        /// Shuts down the monitoring.
        /// </summary>
        /// <remarks>When the user is done with the CellularLineMonitor object, the ShutDown method must be called to dispose of any unmanaged objects.</remarks>
        public virtual void ShutDown()
        {
            if (IsRunTime)
            {
                if(m_line!=null)
                    m_line.Dispose();
                if(m_telephony != null)
                    m_telephony.Shutdown();
                m_telephony = null;
                m_line = null;
            }
        }

        /// <summary>
        /// Gets the current <see cref="Call"/> the <see cref="Line"/> is in.
        /// </summary>
        /// <remarks>Internally this variable is set when a new incoming or outgoing call is detected.</remarks>
        public Call CurrnetCall
        {
            get
            {
                return m_currentCall;
            }
        }

        /// <summary>
        /// Gets the current <see cref="CellularLine"/> associated with the device.
        /// </summary>
        public CellularLine Line
        {
            get
            {
                return m_line;
            }
        }

        /// <summary>
        /// Handles any new calls coming in/out of the phone the 
        /// </summary>
        /// <param name="call"></param>
        protected virtual void OnLineNewCall(Call call)
        {
            call.CallInfo += new Call.CallInfoHandler(OnCallInfo);
            call.Disconnected += new Call.CallDisconnectHandler(OnDisconnected);
            call.Connected += new Call.CallStateHandler(OnConnected);
            call.CallState += new Call.CallStateHandler(OnCallState);

            m_currentCall = call;
            m_logger.WriteLine("New call (" + call.Handle.ToString() + "):" + call.State.ToString());

        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="call"></param>
        /// <param name="state"></param>
        protected virtual void OnCallState(Call call, CallState state)
        {
            if (m_currentCall != null && m_currentCall.Handle != call.Handle)
                return;
            //Get rid of the m_currentCall when we get to callstate idle
            if (call.State == CallState.Idle)
            {
                m_currentCall = null;
            }
            m_logger.WriteLine("call_CallState(" + call.Handle.ToString() + "):" + call.State.ToString() + "\tCallState:" + state.ToString());
        }

        /// <summary>
        /// Raises either the <see cref="NewOutgoingCall"/> event or the <see cref="NewIncomingCall"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="infoState"></param>
        /// <param name="info"></param>
        protected virtual void OnCallInfo(Call call, CallInfoState infoState, CallInformation info)
        {
            if (m_currentCall != null && m_currentCall.Handle != call.Handle)
                return;
            
            m_logger.WriteLine("call_CallInfo(" + call.Handle.ToString() + "):" + call.State.ToString() + "\tInfoState:" + infoState.ToString());

            if (call.State == CallState.Proceeding && infoState == CallInfoState.ConnectedId)
            {
                OnOutgoingCall(call, call.State, infoState);
            }
            else if (call.State == CallState.Offering && infoState == CallInfoState.CallerId)
            {
                OnIncomingCall(call, call.State, infoState);
            }
        }

        /// <summary>
        /// Raises the <see cref="LineConnected"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="state"></param>
        protected virtual void OnConnected(Call call, CallState state)
        {
            if (m_currentCall != null && m_currentCall.Handle != call.Handle)
                return;

            m_logger.WriteLine("call_Connected(" + call.Handle.ToString() + "):" + call.State.ToString() + "\tCallState:" + state.ToString());

            if (state == CallState.Connected)
                OnLineConnected(call, state);
        }

        /// <summary>
        /// Raises the <see cref="LineDisconnected"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="state"></param>
        /// <param name="disconnectMode"></param>
        protected virtual void OnDisconnected(Call call, CallState state, DisconnectMode disconnectMode)
        {
            if (m_currentCall != null && m_currentCall.Handle != call.Handle)
                return;

            m_logger.WriteLine("call_Disconnected(" + call.Handle.ToString() + "):" + call.State.ToString() + "\tCallState:" + state.ToString() + "\tDisconnectedMode:" + disconnectMode.ToString());

            if (disconnectMode == DisconnectMode.Busy)
            {
                OnLineBusy(call, call.State);
            }
            if (state == CallState.Disconnected)
            {
                OnLineDisconnected(call, state,disconnectMode);
            }
        }

        /// <summary>
        /// Raises the <see cref="NewIncomingCall"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="callState"></param>
        /// <param name="callInfoState"></param>
        protected virtual void OnIncomingCall(Call call, CallState callState, CallInfoState callInfoState)
        {
            //Offering is incoming
            if (NewIncomingCall != null)
            {
                NewIncomingCall(call, callState, callInfoState);
            }
        }

        /// <summary>
        /// Raises the <see cref="NewOutgoingCall"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="callState"></param>
        /// <param name="callInfoState"></param>
        protected virtual void OnOutgoingCall(Call call, CallState callState, CallInfoState callInfoState)
        {
            //Proceeding is outgoing
            if (NewOutgoingCall != null)
            {
                NewOutgoingCall(call, callState, callInfoState);
            }
        }

        /// <summary>
        /// Raises the <see cref="LineConnected"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="callState"></param>
        protected virtual void OnLineConnected(Call call, CallState callState)
        {
            //Call connected
            if (LineConnected != null)
            {
                LineConnected(call, callState);
            }
        }

        /// <summary>
        /// Raises the <see cref="LineDisconnected"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="callState"></param>
        /// <param name="disconnectMode"></param>
        protected virtual void OnLineDisconnected(Call call, CallState callState, DisconnectMode disconnectMode)
        {
            if (LineDisconnected != null)
            {
                this.LineDisconnected(call, callState, disconnectMode);
            }
        }

        /// <summary>
        /// Raises the <see cref="LineBusy"/> event.
        /// </summary>
        /// <param name="call"></param>
        /// <param name="callState"></param>
        protected virtual void OnLineBusy(Call call, CallState callState)
        {
            if (LineBusy != null)
            {
                LineBusy(call, callState);
            }
        }

        /// <summary>
        /// Drops or hangs up ont the current <see cref="Call"/> the <see cref="Line"/> is in.
        /// </summary>
        /// <param name="errorCode"><see cref="ErrorCode"/> returned if the line was unable to be droped</param>
        /// <returns>true if the line dropped successfully or false if it was unable to drop the line</returns>
        /// <remarks>This method will hang up on the current call.  If the call succeeds a true will be returned.  If the 
        /// call fails to hang up then a false will be returned with the with the corresponding <see cref="ErrorCode"/> in the errorCode 
        /// parameter.</remarks>
        public virtual bool HangUp(out ErrorCode errorCode)
        {
            if (m_currentCall == null)
                throw new ArgumentException("Not in a call (m_currentCall is null).");

            OpenNETCF.Telephony.NativeMethods.lineSetCallPrivilege(m_currentCall.Handle, CallPrivilege.Owner);

            int t = NativeMethods.lineDrop(m_currentCall.Handle, null, 0);
            errorCode = (ErrorCode)t;
            return t>=0;
        }

        /// <summary>
        /// Determine if this instance is running against .NET Framework
        /// </summary>
        internal static bool IsDesignTime
        {
            get
            {
                // Determine if this instance is running against .NET Framework by using the MSCoreLib PublicKeyToken
                System.Reflection.Assembly mscorlibAssembly = typeof(int).Assembly;
                if ((mscorlibAssembly != null))
                {
                    if (mscorlibAssembly.FullName.ToUpper().EndsWith("B77A5C561934E089"))
                    {
                        return true;
                    }
                }
                return false;
            }
        }
        /// <summary>
        /// Determine if this instance is running against .NET Compact Framework
        /// </summary>
        internal static bool IsRunTime
        {
            get
            {
                // Determine if this instance is running against .NET Compact Framework by using the MSCoreLib PublicKeyToken
                System.Reflection.Assembly mscorlibAssembly = typeof(int).Assembly;
                if ((mscorlibAssembly != null))
                {
                    if (mscorlibAssembly.FullName.ToUpper().EndsWith("969DB8053D3322AC"))
                    {
                        return true;
                    }
                }
                return false;
            }
        }

        void m_telephony_LineMessage(LineMessage msg)
        {
            m_tapiLogger.WriteLine(
                "Line Message(" + msg.Param2 + "):" + msg.MessageID);
        }

    }
}
