// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.IO;
using System.Diagnostics;

namespace OpenNETCF
{
    internal class DebugLogger
    {
        string m_name = "";
        string m_indent = "";
        bool m_timestamps = false;
        bool m_startLine = true;
        string m_errorPrefix = "ERROR>";

        private object m_syncRoot = new object();
        private static string m_path;
        private static object m_pathSyncRoot = new object();

        public DebugLogger(string fileName)
            : this(fileName, false, true)
        {
        }

        public DebugLogger(string fileName, bool timestamps, bool truncate)
        {
            m_timestamps = timestamps;
            m_name = fileName;

            if (truncate)
            {
                if (File.Exists(m_name))
                {
                    try
                    {
                        File.Delete(m_name);
                    }
                    catch {/*ignore delete exceptions*/ }
                }
            }
        }

        private string DebugPath(string filename)
        {
            lock (m_pathSyncRoot)
            {
                if (m_path == null)
                {
                    m_path = Path.Combine(@"\Logs", DateTime.Now.ToString("MM-dd-yyyy-HH.mm.ss"));
                    if(!Directory.Exists(m_path))
                        Directory.CreateDirectory(m_path);
                }
                return Path.Combine(m_path, Path.GetFileName(filename));
            }
        }

        public bool TimeStamps
        {
            set { m_timestamps = value; }
            get { return m_timestamps; }
        }

        public string ErrorPrefix
        {
            set { m_errorPrefix = value; }
            get { return m_errorPrefix; }
        }

        [Conditional("TRACE")]
        public void Indent()
        {
            m_indent += "  ";
        }

        [Conditional("TRACE")]
        public void Unindent()
        {
            int length = m_indent.Length;
            if(length < 2)
            {
                return;
            }
            if (length == 2)
            {
                m_indent = "";
            }
            m_indent = new string(' ', length - 2);
        }

        [Conditional("TRACE")]
        public void WriteLine(object obj)
        {
            WriteLine(obj.ToString());
        }

        [Conditional("TRACE")]
        public void WriteLine(string text, bool suppressTimestamp)
        {
            bool oldTimestamp = TimeStamps;
            TimeStamps = false;
            WriteLine(text);
            TimeStamps = oldTimestamp;
        }

        [Conditional("TRACE")]
        public void WriteLine(string text)
        {
            Write(text + "\r\n");
            m_startLine = true;
        }

        [Conditional("TRACE")]
        public void WriteError(string text)
        {
            WriteLine(string.Format("{0}{1}", ErrorPrefix, text));
        }

        [Conditional("TRACE")]
        public void WriteException(string text, Exception e)
        {
            WriteLine(string.Format("{0}{1} : {2}",ErrorPrefix, text, e.Message));
        }

        [Conditional("TRACE")]
        public void Write(string text)
        {
            lock (m_syncRoot)
            {
                TextWriter writer;
                try
                {
                    writer = File.AppendText(m_name);
                }
                catch(Exception)
                {
                    // unable to open, so just bail out
                    goto done;
                }
                try
                {
                    if (m_timestamps && m_startLine)
                    {
                        writer.Write(string.Format("{0:X8}: {1}{2}", Environment.TickCount, m_indent, text));
                    }
                    else
                    {
                        writer.Write(string.Format("{0}{1}", m_indent, text));
                    }
                }
                catch { }

                writer.Close();
            done: ;
            }
            m_startLine = false;
        }
    }
}
