// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System.Runtime.InteropServices;
using System.Text;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// The DeviceCapabilities structure describes the capabilities of a line device. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class DeviceCapabilities : PhoneBase
    {
        /// <summary>
        /// Creates an instance of DeviceCapabilities
        /// </summary>
        /// <param name="nSize"></param>
        public DeviceCapabilities(int nSize)
            : base(nSize)
        {
            TotalSize = nSize;
        }

        /// <summary>
        /// 
        /// </summary>
        protected override int MinimumSize { get { return 1024; } }

        /// <summary>
        /// 
        /// </summary>
        public string LineName { get { return LineNameSize == 0 ? "" : Encoding.Unicode.GetString(Data, LineNameOffset, LineNameSize - 1); } }
        /// <summary>
        /// 
        /// </summary>
        public string ProviderName
        {
            get
            {
                return ProviderInfoSize == 0 ? "" : Encoding.Unicode.GetString(Data, ProviderInfoOffset, ProviderInfoSize - 1);
            }
        }
        /// <summary>
        /// 
        /// </summary>
        public string[] DeviceClasses
        {
            get
            {
                string sClass = Encoding.Unicode.GetString(Data, DeviceClassesOffset, DeviceClassesSize - 1);

                return sClass.Split('\0');
            }
        }
        /// <summary>
        /// The total size in bytes allocated to this data structure.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// The size in bytes for this data structure that is needed to hold all of the returned information.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// Size of the portion of this data structure that contains useful information, in bytes.
        /// </summary>
        public int UsedSize;

        /// <summary>
        /// Size of the variably sized field containing service provider information, in bytes.
        /// </summary>
        public int ProviderInfoSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int ProviderInfoOffset;

        /// <summary>
        /// Size of the variably sized device field containing switch information, in bytes.
        /// </summary>
        public int SwitchInfoSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int SwitchInfoOffset;

        /// <summary>
        /// Permanent identifier by which the line device is known in the system's configuration. It is a permanent name for the line device.
        /// </summary>
        public int PermanentLineID;
        /// <summary>
        /// Size of the variably sized device field containing a user configurable name for this line device, in bytes.
        /// </summary>
        public int LineNameSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int LineNameOffset;
        /// <summary>
        /// String format used with this line device.
        /// </summary>
        public int StringFormat;
        internal AddressMode AddressModes;
        /// <summary>
        /// Number of addresses associated with this line device.
        /// </summary>
        public int AddressCount;
        /// <summary>
        /// Flag array that indicates the different bearer modes that the address is able to support.
        /// </summary>
        public BearerMode BearerModes;
        /// <summary>
        /// Maximum data rate for information exchange over the call, in bits per second.
        /// </summary>
        public int MaxRate;
        /// <summary>
        /// Flag array that indicates the different media types the address is able to support. 
        /// </summary>
        public MediaMode MediaModes;

        /// <summary>
        /// Tones that can be generated on this line. 
        /// </summary>
        public int GenerateToneModes;
        /// <summary>
        /// Maximum number of frequencies that can be specified in describing a general tone using the LINEGENERATETONE data structure when generating a tone using lineGenerateTone
        /// </summary>
        public int GenerateToneMaxNumFreq;
        /// <summary>
        /// Digit modes than can be generated on this line.
        /// </summary>
        public int GenerateDigitModes;
        /// <summary>
        /// Maximum number of frequencies that can be specified in describing a general tone using the LINEMONITORTONE data structure when monitoring a general tone using lineMonitorTones. 
        /// </summary>
        public int MonitorToneMaxNumFreq;
        /// <summary>
        /// Maximum number of entries that can be specified in a tone list to lineMonitorTones.
        /// </summary>
        public int MonitorToneMaxNumEntries;
        /// <summary>
        /// Digit modes than can be detected on this line.
        /// </summary>
        public int MonitorDigitModes;
        /// <summary>
        /// Minimum value that can be specified for both the first digit and inter-digit timeout values used by lineGatherDigits, in milliseconds. 
        /// </summary>
        public int GatherDigitsMinTimeout;
        /// <summary>
        /// Maximum value that can be specified for both the first digit and inter-digit timeout values used by lineGatherDigits, in milliseconds. 
        /// </summary>
        public int GatherDigitsMaxTimeout;

        /// <summary>
        /// Maximum number of entries that can be specified in the digit list parameter of lineSetMediaControl.
        /// </summary>
        public int MedCtlDigitMaxListSize;
        /// <summary>
        /// Maximum number of entries that can be specified in the media list parameter of lineSetMediaControl.
        /// </summary>
        public int MedCtlMediaMaxListSize;
        /// <summary>
        /// Maximum number of entries that can be specified in the tone list parameter of lineSetMediaControl.
        /// </summary>
        public int MedCtlToneMaxListSize;
        /// <summary>
        /// Maximum number of entries that can be specified in the call state list parameter of lineSetMediaControl.
        /// </summary>
        public int MedCtlCallStateMaxListSize;

        /// <summary>
        /// Boolean device capabilities. 
        /// </summary>
        public DeviceCapability DevCapFlags;
        /// <summary>
        /// Maximum number of (minimum bandwidth) calls that can be active (connected) on the line at any one time. 
        /// </summary>
        public int MaxNumActiveCalls;
        /// <summary>
        /// Effect on the active call when answering another offering call on a line device.
        /// </summary>
        public AnswerMode AnswerMode;
        /// <summary>
        /// Number of different ring modes that can be reported in the LINE_LINEDEVSTATE message with the ringing indication.
        /// </summary>
        public int RingModes;
        /// <summary>
        /// Different line status components for which the application may be notified in a LINE_LINEDEVSTATE message on this line.
        /// </summary>
        public DeviceState LineStates;

        /// <summary>
        /// Maximum size of user-user information, including the null terminator, that can be sent during a call accept.
        /// </summary>
        public int UUIAcceptSize;
        /// <summary>
        /// Maximum size of user-user information, including the null terminator, that can be sent during a call answer.
        /// </summary>
        public int UUIAnswerSize;
        /// <summary>
        /// Maximum size of user-user information, including the null terminator, that can be sent during a make call.
        /// </summary>
        public int UUIMakeCallSize;
        /// <summary>
        /// Maximum size of user-user information, including the null terminator, that can be sent during a call drop.
        /// </summary>
        public int UUIDropSize;
        /// <summary>
        ///     Maximum size of user-user information, including the null terminator, that can be sent separately any time during a call with lineSendUserUserInfo.
        /// </summary>
        public int UUISendUserUserInfoSize;
        /// <summary>
        /// Maximum size of user-user information, including the null terminator, that can be received in the LINECALLINFO structure.
        /// </summary>
        public int UUICallInfoSize;

        /// <summary>
        /// Minimum value for the dial parameters that can be set for calls on this line, in milliseconds.
        /// </summary>
        public DialParameters MinDialParams;
        /// <summary>
        /// Maximum value for the dial parameters that can be set for calls on this line, in milliseconds.
        /// </summary>
        public DialParameters MaxDialParams;
        /// <summary>
        /// Default dial parameters used for calls on this line. 
        /// </summary>
        public DialParameters DefaultDialParams;

        /// <summary>
        /// Number of terminals that can be set for this line device, its addresses, or its calls. Individual terminals are referred to by terminal IDs and range from zero to one less than the value indicated by dwNumTerminals.
        /// </summary>
        public int NumTerminals;
        /// <summary>
        /// Size of the variably sized device field containing an array with entries of type LINETERMCAPS, in bytes.
        /// </summary>
        public int TerminalCapsSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int TerminalCapsOffset;
        /// <summary>
        /// Size of each of the terminal text descriptions, including the null terminator
        /// </summary>
        public int TerminalTextEntrySize;
        /// <summary>
        /// Size of the variably sized device-specific field, in bytes. 
        /// </summary>
        public int TerminalTextSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int TerminalTextOffset;

        /// <summary>
        /// Size of the variably sized device-specific field, in bytes. 
        /// </summary>
        public int DevSpecificSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int DevSpecificOffset;

        /// <summary>
        /// Features available for this line
        /// </summary>
        public int LineFeatures;

        /// <summary>
        /// DeviceStatus flag values that can be modified using SetLineDevStatus
        /// </summary>
        public int SettableDevStatus;
        /// <summary>
        /// Size of a string containing the device class identifiers supported on one or more addresses on this line, including the null terminator, in bytes.
        /// </summary>
        public int DeviceClassesSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int DeviceClassesOffset;
    }
}
