// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;

namespace OpenNETCF.Telephony
{
	internal enum InitializeExOptions
	{
		USEHIDDENWINDOW      =0x00000001,  // TAPI v2.0
		USEEVENT             =0x00000002,  // TAPI v2.0
		USECOMPLETIONPORT    =0x00000003,  // TAPI v2.0
	}

    /// <summary>
    /// The MediaControl bit-flag constants describe a set of generic operations on media streams. The interpretations are determined by the media stream. The line device must have the media-control capability for any media-control operation to be effective.
    /// </summary>
	[Flags]
	public enum MediaControl
	{
        /// <summary>
        /// No change is to be made to the media stream.
        /// </summary>
        None = 0x00000001,
        /// <summary>
        /// Start the media stream.
        /// </summary>
        Start = 0x00000002,
        /// <summary>
        /// Reset the media stream. Equivalent to an end-of-input. All buffers are released.
        /// </summary>
        Reset = 0x00000004,
        /// <summary>
        /// Temporarily pause the media stream. The speed of the media stream is returned to normal.
        /// </summary>
        Pause = 0x00000008,
        /// <summary>
        /// Resume a paused media stream.
        /// </summary>
        Resume = 0x00000010,
        /// <summary>
        /// The speed of the media stream is increased by some stream-defined quantity.
        /// </summary>
        RateUp = 0x00000020,
        /// <summary>
        /// The speed of the media stream is decreased by some stream-defined quantity.
        /// </summary>
        RateDown = 0x00000040,
        /// <summary>
        /// The speed of the media stream is returned to normal.
        /// </summary>
        RateNormal = 0x00000080,
        /// <summary>
        /// The amplitude of the media stream is increased by some stream-defined quantity.
        /// </summary>
        VolumeUp = 0x00000100,
        /// <summary>
        /// The amplitude of the media stream is decreased by some stream-defined quantity.
        /// </summary>
        VolumeDown = 0x00000200,
        /// <summary>
        /// The amplitude of the media stream is returned to normal.
        /// </summary>
        VolumeNormal = 0x00000400,
	}

    /// <summary>
    /// These constants describe media types (or modes) of a communications session or call. 
    /// </summary>
	[Flags]
	public enum MediaMode
	{
        /// <summary>
        /// A media stream exists but its mode is not currently known and may become known later. This would correspond to a call with an unclassified media type. In typical analog telephony environments, an incoming call's media type may be unknown until after the call has been answered and the media stream has been filtered to make a determination. 
        /// </summary>
		Unknown                   =0x00000002,
        /// <summary>
        /// Voice energy was detected on the call, and the call is handled as an interactive voice call with humans on both ends.
        /// </summary>
		InteractiveVoice          =0x00000004,
        /// <summary>
        /// Voice energy was detected on the call, and the voice is locally handled by an automated application such as with an answering machine application. When a service provider cannot distinguish between interactive and automated voice on an incoming call, it will report the call as interactive voice.
        /// </summary>
		AutomatedVoice            =0x00000008,
        /// <summary>
        /// A data modem session on the call. Current modem protocols require the called station to initiate the handshake. For an incoming data modem call, the application can typically make no positive detection. How the service provider makes this determination is its choice. For example, a period of silence just after answering an incoming call can be used as a heuristic to decide that this might be a data modem call.
        /// </summary>
		DataModem                 =0x00000010,
        /// <summary>
        /// A group 3 fax is being sent or received over the call.
        /// </summary>
		G3Fax                     =0x00000020,
        /// <summary>
        /// A Telephony Devices for the Deaf (TDD) () session on the call.
        /// </summary>
		TDD                       =0x00000040,
        /// <summary>
        /// A group 4 fax is being sent or received over the call.
        /// </summary>
		G4Fax                     =0x00000080,
        /// <summary>
        /// A digital data stream of unspecified format. 
        /// </summary>
		DigitalData               =0x00000100,
        /// <summary>
        /// A teletex session on the call. Teletex is one of the telematic services.
        /// </summary>
		Teletex                   =0x00000200,
        /// <summary>
        /// A videotex session on the call. Videotex is one the telematic services.
        /// </summary>
		Videotex                  =0x00000400,
        /// <summary>
        /// A telex session on the call. Telex is one of the telematic services.
        /// </summary>
		Telex                     =0x00000800,
        /// <summary>
        /// A mixed session on the call. Mixed is one of the ISDN telematic services.
        /// </summary>
		Mixed                     =0x00001000,
        /// <summary>
        /// An Analog Display Services Interface (ADSI) session on the call. ADSI enhances voice calls with alphanumeric information downloaded to the phone and the use of soft buttons on the phone.
        /// </summary>
		ADSI                      =0x00002000,
        /// <summary>
        /// The media type of the call is VoiceView. TAPI versions 1.4 and later.
        /// </summary>
		VoiceView                 =0x00004000,      // TAPI v1.4
	}

    /// <summary>
    /// Error codes that TAPI can return when invoking operations on lines, addresses, or calls.
    /// </summary>
	public enum ErrorCode: uint
	{
        /// <summary>
        /// The line cannot be opened due to a persistent condition, such as that of a serial port being exclusively opened by another process.
        /// </summary>
		Allocated                       = 0x00000001,
        /// <summary>
        /// The specified device identifier or line device identifier, such as in a dwDeviceID parameter, is invalid or out of range.
        /// </summary>
		BadDeviceID                     = 0x00000002,
        /// <summary>
        /// The bearer mode member is invalid, is not available, or the call's bearer mode cannot be changed to the specified bearer mode.
        /// </summary>
		BearerModeUnavailable           = 0x00000003,
        /// <summary>
        /// All call appearances on the specified address are currently in use.
        /// </summary>
		CallUnavailable                 = 0x00000005,
        /// <summary>
        /// The maximum number of outstanding call completions has been exceeded.
        /// </summary>
		CompletionOverrun               = 0x80000006,
        /// <summary>
        /// The maximum number of parties for a conference has been reached, or the requested number of parties cannot be satisfied.
        /// </summary>
		ConferenceFull                  = 0x80000007,
        /// <summary>
        /// The dialable address parameter contains dialing control characters that are not processed by the service provider.
        /// </summary>
		DialBilling                     = 0x00000008,
        /// <summary>
        /// The dialable address parameter contains dialing control characters that are not processed by the service provider.
        /// </summary>
		DialDialTone                    = 0x00000009,
        /// <summary>
        /// The dialable address parameter contains dialing control characters that are not processed by the service provider.
        /// </summary>
		DialPrompt                      = 0x8000000A,
        /// <summary>
        /// The dialable address parameter contains dialing control characters that are not processed by the service provider.
        /// </summary>
		DialQuiet                       = 0x8000000B,
        /// <summary>
        /// The application requested a TAPI version or version range that is either incompatible with, or cannot be supported by, the Telephony API implementation and the corresponding service provider.
        /// </summary>
		IncompatibleAPIVersion          = 0x8000000C,
        /// <summary>
        /// The application requested an extension version range that is either invalid or cannot be supported by the corresponding service provider.
        /// </summary>
		IncompatibleExtensionVersion    = 0x8000000D,
        /// <summary>
        /// The Telephon.ini file cannot be read or understood properly by TAPI because of internal inconsistencies or formatting problems. For example, the [Locations], [Cards], or [Countries] section of the Telephon.ini file may be corrupted or inconsistent.
        /// </summary>
		IniFileCorrupt                  = 0x8000000E,
        /// <summary>
        /// The line device is in use and cannot currently be configured, allow a party to be added, allow a call to be answered, allow a call to be placed, or allow a call to be transferred.
        /// </summary>
		InUse                           = 0x8000000F,
        /// <summary>
        /// A specified address is either invalid or not allowed. If invalid, the address contains invalid characters or digits, or the destination address contains dialing control characters, W, @, $, or ?, that are not supported by the service provider. If the address is not allowed, the specified address is either not assigned to the specified line or is not valid for address redirection.
        /// </summary>
		InvalidAddress                  = 0x80000010,
        /// <summary>
        /// The specified address identifier is either invalid or out of range. 
        /// </summary>
		InvalidAddressID                = 0x80000011,
        /// <summary>
        /// The specified address mode is invalid.
        /// </summary>
		InvalidAddressMode              = 0x80000012,
        /// <summary>
        /// The specified address state contains one or more bits that are not valid. 
        /// </summary>
		InvalidAddressState             = 0x80000013,
        /// <summary>
        /// The application handle (such as specified by a hLineApp parameter) or the application registration handle is invalid.
        /// </summary>
		InvalidApplicationHandle        = 0x80000014,
        /// <summary>
        /// The specified application name is invalid. If an application name is specified by the application, it is assumed that the string does not contain any non-displayable characters, and is zero-terminated.
        /// </summary>
		InvalidApplicationName          = 0x80000015,
        /// <summary>
        /// The specified bearer mode is invalid.
        /// </summary>
		InvalidBearerMode               = 0x80000016,
        /// <summary>
        /// The specified completion is invalid.
        /// </summary>
		InvalidCompletion               = 0x80000017,
        /// <summary>
        /// The specified call handle is not valid. For example, the handle is not NULL but does not belong to the given line. In some cases, the specified call device handle is invalid.
        /// </summary>
		InvalidCallHandle               = 0x80000018,
        /// <summary>
        /// The specified call parameters are invalid.
        /// </summary>
		InvalidCallParameters           = 0x80000019,
        /// <summary>
        /// The specified call privilege parameter is invalid.
        /// </summary>
		InvalidCallPrivilege            = 0x8000001A,
        /// <summary>
        /// The specified select parameter is invalid.
        /// </summary>
		InvalidCallSelect               = 0x8000001B,
        /// <summary>
        /// The current state of a call is not in a valid state for the requested operation.
        /// </summary>
		InvalidCallState                = 0x8000001C,
        /// <summary>
        /// The specified call state list is invalid.
        /// </summary>
		InvalidCallStateList            = 0x8000001D,
        /// <summary>
        /// The permanent card identifier specified could not be found in any entry in the [Cards] section in the registry.
        /// </summary>
		InvalidCard                     = 0x8000001E,
        /// <summary>
        /// The completion identifier is invalid.
        /// </summary>
		InvalidCompletionID             = 0x8000001F,
        /// <summary>
        /// The specified call handle for the conference call is invalid or is not a handle for a conference call.
        /// </summary>
		InvalidConferenceCallHandle     = 0x80000020,
        /// <summary>
        /// The specified consultation call handle is invalid.
        /// </summary>
		InvalidConsultationCallHandle   = 0x80000021,
        /// <summary>
        /// The specified country/region code is invalid.
        /// </summary>
		InvalidCountryCode              = 0x80000022,
        /// <summary>
        /// The line device has no associated device for the given device class, or the specified line does not support the indicated device class.
        /// </summary>
		InvalidDeviceClass              = 0x80000023,
        /// <summary>
        /// The line device handle is invalid.
        /// </summary>
		InvalidDeviceHandle             = 0x80000024,
        /// <summary>
        /// The dialing parameters are invalid. 
        /// </summary>
		InvalidDialParameters           = 0x80000025,
        /// <summary>
        /// The specified digit list is invalid. 
        /// </summary>
		InvalidDigitList                = 0x80000026,
        /// <summary>
        /// The specified digit mode is invalid.
        /// </summary>
		InvlaidDigitMode                = 0x80000027,
        /// <summary>
        /// The specified termination digits are not valid.
        /// </summary>
		InvalidTerminationDigits        = 0x80000028,
        /// <summary>
        /// The service provider extension version number is invalid. 
        /// </summary>
		InvalidExtensionVersion         = 0x80000029,
        /// <summary>
        /// The specified group identifier is invalid. 
        /// </summary>
		InvalidGroupID                  = 0x8000002A,
        /// <summary>
        /// The specified call, device, line device, or line handle is invalid.
        /// </summary>
		InvalidLineHandle               = 0x8000002B,
        /// <summary>
        /// The device configuration may not be changed in the current line state. The line may be in use by another application or the LineStates parameter contains one or more bits that are not defined constants. The InvalidLineState value can also indicate that the device is Disconnected or OutOfService.
        /// </summary>
		InvalidLineState                = 0x8000002C,
        /// <summary>
        /// The permanent location identifier could not be found in any entry in the [Locations] section in the registry.
        /// </summary>
		InvalidLocation                 = 0x8000002D,
        /// <summary>
        /// The specified media list is invalid.
        /// </summary>
		InvalidMediaList                = 0x8000002E,
        /// <summary>
        /// The list of media types (modes) to be monitored contains invalid information, the specified media type parameter is invalid, or the service provider does not support the specified media type.
        /// </summary>
		InvalidMediaMode                = 0x8000002F,
        /// <summary>
        /// The specified MessageID is invalid
        /// </summary>
		InvalidMessageID                = 0x80000030,
        /// <summary>
        /// A parameter or structure that a parameter points to contains invalid information, a country/region code is invalid, a window handle is invalid, or the specified forward list parameter contains invalid information.
        /// </summary>
		InvalidParameter                = 0x80000032,
        /// <summary>
        /// The park identifier is invalid.
        /// </summary>
		InvalidParkID                   = 0x80000033,
        /// <summary>
        /// The specified park mode is invalid.
        /// </summary>
		InvalidParkMode                 = 0x80000034,
        /// <summary>
        /// One or more of the specified pointer parameters, such as lpCallList, lpdwAPIVersion, lpExtensionID, lpdwExtVersion, lphIcon, lpLineDevCaps, and lpToneList, are invalid, or a required pointer to an output parameter is NULL.
        /// </summary>
		InvalidPointer                  = 0x80000035,
        /// <summary>
        /// An invalid flag or combination of flags was set for the dwPrivileges parameter in lineOpen.
        /// </summary>
		InvalidPrivilegeSelected        = 0x80000036,
        /// <summary>
        /// The specified rate is invalid.
        /// </summary>
		InvalidRate                     = 0x80000037,
        /// <summary>
        /// The specified request mode is invalid.
        /// </summary>
		InvalidRequestMode              = 0x80000038,
        /// <summary>
        /// The specified terminal identifier is invalid.
        /// </summary>
		InvalidTerminalID               = 0x80000039,
        /// <summary>
        /// The specified terminal mode is invalid.
        /// </summary>
		InvalidTerminalMode             = 0x8000003A,
        /// <summary>
        /// Timeouts are not supported or a value falls outside the supported range.
        /// </summary>
		InvalidTimeout                  = 0x8000003B,
        /// <summary>
        /// The specified custom tone does not represent a valid tone or is made up of too many frequencies or the specified tone structure does not describe a valid tone.
        /// </summary>
		InvalidTone                     = 0x8000003C,
        /// <summary>
        /// The specified tone list is invalid.
        /// </summary>
		InvalidToneList                 = 0x8000003D,
        /// <summary>
        /// The specified tone mode is invalid.
        /// </summary>
		InvalidToneMode                 = 0x8000003E,
        /// <summary>
        /// The specified transfer mode parameter is invalid.
        /// </summary>
		InvalidTransferMode             = 0x8000003F,
        /// <summary>
        /// LINEMAPPER was the value passed in the dwDeviceID parameter, but no lines were found that match the requirements specified in the lpCallParams parameter.
        /// </summary>
		LineMapperFailed                = 0x80000040,
        /// <summary>
        /// The specified call is not a conference call handle or a participant call.
        /// </summary>
		NoConference                    = 0x80000041,
        /// <summary>
        /// The specified device identifier, which was previously valid, is no longer accepted because the associated device has been removed from the system since TAPI was last initialized. Alternately, the line device has no associated device for the given device class.
        /// </summary>
		NoDevice                        = 0x80000042,
        /// <summary>
        /// Either Tapiaddr.dll could not be located or the telephone service provider for the specified device found that one of its components is missing or corrupt in a way that was not detected at initialization time. The user should be advised to use the Telephony Control Panel to correct the problem.
        /// </summary>
		NoDriver                        = 0x80000043,
        /// <summary>
        /// Insufficient memory to perform the operation, or unable to lock memory.
        /// </summary>
		OutOfMemory                     = 0x80000044,
        /// <summary>
        /// There currently is no request pending of the indicated mode, or the application is no longer the highest-priority application for the specified request mode.
        /// </summary>
		NoRequest                       = 0x80000045,
        /// <summary>
        /// The application does not have owner privilege to the specified call.
        /// </summary>
		NotOwner                        = 0x80000046,
        /// <summary>
        /// The application is not registered as a request recipient for the indicated request mode.
        /// </summary>
		NotRegistered                   = 0x80000047,
        /// <summary>
        /// The operation failed for an unspecified or unknown reason.
        /// </summary>
		OperationFailed                 = 0x80000048,
        /// <summary>
        /// The operation is not available, such as for the given device or specified line.
        /// </summary>
		OperationUnavailable            = 0x80000049,
        /// <summary>
        /// The service provider currently does not have enough bandwidth available for the specified rate.
        /// </summary>
		RateUnavailable                 = 0x8000004A,
        /// <summary>
        /// Insufficient resources to complete the operation. For example, a line cannot be opened due to a dynamic resource overcommitment.
        /// </summary>
		ResourceUnavailable             = 0x8000004B,
        /// <summary>
        /// More requests are pending than the device can handle.
        /// </summary>
		RequestOverrun                  = 0x8000004C,
        /// <summary>
        /// The TotalSize member of a structure does not specify enough memory to contain the fixed portion of the specified structure.
        /// </summary>
		StructureTooSmall               = 0x8000004D,
        /// <summary>
        /// A target for the call handoff was not found. This can occur if the named application did not open the same line with the LINECALLPRIVILEGE_Owner bit in the dwPrivileges parameter of lineOpen. Or, in the case of media-mode handoff, no application has opened the same line with the LINECALLPRIVILEGE_Owner bit in the dwPrivileges parameter of lineOpen and with the media type specified in the MediaMode parameter having been specified in the dwMediaModes parameter of lineOpen.
        /// </summary>
		TargetNotFound                  = 0x8000004E,
        /// <summary>
        /// The application invoking this operation is the target of the indirect handoff. That is, TAPI has determined that the calling application is also the highest priority application for the given media type.
        /// </summary>
		TargetSelf                      = 0x8000004F,
        /// <summary>
        /// The operation was invoked before any application called lineInitialize or lineInitializeEx.
        /// </summary>
		Uninitialized                   = 0x80000050,
        /// <summary>
        /// The string containing user-user information exceeds the maximum number of bytes specified in the dwUUIAcceptSize, dwUUIAnswerSize, dwUUIDropSize, dwUUIMakeCallSize, or dwUUISendUserUserInfoSize member of LINEDEVCAPS, or the string containing user-user information is too long.
        /// </summary>
		UserInfoTooBig                  = 0x80000051,
        /// <summary>
        /// If TAPI reinitialization has been requested, for example as a result of adding or removing a telephony service provider, then lineInitialize, lineInitializeEx, or lineOpen requests are rejected with this error until the last application shuts down its usage of TAPI (using lineShutdown), at which time the new configuration becomes effective and applications are once again permitted to call lineInitialize or lineInitializeEx.
        /// </summary>
		Reinit                          = 0x80000052,
        /// <summary>
        /// The target call address has call blocking enabled.
        /// </summary>
		AddressBlocked                  = 0x80000053,
        /// <summary>
        /// The billing mode of the call was rejected.
        /// </summary>
		BillingRejected                 = 0x80000054,
        /// <summary>
        /// The application invoked a feature that is not available on this line. 
        /// </summary>
		InvalidFeature                  = 0x80000055,
        /// <summary>
        /// A telephony service provider that does not support multiple instances is listed more than once in the [Providers] section in the registry. The application should advise the user to use the Telephony Control Panel to remove the duplicated driver.
        /// </summary>
		NoMultipleInstance              = 0x80000056,
        /// <summary>
        /// The agent ID is invalid. This value is exposed only to applications that negotiate a TAPI version of 2.0 or higher.
        /// </summary>
		InvalidAgentID                  = 0x80000057,      // TAPI v2.0
        /// <summary>
        /// The agent group is invalid.  This value is exposed only to applications that negotiate a TAPI version of 2.0 or higher.
        /// </summary>
		InvalidAgentGroup                 = 0x80000058,      // TAPI v2.0
        /// <summary>
        /// The application used an invalid password. This value is exposed only to applications that negotiate a TAPI version of 2.0 or higher.
        /// </summary>
		InvalidPassword                 = 0x80000059,      // TAPI v2.0
        /// <summary>
        /// The agent state is invalid.  This value is exposed only to applications that negotiate a TAPI version of 2.0 or higher.
        /// </summary>
		InvalidAgentState               = 0x8000005A,      // TAPI v2.0
        /// <summary>
        /// The agent activity is invalid.  This value is exposed only to applications that negotiate a TAPI version of 2.0 or higher.
        /// </summary>
		InvalidAgentActivity            = 0x8000005B,      // TAPI v2.0
        /// <summary>
        /// Use of the ":" dial modifier is not supported. This value is exposed only to applications that negotiate a TAPI version of 2.0 or higher.
        /// </summary>
		VoiceDetect                     = 0x8000005C,      // TAPI v2.0
		// ExTAPI LINEERR_ constants
        /// <summary>
        /// The specified password was incorrect
        /// </summary>
		IncorrectPassword               = 0x80010001,
	}

    /// <summary>
    /// The DisconnectMode bit-flag constants describe different reasons for a remote disconnect request. A disconnect mode is available as call status to the application after the call state transitions to disconnected.
    /// </summary>
    /// <remarks>Changed from internal to public in v1.1</remarks>
	[Flags]
	public enum DisconnectMode
	{
        /// <summary>
        /// This is a normal disconnect request by the remote party. The call was terminated normally.
        /// </summary>
        Normal = 0x00000001,
        /// <summary>
        /// The reason for the disconnect request is unknown but may become known later.
        /// </summary>
        Unknown = 0x00000002,
        /// <summary>
        /// The remote user has rejected the call.
        /// </summary>
        Reject = 0x00000004,
        /// <summary>
        /// The call was picked up from elsewhere.
        /// </summary>
        Pickup = 0x00000008,
        /// <summary>
        /// The call was forwarded by the switch.
        /// </summary>
        Forwarded = 0x00000010,
        /// <summary>
        /// The remote user's station is busy.
        /// </summary>
        Busy = 0x00000020,
        /// <summary>
        /// The remote user's station does not answer.
        /// </summary>
        NoAnswer = 0x00000040,
        /// <summary>
        /// The destination address is invalid.
        /// </summary>
        BadAddress = 0x00000080,
        /// <summary>
        /// The remote user could not be reached.
        /// </summary>
        Unreachable = 0x00000100,
        /// <summary>
        /// The network is congested.
        /// </summary>
        Congestion = 0x00000200,
        /// <summary>
        /// The remote user's station equipment is incompatible with the type of call requested.
        /// </summary>
        Incompatible = 0x00000400,
        /// <summary>
        /// The reason for the disconnect is unavailable and will not become known later.
        /// </summary>
        Unavailable = 0x00000800,
        /// <summary>
        /// A dial tone was not detected within a service-provider defined timeout, at a point during dialing when one was expected (such as at a "W" in the dialable string). This can also occur without a service-provider-defined timeout period or without a value specified in the WaitForDialTone member of the DialParameters structure.
        /// </summary>
        NoDialTone = 0x00001000,      // TAPI v1.4

        /// <summary>
        /// The call could not be connected because the destination number has been changed, but automatic redirection to the new number is not provided.
        /// </summary>
        NumberChanged = 0x00002000,      // TAPI v2.0
        /// <summary>
        /// The call could not be connected or was disconnected because the destination device is out of order (hardware failure). 
        /// </summary>
        OutOfOrder = 0x00004000,      // TAPI v2.0
        /// <summary>
        /// he call could not be connected or was disconnected because of a temporary failure in the network; the call can be reattempted later and is expected to eventually complete.
        /// </summary>
        TempFailure = 0x00008000,      // TAPI v2.0
        /// <summary>
        /// he call could not be connected or was disconnected because the minimum quality of service could not be obtained or sustained.
        /// </summary>
        QOSUnavailable = 0x00010000,      // TAPI v2.0
        /// <summary>
        /// he call could not be connected because calls from the origination address are not being accepted at the destination address. 
        /// </summary>
        Blocked = 0x00020000,      // TAPI v2.0
        /// <summary>
        /// The call could not be connected because the destination has invoked the Do Not Disturb feature. 
        /// </summary>
		DoNotDisturb = 0x00040000,      // TAPI v2.0
        /// <summary>
        /// The call was cancelled.
        /// </summary>
        Cancelled = 0x00080000,      // TAPI v2.0
        /// <summary>
        /// 
        /// </summary>
        PhoneConnectionFailure = (Normal | (0xd0 << 16)),
        /// <summary>
        /// 
        /// </summary>
        InvalidSimCard = (Normal | (0xd1 << 16)),
        /// <summary>
        /// 
        /// </summary>
        SimCardBusy = (Normal | (0xd2 << 16)),
        /// <summary>
        /// 
        /// </summary>
        NetworkServiceNotAvailable = (Normal | (0xd3 << 16)),
        /// <summary>
        /// 
        /// </summary>
        EmergencyOnly = (Normal | (0xd4 << 16)),
	}

    /// <bugs>
    /// <bug id="26">Cannot access the field MessageID from struct LineMessage</bug>
    /// <bug id="42">MessageID on LineMessage should be made public</bug>
    /// </bugs>
    /// <summary></summary>
	public enum LineMessages
	{
		LINE_ADDRESSSTATE                       =0,
		LINE_CALLINFO                           =1,
		LINE_CALLSTATE                          =2,
		LINE_CLOSE                              =3,
		LINE_DEVSPECIFIC                        =4,
		LINE_DEVSPECIFICFEATURE                 =5,
		LINE_GATHERDIGITS                       =6,
		LINE_GENERATE                           =7,
		LINE_LINEDEVSTATE                       =8,
		LINE_MONITORDIGITS                      =9,
		LINE_MONITORMEDIA                       =10,
		LINE_MONITORTONE                        =11,
		LINE_REPLY                              =12,
		LINE_REQUEST                            =13,
		PHONE_BUTTON                            =14,
		PHONE_CLOSE                             =15,
		PHONE_DEVSPECIFIC                       =16,
		PHONE_REPLY                             =17,
		PHONE_STATE                             =18,
		LINE_CREATE                             =19,             // TAPI v1.4
		PHONE_CREATE                            =20,             // TAPI v1.4

		LINE_AGENTSPECIFIC                      =21,             // TAPI v2.0
		LINE_AGENTSTATUS                        =22,             // TAPI v2.0
		LINE_APPNEWCALL                         =23,             // TAPI v2.0
		LINE_PROXYREQUEST                       =24,             // TAPI v2.0
		LINE_REMOVE                             =25,             // TAPI v2.0
		PHONE_REMOVE                            =26,             // TAPI v2.0

        #region TapiEx Messages
        /// <summary>
        /// Sent to the application every time the equipment state changes. NOTE: Only supported for phone devices.
        /// </summary>
        /// <remarks>
        /// To return the same information on demand use the <see cref="CellularLine.GetEquipmentState"/> method or <see cref="NativeMethods.lineGetEquipmentState"/>.
        /// The <see cref="LineMessage"/> struct params are as follows:
        /// dwParam2 = Indicates the equipment state. See the <see cref="EquipmentState"/>  for possible values. 
        /// dwParam3 = Indicates whether the major radio functions are on or off. See the <see cref="RadioState"/> constants for possible values.
        /// </remarks>
        LINE_EQUIPSTATECHANGE = 0x00000100,

        /// <summary>
        /// Sent to the application every time the General Packet Radio Service (GPRS) class is changed.   NOTE: Only supported for phone devices.
        /// </summary>
        /// <remarks>
        /// To return the same information on demand use the <see cref="CellularLine.GPRSClasses"/> method or <see cref="NativeMethods.lineGetGPRSClass"/>.
        /// The <see cref="LineMessage"/> struct params are as follows:
        /// dwParam2 = The class of the mobile and network. See the <see cref="LineGprsClass"/> for possible values. 
        /// dwParam3 = Indicates the change that occured. See the <see cref="LineGprsClassChanged"/> constants for possible values.
        /// </remarks>
        LINE_GPRSCLASS = 0x00000101,

        /// <summary>
        /// Sent to the application every time the registration state changes. This includes switching to a different cell tower.  NOTE: Only supported for phone devices.  For CDMA devices see <see cref="LINE_REGISTERSTATE"/>
        /// </summary>
        /// <remarks>
        /// To return the same information on demand use the <see cref="CellularLine.RegistrationStatus"/> method or <see cref="NativeMethods.lineGetRegisterStatus"/>.
        /// The <see cref="LineMessage"/> struct params are as follows:
        /// dwParam2 = The registration state. See the <see cref="RegistrationStatus"/> for possible values.
        /// </remarks>
        LINE_GPRSREGISTERSTATE = 0x00000102,

        /// <summary>
        /// Sent to the application every time the radio is inserted or removed.  NOTE: Only supported for phone devices.
        /// </summary>
        /// <remarks>
        /// To return the same information on demand use the <see cref="CellularLine.RadioPresent"/> method or <see cref="NativeMethods.lineGetRadioPresence"/>.
        /// The <see cref="LineMessage"/> struct params are as follows:
        /// dwParam2 = The new equipment state. See the <see cref="RadioPresence"/> for possible values. 
        /// </remarks>
        LINE_RADIOPRESENCE = 0x00000103,

        /// <summary>
        /// Sent to the application every time the registration state changes. This includes switching to a different cell tower.  NOTE: Only supported for phone devices.
        /// </summary>
        /// <remarks>
        /// To return the same information on demand use the <see cref="CellularLine.RegistrationStatus"/> method or <see cref="NativeMethods.lineGetRegisterStatus"/>.
        /// The <see cref="LineMessage"/> struct params are as follows:
        /// dwParam2 = The registration state. See the <see cref="RegistrationStatus"/> for possible values.
        /// </remarks>
        LINE_REGISTERSTATE = 0x00000104,
        // dwParam1 = LINE_REGISTERSTATE
        // dwParam2 = One of the LINEREGSTATUS_* constants
        // dwParam3 Unused

        //TODO should this be added in future?
        //  LINE_USSD                               = 0x00000105,
        // dwParam1 = LINE_USSD
        // dwParam2 = Message identifier
        // dwParam3 = Size in bytes of message

        /// <summary>
        /// Sent to the application every time the current line changes.  NOTE: Only supported for phone devices.
        /// </summary>
        /// <remarks>
        /// The <see cref="LineMessage"/> struct params are as follows:
        /// dwParam2 = New line identifier
        /// dwParam3 = New address ID
        /// </remarks>
        LINE_CURRENTLINECHANGE = 0x00000106,

        /// <summary>
        /// Sent to the application every time the current system changes.  NOTE: Only supported for phone devices.
        /// </summary>
        /// <remarks>
        /// To return the same information on demand use the <see cref="CellularLine.LineSystemType"/> property or <see cref="NativeMethods.lineGetCurrentSystemType"/>.
        /// The <see cref="LineMessage"/> struct params are as follows:
        /// dwParam2 = New system coverage. See <see cref="LineSystemType"/>.
        /// dwParam3 = unused
        /// </remarks>
        LINE_CURRENTSYSTEMCHANGE = 0x00000107
        #endregion
	}

	[Flags]
	internal enum AddressCapability
	{
		FWDNUMRINGS            =0x00000001,
		PICKUPGROUPID          =0x00000002,
		SECURE                 =0x00000004,
		BLOCKIDDEFAULT         =0x00000008,
		BLOCKIDOVERRIDE        =0x00000010,
		DIALED                 =0x00000020,
		ORIGOFFHOOK            =0x00000040,
		DESTOFFHOOK            =0x00000080,
		FWDCONSULT             =0x00000100,
		SETUPCONFNULL          =0x00000200,
		AUTORECONNECT          =0x00000400,
		COMPLETIONID           =0x00000800,
		TRANSFERHELD           =0x00001000,
		TRANSFERMAKE           =0x00002000,
		CONFERENCEHELD         =0x00004000,
		CONFERENCEMAKE         =0x00008000,
		PARTIALDIAL            =0x00010000,
		FWDSTATUSVALID         =0x00020000,
		FWDINTEXTADDR          =0x00040000,
		FWDBUSYNAADDR          =0x00080000,
		ACCEPTTOALERT          =0x00100000,
		CONFDROP               =0x00200000,
		PICKUPCALLWAIT         =0x00400000,

		PREDICTIVEDIALER       =0x00800000,      // TAPI v2.0
		QUEUE                  =0x01000000,      // TAPI v2.0
		ROUTEPOINT             =0x02000000,      // TAPI v2.0
		HOLDMAKESNEW           =0x04000000,      // TAPI v2.0
		NOINTERNALCALLS        =0x08000000,      // TAPI v2.0
		NOEXTERNALCALLS        =0x10000000,      // TAPI v2.0
		SETCALLINGID           =0x20000000,      // TAPI v2.0
	}

    /// <summary>
    /// These bit-flag constants describe different bearer modes of a call. When an application makes a call, it can request a specific bearer mode. These modes are used to select a certain quality of service for the requested connection from the underlying telephone network. Bearer modes available on a given line are a device capability of the line. 
    /// </summary>
	[Flags]
	public enum BearerMode
	{
        /// <summary>
        /// This is a regular 3.1 kHz analog voice-grade bearer service. Bit integrity is not assured. Voice can support fax and modem media types.
        /// </summary>
		Voice                    =0x00000001,
        /// <summary>
        /// This corresponds to G.711 speech transmission on the call. The network can use processing techniques such as analog transmission, echo cancellation, and compression/decompression. Bit integrity is not assured. Speech is not intended to support fax and modem media types.
        /// </summary>
		Speech                   =0x00000002,
        /// <summary>
        /// The multiuse mode defined by ISDN.
        /// </summary>
		Multiuse                 =0x00000004,
        /// <summary>
        /// The unrestricted data transfer on the call. The data rate is specified separately.
        /// </summary>
		Data                     =0x00000008,
        /// <summary>
        /// The alternate transfer of speech or unrestricted data on the same call (ISDN).
        /// </summary>
		AlternateSpeechData      =0x00000010,
        /// <summary>
        /// This corresponds to a non-call-associated signaling connection from the application to the service provider or switch. This is treated as a media stream by TAPI.
        /// </summary>
		NonCallSignaling         =0x00000020,
        /// <summary>
        /// When a call is active in LINEBEARERMODE_PASSTHROUGH, the service provider gives direct access to the attached hardware for control by the application. This mode is used primarily by applications desiring temporary direct control over asynchronous modems, accessed through the Win32 communication functions, for the purpose of configuring or using special features not otherwise supported by the service provider. TAPI version 1.4 and later.
        /// </summary>
		Passthrough              =0x00000040,      // TAPI v1.4
	}

	public enum CallSelect
	{
		LINE                     =0x00000001,
		ADDRESS                  =0x00000002,
		CALL                     =0x00000004,
		DEVICEID                 =0x00000008,      // TAPI v2.1
		CALLID                   =0x00000010,      // TAPI v3.0
	}

	[Flags]
	public enum AddressMode
	{
		ADDRESSID               =0x00000001,
		DIALABLEADDR            =0x00000002,
	}

    /// <summary>
    /// The CallInfoState bit-flag constants describe various call information items about which an application can be notified
    /// </summary>
	[Flags]
	public enum CallInfoState
	{
        /// <summary>
        /// Call information items other than those listed below have changed. The application should check the current call information to determine which items have changed.
        /// </summary>
		Other                 =0x00000001,
        /// <summary>
        /// The device-specific field of the call-information record.
        /// </summary>
        DeviceSpecific = 0x00000002,
        /// <summary>
        /// The bearer mode field of the call-information record.
        /// </summary>
        BearerMode = 0x00000004,
        /// <summary>
        /// The rate field of the call-information record.
        /// </summary>
        Rate = 0x00000008,
        /// <summary>
        /// The media type field of the call-information record.
        /// </summary>
        MediaMode = 0x00000010,
        /// <summary>
        /// The application-specific field of the call-information record.
        /// </summary>
        ApplicationSpecific = 0x00000020,
        /// <summary>
        /// The call ID field of the call-information record.
        /// </summary>
        CallId = 0x00000040,
        /// <summary>
        /// The related call ID field of the call-information record.
        /// </summary>
        RelatedCallId = 0x00000080,
        /// <summary>
        /// The origin field of the call-information record.
        /// </summary>
        Origin = 0x00000100,
        /// <summary>
        /// The reason field of the call-information record.
        /// </summary>
        Reason = 0x00000200,
        /// <summary>
        /// The completion identifier field of the call-information record.
        /// </summary>
        CompletionId = 0x00000400,
        /// <summary>
        /// The number of owner field in the call-information record was increased.
        /// </summary>
        OwnerIncrement = 0x00000800,
        /// <summary>
        /// The number of owner field in the call-information record was decreased.
        /// </summary>
        OwnerDecrement = 0x00001000,
        /// <summary>
        /// One or more of the digit, tone, or media monitoring fields in the call-information record.
        /// </summary>
        MonitorsChanged = 0x00002000,
        /// <summary>
        /// The trunk field of the call-information record.
        /// </summary>
        Trunk = 0x00004000,
        /// <summary>
        /// One of the callerID-related fields of the call-information record.
        /// </summary>
        CallerId = 0x00008000,
        /// <summary>
        /// One of the calledID-related fields of the call-information record.
        /// </summary>
        CalledId = 0x00010000,
        /// <summary>
        /// One of the connectedID-related fields of the call-information record.
        /// </summary>
        ConnectedId = 0x00020000,
        /// <summary>
        /// The address identifier of the location to which a call has been redirected.
        /// </summary>
        RedirectionId = 0x00040000,
        /// <summary>
        /// The address identifier of the location that redirected a call.
        /// </summary>
        RedirectingId = 0x00080000,
        /// <summary>
        /// The display field of the call-information record.
        /// </summary>
        Display = 0x00100000,
        /// <summary>
        /// The user-user information of the call-information record.
        /// </summary>
        UserUserInfo = 0x00200000,
        /// <summary>
        /// The high level compatibility field of the call-information record.
        /// </summary>
        HighLevelCompatibility = 0x00400000,
        /// <summary>
        /// The low level compatibility field of the call-information record.
        /// </summary>
        LowLevelCompatibility = 0x00800000,
        /// <summary>
        /// The charging information of the call-information record.
        /// </summary>
        ChargingInfo = 0x01000000,
        /// <summary>
        /// The terminal mode information of the call-information record.
        /// </summary>
        Terminal = 0x02000000,
        /// <summary>
        /// The terminal mode information of the call-information record.
        /// </summary>
        DialParameters = 0x04000000,
        /// <summary>
        /// One or more of the digit, tone, or media monitoring fields in the call-information record.
        /// </summary>
        MonitorModes = 0x08000000,

        /// <summary>
        /// The CallTreatment member in CallInfo has been updated.
        /// </summary>
        Treatment = 0x10000000,      // TAPI v2.0
        /// <summary>
        /// One or more of the QOS members in CallInfo has been updated.
        /// </summary>
        QOS = 0x20000000,      // TAPI v2.0
        /// <summary>
        /// The CallData member in CallInfo has been updated.
        /// </summary>
        CallData = 0x40000000,      // TAPI v2.0
	}

    /// <summary>
    /// These bit-flag constants describe the call states a call can be in.
    /// </summary>
	[Flags]
	public enum CallState
	{
        /// <summary>
        /// The call exists but has not been connected. No activity exists on the call. This means that no call is currently active. A call can never transition out of the idle state.
        /// </summary>
		Idle                      =0x00000001,
        /// <summary>
        /// The call is being offered to the station, signaling the arrival of a new call. The offering state is not the same as causing a phone or computer to ring. In some environments, a call in the offering state does not ring the user until the switch instructs the line to ring. For example this state is in use when an incoming call appears on several station sets but only the primary address rings. The instruction to ring does not affect any call states.
        /// </summary>
		Offering                  =0x00000002,
        /// <summary>
        /// The call was in the offering state and has been accepted. This indicates to other, monitoring, applications that the current owner application has claimed responsibility for answering the call. In ISDN, the accepted state is entered when the called-party equipment sends a message to the switch indicating that it is willing to present the call to the called person. This has the side effect of alerting (ringing) the users at both ends of the call. An incoming call can always be immediately answered without first being separately accepted.
        /// </summary>
		Accepted                  =0x00000004,
        /// <summary>
        /// The call is receiving a dial tone from the switch. This means that the switch is ready to receive a dialed number.
        /// </summary>
		DialTone                  =0x00000008,
        /// <summary>
        /// The originator is dialing digits on the call. The dialed digits are collected by the switch. Note that neither lineGenerateDigits nor TSPI_lineGenerateDigits will place the line into the dialing state.
        /// </summary>
		Dialing                   =0x00000010,
        /// <summary>
        /// The station to be called has been reached, and the destination's switch is generating a ring tone back to the originator. A ringback means that the destination address is being alerted to the call.
        /// </summary>
		Ringback                  =0x00000020,
        /// <summary>
        /// The call is receiving a busy tone. A busy tone indicates that the call cannot be completed. This occurs if either a circuit (trunk) or the remote party's station are in use.
        /// </summary>
		Busy                      =0x00000040,
        /// <summary>
        /// The call is receiving a special information signal that precedes a prerecorded announcement indicating why a call cannot be completed. 
        /// </summary>
		SpecialInfo               =0x00000080,
        /// <summary>
        /// The call has been established and the connection is made. Information is able to flow over the call between the originating address and the destination address.
        /// </summary>
		Connected                 =0x00000100,
        /// <summary>
        /// Dialing has completed and the call is proceeding through the switch or telephone network. This occurs after dialing is complete and before the call reaches the dialed party, as indicated by ringback, busy, or answer.
        /// </summary>
		Proceeding                =0x00000200,
        /// <summary>
        /// The call is on hold by the switch. This frees the physical line. This allows another call to use the line.
        /// </summary>
		OnHold                    =0x00000400,
        /// <summary>
        /// The call is a member of a conference call and is logically in the connected state.
        /// </summary>
		Conferenced               =0x00000800,
        /// <summary>
        /// The call is currently on hold while it is being added to a conference.
        /// </summary>
		OnHoldPendingConference   =0x00001000,
        /// <summary>
        /// The call is currently on hold awaiting transfer to another number.
        /// </summary>
		OnHoldPendingTransfer      =0x00002000,
        /// <summary>
        /// The remote party has disconnected from the call.
        /// </summary>
		Disconnected              =0x00004000,
        /// <summary>
        /// The call exists, but its state is currently unknown. This may be the result of poor call progress detection by the service provider. A call state message with the call state set to unknown may also be generated to inform the TAPI DLL about a new call at a time when the actual call state of the call is not exactly known
        /// </summary>
		Unknown                   =0x00008000,
	}

    /// <summary>
    /// The DeviceState bit-flag constants describe various line status events.
    /// </summary>
	[Flags]
	public enum DeviceState
	{
        /// <summary>
        /// Device-status items other than those listed below have changed. The application should check the current device status to determine which items have changed.
        /// </summary>
		Other                      =0x00000001,
        /// <summary>
        /// The switch tells the line to alert the user.
        /// </summary>
        Ringing = 0x00000002,
        /// <summary>
        /// The line was previously disconnected and is now connected to TAPI.
        /// </summary>
        Connected = 0x00000004,
        /// <summary>
        /// This line was previously connected and is now disconnected from TAPI.
        /// </summary>
        Disconnected = 0x00000008,
        /// <summary>
        /// The message waiting indicator is turned on.
        /// </summary>
        MsgWaitOn = 0x00000010,
        /// <summary>
        /// The message waiting indicator is turned off.
        /// </summary>
        MsgWaitOff = 0x00000020,
        /// <summary>
        /// The line is connected to TAPI. This happens when TAPI is first activated or when the line wire is physically plugged in and in-service at the switch while TAPI is active.
        /// </summary>
        InService = 0x00000040,
        /// <summary>
        /// The line is out of service at the switch or physically disconnected. TAPI cannot be used to operate on the line device.
        /// </summary>
        OutOfService = 0x00000080,
        /// <summary>
        /// Maintenance is being performed on the line at the switch. TAPI cannot be used to operate on the line device.
        /// </summary>
        Maintenance = 0x00000100,
        /// <summary>
        /// The line has been opened by another application.
        /// </summary>
        Open = 0x00000200,
        /// <summary>
        /// The line has been closed by another application.
        /// </summary>
        Close = 0x00000400,
        /// <summary>
        /// The number of calls on the line device has changed.
        /// </summary>
        NumberOfCallsChanged = 0x00000800,
        /// <summary>
        /// The number of outstanding call completions on the line device has changed.
        /// </summary>
        NumberOfCompletionsChanged = 0x00001000,
        /// <summary>
        /// The terminal settings have changed. This can happen, for example, if multiple line devices share terminals among them (for example, two lines sharing a phone terminal).
        /// </summary>
        Terminals = 0x00002000,
        /// <summary>
        /// The roam mode of the line device has changed.
        /// </summary>
        RoamMode = 0x00004000,
        /// <summary>
        /// The battery level has changed significantly (cellular).
        /// </summary>
        Battery = 0x00008000,
        /// <summary>
        /// The signal level has changed significantly (cellular).
        /// </summary>
        Signal = 0x00010000,
        /// <summary>
        /// The line's device-specific information has changed.
        /// </summary>
        DeviceSpecific = 0x00020000,
        /// <summary>
        /// Items have changed in the configuration of line devices. To become aware of these changes (as for the appearance of new line devices) the application should reinitialize its use of TAPI.
        /// </summary>
        ReInit = 0x00040000,
        /// <summary>
        /// The locked status of the line device has changed. 
        /// </summary>
        Lock = 0x00080000,
        /// <summary>
        /// Indicates that, due to configuration changes made by the user or other circumstances, one or more of the members in the DeviceCapabilities structure for the address have changed. 
        /// </summary>
        CapabilitiesChanged = 0x00100000,      // TAPI v1.4
        /// <summary>
        /// Indicates that configuration changes have been made to one or more of the media devices associated with the line device. 
        /// </summary>
        ConfigurationChanged = 0x00200000,      // TAPI v1.4
        /// <summary>
        /// Indicates that, due to configuration changes made by the user or other circumstances
        /// </summary>
        TranslateChanged = 0x00400000,      // TAPI v1.4
        /// <summary>
        /// Indicates that the call completion identified by the completion identifier contained in the dwParam2 parameter of the DeviceState message has been externally canceled and is no longer considered valid
        /// </summary>
        CompletionCancelled = 0x00800000,      // TAPI v1.4
        /// <summary>
        /// Indicates that the device is being removed from the system by the service provider (most likely through user action, through a control panel or similar utility).
        /// </summary>
        Removed = 0x01000000,      // TAPI v1.4
	}

	/// <summary>
    /// Bit-flag constants are a collection of Booleans describing various line device capabilities.
    /// </summary>
    [Flags]
	public enum DeviceCapability
	{
		/// <summary>
        /// Specifies whether calls on different addresses on this line can be conferenced
        /// </summary>
        CrossAddrConf           =0x00000001,
		/// <summary>
        /// Specifies whether high-level compatibility information elements are supported on this line.
        /// </summary>
        HighLevelCompatibility  =0x00000002,
		/// <summary>
        /// Specifies whether low-level compatibility information elements are supported on this line.
        /// </summary>
        LowLevelCompatibility   =0x00000004,
        /// <summary>
        /// Specifies whether media-control operations are available for calls at this line.
        /// </summary>
        MediaControl = 0x00000008,
        /// <summary>
        /// Specifies whether lineMakeCall, lineDial, TSPI_lineMakeCall, or TSPI_lineDial is able to deal with multiple addresses at once (as for inverse multiplexing).
        /// </summary>
        MultipleAddresses = 0x00000010,
        /// <summary>
        /// Specifies what happens when an open line is closed while the application has calls active on the line. 
        /// </summary>
        CloseDrop = 0x00000020,
        /// <summary>
        /// These flags indicate whether the "$" dialable string modifier is supported for a given line device.
        /// </summary>
        DialBilling = 0x00000040,
        /// <summary>
        /// These flags indicate whether the "@" dialable string modifier is supported for a given line device.
        /// </summary>
        DialQuiet = 0x00000080,
        /// <summary>
        /// These flags indicate whether the "W" dialable string modifier is supported for a given line device.
        /// </summary>
        DialDialTone = 0x00000100,
        /// <summary>
        /// Indicates whether a Media Service Provider (MSP) is associated with the line. This flag is exposed only to applications that negotiate a TAPI version of 3.0 or higher.
        /// </summary>
        MSP = 0x00000200,      // TAPI v3.0
        /// <summary>
        /// Indicates whether call hubs are supported on this line. This flag is exposed only to applications that negotiate a TAPI version of 3.0 or higher.
        /// </summary>
        CallHub = 0x00000400,      // TAPI v3.0
        /// <summary>
        /// Indicates whether call hub tracking is supported on this line. This flag is exposed only to applications that negotiate a TAPI version of 3.0 or higher.
        /// </summary>
        CallHubTracking = 0x00000800,      // TAPI v3.0
        /// <summary>
        /// Indicates whether provider-specific Interfaces have been implemented. This flag is exposed only to applications that negotiate a TAPI version of 3.0 or higher.
        /// </summary>
        PrivateObjects = 0x00001000,      // TAPI v3.0
	}

    /// <summary>
    /// The AnswerMode bit-flag constants describe how an existing active call on a line device is affected by answering another offering call on the same line.
    /// </summary>
	[Flags]
	public enum AnswerMode
	{
        /// <summary>
        /// Answering another call on the same line has no effect on the existing active call on the line.
        /// </summary>
        None = 0x00000001,
        /// <summary>
        /// The currently active call will automatically be dropped.
        /// </summary>
        Drop = 0x00000002,
        /// <summary>
        /// The currently active call will automatically be placed on hold.
        /// </summary>
        Hold = 0x00000004,
	}

    /// <summary>
    /// The CallPrivilege bit-flag constants describe the kinds of access rights or privileges an application with a call handle may have to the corresponding call.
    /// </summary>
	[Flags]
	public enum CallPrivilege
	{
        /// <summary>
        /// The application has no privileges to the call. The application's handle is void and should not be used.
        /// </summary>
        None = 0x00000001,
        /// <summary>
        /// The application has monitor privileges to the call. These privileges allow the application to monitor state changes and query information and status about the call.
        /// </summary>
        Monitor = 0x00000002,
        /// <summary>
        /// The application has owner privileges to the call. These privileges allow the application to manipulate the call in ways that affect the state of the call.
        /// </summary>
        Owner = 0x00000004,
	}

    /// <summary>
    /// The DigitMode constants describe different types of inband digit generation.
    /// </summary>
	[Flags]
	public enum DigitMode
	{
        /// <summary>
        /// Uses rotary pulse sequences to signal digits. Valid digits are 0 through 9.
        /// </summary>
		Pulse                     =0x00000001,
        /// <summary>
        /// Uses DTMF tones to signal digits. Valid digits are 0 through 9, '*', '#', 'A', 'B', 'C', and 'D'.
        /// </summary>
		Dtmf                      =0x00000002,
        /// <summary>
        /// Uses DTMF tones to signal digits and detect the down edges. Valid digits are 0 through 9, '*', '#', 'A', 'B', 'C', and 'D'.
        /// </summary>
		DtmfEnd                   =0x00000004,
	}

    /// <summary>
    /// 
    /// </summary>
	public enum CallerIDOptions
	{
        /// <summary>
        /// Accept the default behavior
        /// </summary>
		Default,
        /// <summary>
        /// Block sending of caller-ID information, overriding the current default if necessary
        /// </summary>
		Block,
        /// <summary>
        /// Send caller-ID information, overriding the current default if necessary
        /// </summary>
		Send,
	}

    /// <summary>
    /// The CallOrigin constants describe the origin of a call.
    /// </summary>
	[Flags]
	public enum CallOrigin
	{
        /// <summary>
        /// The call originated from this station as an outgoing call.
        /// </summary>
        Outbound = 0x00000001,
        /// <summary>
        /// The call originated as an incoming call at a station internal to the same switching environment.
        /// </summary>
        Internal = 0x00000002,
        /// <summary>
        /// The call originated as an incoming call on an external line.
        /// </summary>
        External = 0x00000004,
        /// <summary>
        /// The call origin is currently unknown but may become known later.
        /// </summary>
        Unknown = 0x00000010,
        /// <summary>
        /// The call origin is not available and will never become known for this call.
        /// </summary>
        Unavailable = 0x00000020,
        /// <summary>
        /// The call handle is for a conference call, that is, it is the application's connection to the conference bridge in the switch.
        /// </summary>
        Conference = 0x00000040,
        /// <summary>
        /// The call originated as an incoming call, but the service provider is unable to determine whether it came from another station on the same switch or from an external line.
        /// </summary>
        Inbound = 0x00000080,      // TAPI v1.4
	}

    /// <summary>
    /// The CallParameterFlags constants describe various status flags about a call.
    /// </summary>
	[Flags]
	public enum CallParameterFlags
	{
		/// <summary>
        /// The call should be set up as secure.
        /// </summary>
        Secure               =0x00000001,
        /// <summary>
        /// The call should be originated on an idle call appearance and not join a call in progress. 
        /// </summary>
        Idle = 0x00000002,
		/// <summary>
        /// The originator identity should be concealed (block caller ID).
		/// </summary>
        BlockId = 0x00000004,
        /// <summary>
        /// The originator's phone should be automatically taken offhook.
        /// </summary>
        OriginatorOffHook = 0x00000008,
        /// <summary>
        /// The called party's phone should be automatically taken offhook.
        /// </summary>
        DestinationOffHook = 0x00000010,
        /// <summary>
        /// This bit is used only in conjunction with lineSetupConference and linePrepareAddToConference. 
        /// </summary>
        NoHoldConference = 0x00000020,      // TAPI v2.0
        /// <summary>
        /// This bit is used only when placing a call on an address with predictive dialing capability (LINEADDRCAPFLAGS_PREDICTIVEDIALER is on in the dwAddrCapFlags member in LINEADDRESSCAPS).
        /// </summary>
        PredictiveDial = 0x00000040,      // TAPI v2.0
        /// <summary>
        /// This bit is used only in conjunction with lineSetupTransfer. It combines the operation of lineSetupTransfer followed by lineDial on the consultation call into a single step. 
        /// </summary>
        OneStepTransfer = 0x00000080,      // TAPI v2.0
	}

    /// <summary>
    /// The DialToneMode bit-flag constants describe different types of dial tones. A special dial tone typically carries a special meaning (as with message waiting).
    /// </summary>
    [Flags]
	public enum DialToneMode
	{
	    /// <summary>
        /// This is a normal dial tone, which typically is a continuous tone.
	    /// </summary>
		Normal                 =0x00000001,
        /// <summary>
        /// This is a special dial tone indicating that a certain condition (known by the switch or network) is currently in effect. Special dial tones typically use an interrupted tone. As with a normal dial tone, this indicates that the switch is ready to receive the number to be dialed.
        /// </summary>
        Special = 0x00000002,
        /// <summary>
        /// This is an internal dial tone, as within a PBX.
        /// </summary>
        Internal = 0x00000004,
        /// <summary>
        /// This is an external (public network) dial tone.
        /// </summary>
        External = 0x00000008,
        /// <summary>
        /// The dial tone mode is not currently known but may become known later.
        /// </summary>
        Unknown = 0x00000010,
        /// <summary>
        /// The dial tone mode is unavailable and will not become known.
        /// </summary>
        Unavailable = 0x00000020,
	}
    
	/// <summary>
    /// Bit-flag constants describe the reason for a call.
    /// </summary>
    [Flags]
	public enum CallReason
	{
		/// <summary>
        /// This is a direct incoming or outgoing call.
        /// </summary>
        Direct                   =0x00000001,
        /// <summary>
        /// This call was forwarded from another extension that was busy at the time of the call.
        /// </summary>
        FwdBusy = 0x00000002,
        /// <summary>
        /// The call was forwarded from another extension that didn't answer the call after some number of rings.
        /// </summary>
        FwdNoAnswer = 0x00000004,
        /// <summary>
        /// The call was forwarded unconditionally from another number.
        /// </summary>
        FwdUnconditional = 0x00000008,
        /// <summary>
        /// The call was picked up from another extension.
        /// </summary>
        Pickup = 0x00000010,
        /// <summary>
        /// The call was retrieved as a parked call.
        /// </summary>
        Unpark = 0x00000020,
        /// <summary>
        /// The call was redirected to this station.
        /// </summary>
        Redirect = 0x00000040,
        /// <summary>
        /// The call was the result of a call completion request.
        /// </summary>
        CallCompletion = 0x00000080,
        /// <summary>
        /// The call has been transferred from another number.
        /// </summary>
        Transfer = 0x00000100,
        /// <summary>
        /// The call is a reminder (or "recall") that the user has a call parked or on hold for (potentially) a long time.
        /// </summary>
        Reminder = 0x00000200,
        /// <summary>
        /// The reason for the call is currently unknown but may become known later.
        /// </summary>
        Unknown = 0x00000400,
        /// <summary>
        /// The reason for the call is unavailable and will not become known later.
        /// </summary>
        Unavailable = 0x00000800,
        /// <summary>
        /// The call intruded onto the line, either by a call completion action invoked by another station or by operator action. 
        /// </summary>
        Intruded = 0x00001000,      // TAPI v1.4
        /// <summary>
        /// The call was parked on the address. Usually, it appears initially in the onhold state.
        /// </summary>
        Parked = 0x00002000,      // TAPI v1.4

        /// <summary>
        /// The call was camped on the address
        /// </summary>
        CampedOn = 0x00004000,      // TAPI v2.0
        /// <summary>
        /// The call appears on the address because the switch needs routing instructions from the application. 
        /// </summary>
        RouteRequest = 0x00008000,      // TAPI v2.0
	}

    /// <summary>
    /// The LineFeature constants list the operations that can be invoked on a line using this API.
    /// </summary>
	[Flags]
	public enum LineFeature
	{
        /// <summary>
        /// Device-specific operations can be used on the line.
        /// </summary>
        DeviceSpecific = 0x00000001,
        /// <summary>
        /// Device-specific features can be used on the line.
        /// </summary>
        DeviceSpecificFeature = 0x00000002,
        /// <summary>
        /// Forwarding of all addresses can be used on the line.
        /// </summary>
        Forward = 0x00000004,
        /// <summary>
        /// An outgoing call can be placed on this line using an unspecified address.
        /// </summary>
        MakeCall = 0x00000008,
        /// <summary>
        /// Media control can be set on this line.
        /// </summary>
        SetMediaControl = 0x00000010,
        /// <summary>
        /// Terminal modes for this line can be set.
        /// </summary>
        SetTerminal = 0x00000020,

        /// <summary>
        /// The lineSetLineDevStatus function can be invoked on the line device. 
        /// </summary>
        SetDeviceStatus = 0x00000040,      // TAPI v2.0
        /// <summary>
        /// The lineForward function can be used to forward calls on all address on the line to other numbers. 
        /// </summary>
        ForwardFwd = 0x00000080,      // TAPI v2.0
        /// <summary>
        /// The lineForward function (with an empty destination address) can be used to turn on the Do Not Disturb feature on all addresses on the line.
        /// </summary>
        ForwardDND = 0x00000100,      // TAPI v2.0
	}

    /// <summary>
    /// These bit-flag constants describe the roaming status of a line device. 
    /// </summary>
	[Flags]
	public enum RoamMode
	{
        /// <summary>
        /// The roam mode is currently unknown but may become known later.
        /// </summary>
		Unknown = 0x00000001,
        /// <summary>
        /// The roam mode is unavailable and will not be known.
        /// </summary>
		Unavailable = 0x00000002,
        /// <summary>
        /// The line is connected to the home network node.
        /// </summary>
		Home = 0x00000004,
        /// <summary>
        /// The line is connected to the Roam-A carrier and calls are charged accordingly.
		/// </summary>
		RoamA = 0x00000008,
        /// <summary>
        /// The line is connected to the Roam-B carrier and calls are charged accordingly.
        /// </summary>
		RoamB = 0x00000010,
	}

    /// <summary>
    /// These bit-flag constants describe the conditions under which calls to an address can be forwarded. 
    /// </summary>
	[Flags]
	public enum ForwardMode
	{
        /// <summary>
        /// Forward all calls unconditionally, regardless of their origin. Use this value when unconditional forwarding for internal and external calls cannot be controlled separately. Unconditional forwarding overrides forwarding on busy and/or no answer conditions. 
        /// </summary>
		Unconditional = 0x00000001,
        /// <summary>
        /// Forward all internal calls unconditionally. Use this value when unconditional forwarding for internal and external calls can be controlled separately.
        /// </summary>
		UnconditionalInternal = 0x00000002,
        /// <summary>
        /// Forward all external calls unconditionally. Use this value when unconditional forwarding for internal and external calls can be controlled separately.
        /// </summary>
		UnconditionalExternal = 0x00000004,
        /// <summary>
        /// Unconditionally forward all calls that originated at a specified address (selective call forwarding).
        /// </summary>
		UnconditionalSpecific = 0x00000008,
        /// <summary>
        /// Forward all calls on busy, regardless of their origin. Use this value when forwarding for internal and external calls on busy and on no answer cannot be controlled separately.
        /// </summary>
		Busy = 0x00000010,
        /// <summary>
        /// Forward all internal calls on busy. Use this value when forwarding for internal and external calls on busy and on no answer can be controlled separately. 
        /// </summary>
		BusyInternal = 0x00000020,
        /// <summary>
        /// Forward all external calls on busy. Use this value when forwarding for internal and external calls on busy and on no answer can be controlled separately.
        /// </summary>
		BusyExternal = 0x00000040,
        /// <summary>
        /// Forward on busy all calls that originated at a specified address using a selective call forwarding scheme. 
        /// </summary>
		BusySpecific = 0x00000080,
        /// <summary>
        /// Forward all calls on no answer, regardless of their origin. Use this value when call forwarding for internal and external calls on no answer cannot be controlled separately.
        /// </summary>
		NoAnswer = 0x00000100,
        /// <summary>
        /// Forward all internal calls on no answer. Use this value when forwarding for internal and external calls on no answer can be controlled separately. 
        /// </summary>
		NoAnswerInternal = 0x00000200,
        /// <summary>
        /// Forward all external calls on no answer. Use this value when forwarding for internal and external calls on no answer can be controlled separately.
        /// </summary>
		NoAnswerExternal = 0x00000400,
        /// <summary>
        /// Forward on no answer all calls that originated at a specified address (selective call forwarding).
        /// </summary>
		NoAnswerSpecific = 0x00000800,
        /// <summary>
        /// Forward all calls on busy/no answer, regardless of their origin. Use this value when forwarding for internal and external calls on busy and on no answer cannot be controlled separately. 
        /// </summary>
		BusyNoAnswer = 0x00001000,
        /// <summary>
        /// Forward all internal calls on busy/no answer. Use this value when call forwarding on busy and on no answer cannot be controlled separately for internal calls. 
        /// </summary>
		BusyNoAnswerInternal = 0x00002000,
        /// <summary>
        /// Forward all external calls on busy/no answer. Use this value when call forwarding on busy and on no answer cannot be controlled separately for internal calls.
        /// </summary>
		BusyNoAnswerExternal = 0x00004000,
        /// <summary>
        /// Forward on busy/no answer all calls that originated at a specified address using a selective call forwarding scheme. 
        /// </summary>
		BusyNoAnswerSpecific = 0x00008000,
        /// <summary>
        /// Calls are forwarded, but the conditions under which forwarding will occur are not known at this time. It is possible that the conditions may become known at a future time. TAPI versions 1.4 and later.
        /// </summary>
		Unknown = 0x00010000,      // TAPI v1.4
        /// <summary>
        /// Calls are forwarded, but the conditions under which forwarding will occur are not known, and will never be known by the service provider. TAPI versions 1.4 and later. 
        /// </summary>
		Unavailable = 0x00020000,      // TAPI v1.4
	}

    /// <summary>
    /// These bit-flag constants describe various address status items.
    /// </summary>
	[Flags]
	public enum AddressState
	{
        /// <summary>
        /// Address-status items other than those listed below have changed. The application should check the current address status to determine which items have changed. 
        /// </summary>
		Other = 0x00000001,
        /// <summary>
        /// The device-specific item of the address status has changed. 
        /// </summary>
		DeviceSpecific = 0x00000002,
        /// <summary>
        /// The address has changed to idle. It is not in use by any stations. 
        /// </summary>
		InUseZero = 0x00000004,
        /// <summary>
        /// The address has changed from idle or in use by many bridged stations to being in use by just one station. 
        /// </summary>
		InUseOne = 0x00000008,
        /// <summary>
        /// The monitored or bridged address has changed from being in use by one station to being in use by more than one station. 
        /// </summary>
		InUseMany = 0x00000010,
        /// <summary>
        /// The number of calls on the address has changed. This is the result of events such as a new incoming call, an outgoing call on the address, or a call changing its hold status. This flag covers changes in any of the members dwNumActiveCalls, dwNumOnHoldCalls and dwNumOnHoldPendingCalls in the LINEADDRESSSTATUS. The application should check all three of these members when it receives a LINE_ADDRESSSTATE (numCalls) message. 
        /// </summary>
		CallCount = 0x00000020,
        /// <summary>
        /// The forwarding status of the address has changed, including possibly the number of rings for determining a no-answer condition. The application should check the address status to determine details about the address's current forwarding status. 
        /// </summary>
		Forward = 0x00000040,
        /// <summary>
        /// The terminal settings for the address have changed. 
        /// </summary>
		Terminals = 0x00000080,
        /// <summary>
        /// Indicates that, due to configuration changes made by the user or other circumstances, one or more of the members in the LINEADDRESSCAPS structure for the address have changed. The application should use the lineGetAddressCaps function to read the updated structure. If a service provider sends a LINE_ADDRESSSTATE message containing this value to TAPI, TAPI will pass it along to applications that have negotiated TAPI version 1.4 or later; applications negotiating a previous API version will receive LINE_LINEDEVSTATE messages specifying LINEDEVSTATE_ReInit, requiring them to shut down and reinitialize their connection to TAPI to obtain the updated information. 
        /// </summary>
		CapabilitiesChanged = 0x00000100,      // TAPI v1.4
	}

    /// <summary>
    /// These constants indicate operations that can be invoked for a given address or call. 
    /// </summary>
	[Flags]
	public enum CallFeature
	{
        /// <summary>
        /// Accept the call
        /// </summary>
		Accept = 0x000000001,
        /// <summary>
        /// Add the call to the current conference
        /// </summary>
		AddToConference = 0x00000002,
        /// <summary>
        /// Answer the call
        /// </summary>
		Answer = 0x00000004,
        /// <summary>
        /// Perform a blind transfer on the call
        /// </summary>
		BlindTransfer = 0x00000008,
        /// <summary>
        /// Complete the call transfer
        /// </summary>
		CompleteTransfer = 0x00000020,
        /// <summary>
        /// Dial the destination number for the call
        /// </summary>
		Dial = 0x00000040,
        /// <summary>
        /// Drop the call
        /// </summary>
		Drop = 0x00000080,
        /// <summary>
        /// Generate digits on the call
        /// </summary>
		GenerateDigits = 0x00000200,
        /// <summary>
        /// Generate tones on the call
        /// </summary>
		GenerateTone = 0x00000400, // (lineGenerateTone).  
        /// <summary>
        /// Put the call on hold
        /// </summary>
		Hold = 0x00000800, // (lineHold).  
        /// <summary>
        /// Monitor digits on the call
        /// </summary>
		MonitorDigits = 0x00001000, // (lineMonitorDigits).  
        /// <summary>
        /// Monitor the call's media
        /// </summary>
		MonitorMedia = 0x00002000, // (lineMonitorMedia).
        /// <summary>
        /// Prepare the call for addition to a conference
        /// </summary>
		PrepareForConference = 0x00010000, // (linePrepareAddToConference).  
        /// <summary>
        /// Redirect the call to another destination
        /// </summary>
		Redirect = 0x00020000, // (lineRedirect). 
        /// <summary>
        /// Release current user-user information (TAPI version 1.4 and later).
        /// </summary>
		ReleaseUserUserInfo = 0x10000000, // (lineReleaseUserUserInfo). 
        /// <summary>
        /// Remove the call from the conference
        /// </summary>
		RemoveFromConference = 0x00040000, // (lineRemoveFromConference).  
        /// <summary>
        /// Send user-user information
        /// </summary>
		SendUserUserInfo = 0x00100000, // (lineSendUserUserInfo).  
        /// <summary>
        /// Set call parameters
        /// </summary>
		SetCallParameters = 0x00200000, // (lineSetCallParams)  
        /// <summary>
        /// Set QOS levels for the call. TAPI version 2.0 and later.
        /// </summary>
		SetQOS = 0x40000000, 
        /// <summary>
        /// Set the terminal to be used with the call
        /// </summary>
		SetTerminal = 0x00800000, // (lineSetTerminal).  
        /// <summary>
        /// Set up a conference
        /// </summary>
		SetupConference = 0x01000000, // (lineSetupConference).  
        /// <summary>
        /// Set call treatment. TAPI version 2.0 and later.
        /// </summary>
		SetCallTreatment = 0x20000000,
        /// <summary>
        /// Set up a transfer
        /// </summary>
		SetupTransfer = 0x02000000, // (lineSetupTransfer).  
        /// <summary>
        /// Perform a swap hold operation
        /// </summary>
		SwapHold = 0x04000000, // (lineSwapHold).
        /// <summary>
        /// Remove call from hold.
        /// </summary>
		Unhold = 0x08000000, 
	}

    /// <summary>
    /// The CallFeature2 constants list the supplemental features available for conferencing, transferring, and parking calls.
    /// </summary>
	[Flags]
	public enum CallFeature2
	{
        /// <summary>
        /// If this bit is on, a "no hold conference" can be created
        /// </summary>
		NoHoldConference = 0x00000001, // If this bit is on, a "no hold conference" can be created by using the LINECALLPARAMFLAGS_NoHoldConference option with lineSetupConference. The LINECALLFEATURE_SETUPCONF bit will also be on in the dwCallFeatures member in the LINECALLSTATUS structure. 
        /// <summary>
        /// If this bit is on, "one step transfer" can be created
        /// </summary>
        OneStepTransfer = 0x00000002,	// If this bit is on, "one step transfer" can be created by using the LINECALLPARAMFLAGS_OneStepTransfer option with lineSetupTransfer. The LINECALLFEATURE_SETUPTRANSFER bit will also be on in the dwCallFeatures member.  
        /// <summary>
        /// If this bit is on, the lineCompleteTransfer function can be used to resolve the transfer as a three-way conference. 
        /// </summary>
        TransferConference = 0x00000080,		// If this bit is on, the lineCompleteTransfer function can be used to resolve the transfer as a three-way conference. The LINECALLFEATURE_COMPLETETRANSF bit will also be on in the dwCallFeatures member. 
        /// <summary>
        /// If this bit is on, the lineCompleteTransfer function can be used to resolve the transfer as a normal transfer.
        /// </summary>
        TransferNormal = 0x00000040,		// If this bit is on, the lineCompleteTransfer function can be used to resolve the transfer as a normal transfer. The LINECALLFEATURE_COMPLETETRANSF bit will also be on in the dwCallFeatures member.  
	}

    /// <summary>
    /// 
    /// </summary>
    public enum GPRSPrecedence
    {
        /// <summary>
        /// Unknown
        /// </summary>
        Unknown = (0x00000000),
        /// <summary>
        /// Subscribed value stored in network
        /// </summary>
        Subscribed = (0x00000001),
        /// <summary>
        /// High priority
        /// </summary>
        High = (0x00000002),
        /// <summary>
        /// Normal priority
        /// </summary>
        Normal = (0x00000003),
        /// <summary>
        /// Low priority
        /// </summary>
        Low = (0x00000004),
    }

    /// <summary>
    /// Identifies the delay class. 
    /// </summary>
    public enum GPRSDelay
    {
        /// <summary>
        /// Unknown delay class.
        /// </summary>
        Unknown = (0x00000000),  // Unknown
        /// <summary>
        /// The subscribed value is stored in the network.
        /// </summary>
        Subscribed = (0x00000001),
        /// <summary>
        /// For a secure data unit (SDU) of 128 octets, mean transfer delay of &lt;0.5 seconds and 95 percentile delay of &lt;1.5 seconds. For an SDU of 1024 octets, mean transfer delay of &lt;2 seconds and 95 percentile delay of &lt;7 seconds.
        /// </summary>
        Predictive1 = (0x00000002),
        /// <summary>
        /// For an SDU of 128 octets, mean transfer delay of &lt;5 seconds and 95 percentile delay of &lt;25 seconds. For an SDU of 1024 octets, mean transfer delay of &lt;15 seconds and 95 percentile delay of &lt;75 seconds.
        /// </summary>
        Predictive2 = (0x00000003),
        /// <summary>
        /// For an SDU of 128 octets, mean transfer delay of &lt;50 seconds and 95 percentile delay of &lt;250 seconds. For an SDU of 1024 octets, mean transfer delay of &lt;75 seconds and 95 percentile delay of &lt;375 seconds.
        /// </summary>
        Predictive3 = (0x00000004),
        /// <summary>
        /// Best effort.
        /// </summary>
        BestEffort = (0x00000005),
    }

    /// <summary>
    /// Identifies the reliability class of a packet.
    /// </summary> 
    public enum GPRSReliability
    {
        /// <summary>
        /// Unknown reliability class.
        /// </summary>
        Unknown = (0x00000000),  // Unknown
        /// <summary>
        /// The subscribed value is stored in the network.
        /// </summary>
        Subscribed = (0x00000001),
        /// <summary>
        /// The application is error-sensitive and cannot handle data loss. Do not send real-time traffic.
        /// </summary>
        Class1 = (0x00000002),
        /// <summary>
        /// The application is error-sensitive and can handle infrequent data loss. Do not send real-time traffic.
        /// </summary>
        Class2 = (0x00000003),
        /// <summary>
        /// The application is error-sensitive and can handle data loss, GMM/SM, and short message service (SMS). Do not send real-time traffic.
        /// </summary>
        Class3 = (0x00000004),
        /// <summary>
        /// The application is error-sensitive and can handle data loss. Real-time traffic may be sent. (Requires appropriate settings for delay and throughput.)
        /// </summary>
        Class4 = (0x00000005),
        /// <summary>
        /// The application is not error-sensitive and can handle data loss. Real-time traffic may be sent. (Requires appropriate settings for delay and throughput.)
        /// </summary>
        Class5 = (0x00000006),
    }

    /// <summary>
    /// 
    /// </summary>
    public enum DataRate
    {
        /// <summary>
        /// Unknown speed
        /// </summary>
        Unknown = (0x00000000),
        /// <summary>
        /// Automatic selection of speed
        /// </summary>
        Automatic = (0x00000001),  //
        /// <summary>
        /// 300 bits per second (bps) (V. 21 compression setting)
        /// </summary>
        bps300_v21 = (0x00000002),
        /// <summary>
        /// 300 bps (V. 100 compression setting)
        /// </summary>
        bps300_v110 = (0x00000003),
        /// <summary>
        /// 300 bps (V. 100 compression setting)
        /// </summary>
        bps1200_v22 = (0x00000004),
        /// <summary>
        /// 1,200/75 bps (V.23 compression setting)
        /// </summary>
        bps1200_75_v23 = (0x00000005),
        /// <summary>
        /// 1,200 bps (V.100 compression setting)
        /// </summary>
        bps1200_v110 = (0x00000006),
        /// <summary>
        /// 1,200 bps (V.120 compression setting)
        /// </summary>
        bps1200_v120 = (0x00000007),
        /// <summary>
        /// 2,400 bps (V.22bis compression setting)
        /// </summary>
        bps2400_v22BIS = (0x00000008),
        /// <summary>
        /// 2,400 bps (V.26ter compression setting)
        /// </summary>
        bps2400_v26TER = (0x00000009),
        /// <summary>
        /// 2,400 bps (V.110 or X.31 flag stuffing compression setting)
        /// </summary>
        bps2400_v110 = (0x0000000a),
        /// <summary>
        /// 2,400 bps (V.120 compression setting)
        /// </summary>
        bps2400_v120 = (0x0000000b),
        /// <summary>
        /// 4,800 bps (V.32 compression setting)
        /// </summary>
        bps4800_v32 = (0x0000000c),
        /// <summary>
        /// 4800 bps (V.110 or X.31 flag stuffing compression setting)
        /// </summary>
        bps4800_v110 = (0x0000000d),
        /// <summary>
        /// 4,800 bps (V.120 compression setting)
        /// </summary>
        bps4800_v120 = (0x0000000e),
        /// <summary>
        /// 9,600 bps (V.32 compression setting) 
        /// <remarks>This value is the standard setting used for 9,600 bps circuit switched data calls.</remarks>
        /// </summary>
        bps9600_v32 = (0x0000000f),
        /// <summary>
        /// 9,600 bps (V.34 compression setting)
        /// </summary>
        bps9600_v34 = (0x00000010),
        /// <summary>
        /// 9,600 bps (V.110 or X.31 flag stuffing compression setting)
        /// <remarks>This value is the standard setting used for modemless "quickconnect" 9,600 bps circuit switched data calls.</remarks>
        /// </summary>
        bps9600_v110 = (0x00000011),
        /// <summary>
        /// 9,600 bps (V.120 compression setting)
        /// </summary>
        bps9600_v120 = (0x00000012),
        /// <summary>
        /// 14,400 bps (V.34 compression setting)
        /// </summary>
        bps14400_v34 = (0x00000013),
        /// <summary>
        /// 14,400 bps (V.100 or X.31 flag stuffing compression setting)
        /// </summary>
        bps14400_v110 = (0x00000014),
        /// <summary>
        /// 14,400 bps (V.120 compression setting)
        /// </summary>
        bps14400_v120 = (0x00000015),
        /// <summary>
        /// 19,200 bps (V.34 compression setting)
        /// </summary>
        bps19200_v34 = (0x00000016),
        /// <summary>
        /// 19,200 bps (V.110 or X.31 flag stuffing compression setting)
        /// </summary>
        bps19200_v110 = (0x00000017),
        /// <summary>
        /// 19,200 bps (V.120 compression setting)
        /// </summary>
        bps19200_v120 = (0x00000018),
        /// <summary>
        /// 28,800 bps (V.34 compression setting)
        /// </summary>
        bps28800_v34 = (0x00000019),
        /// <summary>
        /// 28,800 bps (V.110 or X.31 flag stuffing) compression setting
        /// </summary>
        bps28800_v110 = (0x0000001a),
        /// <summary>
        /// 28,800 bps (V.120 compression setting)
        /// </summary>
        bps28800_v120 = (0x0000001b),
        /// <summary>
        /// 38,400 bps (V.110 or X.31 flag stuffing compression setting)
        /// </summary>
        bps38400_v110 = (0x0000001c),
        /// <summary>
        /// 38,400 bps (V.120 compression setting)
        /// </summary>
        bps38400_v120 = (0x0000001d),
        /// <summary>
        /// 48,000 bps (V.110 or X.31 flag stuffing compression setting)
        /// </summary>
        bps48000_v110 = (0x0000001e),
        /// <summary>
        /// 48,000 bps (V.120 compression setting)
        /// </summary>
        bps48000_v120 = (0x0000001f),
        /// <summary>
        /// 56,000 bps (V.110 or X.31 flag stuffing compression setting)
        /// </summary>
        bps56000_v110 = (0x00000020),
        /// <summary>
        /// 56,000 bps (V.120 compression setting)
        /// </summary>
        bps56000_v120 = (0x00000021),
        /// <summary>
        /// 56,000 bps (bit transparent compression setting)
        /// </summary>
        bps56000_TRANSP = (0x00000022),
        /// <summary>
        /// 64,000 bps (bit transparent compression setting)
        /// </summary>
        bps64000_TRANSP = (0x00000023),
    }

    /// <summary>
    /// Identifies the peak throughput class, which specifies the expected data transfer rate across the network in bits per second.
    /// </summary>
    public enum PeakThroughput
    {
        /// <summary>
        /// Unknown peak throughput class.
        /// </summary>
        Unknown = (0x00000000),  // Unknown
        /// <summary>
        /// The subscribed value is stored in the network.
        /// </summary>
        Subscribed = (0x00000001),
        /// <summary>
        /// 8 kilobits per second (kbps)
        /// </summary>
        bps8000 = (0x00000002),  // kbit/second
        /// <summary>
        /// 16 kbps.
        /// </summary>
        bps16000 = (0x00000003),
        /// <summary>
        /// 32 kbps.
        /// </summary>
        bps32000 = (0x00000004),
        /// <summary>
        /// 64 kbps.
        /// </summary>
        bps64000 = (0x00000005),
        /// <summary>
        /// 128 kbps.
        /// </summary>
        bps128000 = (0x00000006),
        /// <summary>
        /// 256 kbps.
        /// </summary>
        bps256000 = (0x00000007),
        /// <summary>
        /// 512 kbps.
        /// </summary>
        bps512000 = (0x00000008),
        /// <summary>
        /// 1024 kbps.
        /// </summary>
        bps1024000 = (0x00000009),
        /// <summary>
        /// 2048 kbps.
        /// </summary>
        bps2048000 = (0x0000000a),
    }

    /// <summary>
    /// Identifies the mean throughput class, which specifies the average data transfer rate across the GPRS network in octets per hour.
    /// </summary>
    public enum MeanThroughput
    {
        /// <summary>
        /// Unknown mean throughput class.
        /// </summary>
        Unknown = (0x00000000),  // Unknown
        /// <summary>
        /// The subscribed value is stored in the network
        /// </summary>
        Subscribed = (0x00000001),
        /// <summary>
        /// 100 octets per hour
        /// </summary>
        bps100 = (0x00000002),  // octets/hour
        /// <summary>
        /// 200 octets per hour
        /// </summary>
        bps200 = (0x00000003),
        /// <summary>
        /// 500 octets per hour
        /// </summary>
        bps500 = (0x00000004),
        /// <summary>
        /// 1000 octets per hour
        /// </summary>
        bps1000 = (0x00000005),
        /// <summary>
        /// 2000 octets per hour
        /// </summary>
        bps2000 = (0x00000006),
        /// <summary>
        /// 5000 octets per hour
        /// </summary>
        bps5000 = (0x00000007),
        /// <summary>
        /// 10000 octets per hour
        /// </summary>
        bps10000 = (0x00000008),
        /// <summary>
        /// 20000 octets per hour
        /// </summary>
        bps20000 = (0x00000009),
        /// <summary>
        /// 50000 octets per hour
        /// </summary>
        bps50000 = (0x0000000a),
        /// <summary>
        /// 100000 octets per hour
        /// </summary>
        bps100000 = (0x0000000b),
        /// <summary>
        /// 200000 octets per hour
        /// </summary>
        bps200000 = (0x0000000c),
        /// <summary>
        /// 500000 octets per hour
        /// </summary>
        bps500000 = (0x0000000d),
        /// <summary>
        /// 1000000 octets per hour
        /// </summary>
        bps1000000 = (0x0000000e),
        /// <summary>
        /// 2000000 octets per hour
        /// </summary>
        bps2000000 = (0x0000000f),
        /// <summary>
        /// 5000000 octets per hour
        /// </summary>
        bps5000000 = (0x00000010),
        /// <summary>
        /// 10000000 octets per hour
        /// </summary>
        bps10000000 = (0x00000011),
        /// <summary>
        /// 20000000 octets per hour
        /// </summary>
        bps20000000 = (0x00000012),
        /// <summary>
        /// 50000000 octets per hour
        /// </summary>
        bps50000000 = (0x00000013),
        /// <summary>
        /// Best effort
        /// </summary>
        BestEffort = (0x00000014),  // Best effort
    }

    /// <summary>
    /// Call-waiting states
    /// </summary>
    [Flags]
    [Obsolete("Use CallWaitingState instead.",true)]
    public enum CallWaiting
    {
        /// <summary>
        /// 
        /// </summary>
        Enabled = 0x00000001,
        /// <summary>
        /// 
        /// </summary>
        Disabled = 0x00000002,
    }

    /// <summary>
    /// Call-waiting states
    /// </summary>
    [Flags]
    public enum CallWaitingState
    {
        /// <summary>
        /// 
        /// </summary>
        Enabled = 0x00000001,
        /// <summary>
        /// 
        /// </summary>
        Disabled = 0x00000002,
    }

    /// <summary>
    /// Line capability classes
    /// </summary>
    [Flags]
    [Obsolete("Use LineCapabilityClass instead")]
    public enum CapabilityClass
    {
        /// <summary>
        /// Voice service capability.
        /// </summary>
        Voice = 0x00000001,
        /// <summary>
        /// Data service capability.
        /// </summary>
        Data = 0x00000002,
        /// <summary>
        /// Fax service capability.
        /// </summary>
        Fax = 0x00000004,
        /// <summary>
        /// SMS service capability.
        /// </summary>
        SMS = 0x00000008,
        /// <summary>
        /// Synchronous data service capability.
        /// </summary>
        SynchronousData = 0x00000010,
        /// <summary>
        ///  Asynchronous data service capability.
        /// </summary>
        AsynchronousData = 0x00000020,
        /// <summary>
        /// Packet service capability.
        /// </summary>
        Packet = 0x00000040,
        /// <summary>
        /// Pad service capability.
        /// </summary>
        Pad = 0x00000080,
        /// <summary>
        /// Capable of all services
        /// </summary>
        All = 0x000000ff,
    }

    /// <summary>
    /// Line capability classes
    /// </summary>
    [Flags]
    public enum LineCapabilityClass
    {
        /// <summary>
        /// Voice service capability.
        /// </summary>
        Voice = 0x00000001,
        /// <summary>
        /// Data service capability.
        /// </summary>
        Data = 0x00000002,
        /// <summary>
        /// Fax service capability.
        /// </summary>
        Fax = 0x00000004,
        /// <summary>
        /// SMS service capability.
        /// </summary>
        SMS = 0x00000008,
        /// <summary>
        /// Synchronous data service capability.
        /// </summary>
        SynchronousData = 0x00000010,
        /// <summary>
        ///  Asynchronous data service capability.
        /// </summary>
        AsynchronousData = 0x00000020,
        /// <summary>
        /// Packet service capability.
        /// </summary>
        Packet = 0x00000040,
        /// <summary>
        /// Pad service capability.
        /// </summary>
        Pad = 0x00000080,
        /// <summary>
        /// Capable of all services
        /// </summary>
        All = 0x000000ff,
    }

    /// <summary>
    /// Line equipment states
    /// </summary>
    public enum EquipmentState
    {
        /// <summary>
        /// Minimum power state
        /// </summary>
        Minimum = 0x00000001,
        /// <summary>
        /// Only the receive radio is powered.
        /// </summary>
        ReceiveOnly = 0x00000002,
        /// <summary>
        /// Only the transmit radio is powered.
        /// </summary>
        TransmitOnly = 0x00000003,
        /// <summary>
        /// Receive and transmit radio are off.
        /// </summary>
        NoRadios = 0x00000004,
        /// <summary>
        /// Full power.
        /// </summary>
        Full = 0x00000005,
    }

    /// <summary>
    /// Line mute states
    /// </summary>
    public enum MuteState
    {
        /// <summary>
        /// 
        /// </summary>
        Enabled = 0x00000001,
        /// <summary>
        /// 
        /// </summary>
        Disabled = 0x00000002,
    }

    /// <summary>
    /// Operator Formats
    /// </summary>
    [Flags]
    public enum OperatorFormat
    {
        /// <summary>
        /// None
        /// </summary>
        None = 0x00000000,
        /// <summary>
        /// Short text
        /// </summary>
        AlphaShort = 0x00000001,
        /// <summary>
        /// Long text
        /// </summary>
        AlphaLong = 0x00000002,
        /// <summary>
        /// Numeric
        /// </summary>
        Numeric = 0x00000004,
    }

    /// <summary>
    /// Radio Presence
    /// </summary>
    public enum RadioPresence
    {
        /// <summary>
        /// A radio module is present.
        /// </summary>
        Present = 0x00000001,
        /// <summary>
        /// A radio module is not present.
        /// </summary>
        NotPresent = 0x00000002,
    }

    /// <summary>
    /// Radio States
    /// </summary>
    public enum RadioState
    {
        /// <summary>
        /// The radio hardware is in Radio OFF mode.
        /// </summary>
        Off = 0x00000001,
        /// <summary>
        /// The radio hardware is in Radio ON mode.
        /// </summary>
        On = 0x00000002,
        /// <summary>
        /// The radio state is unknown
        /// </summary>
        Unknown = 0x00000003,
    }

    /// <summary>
    /// Line registration status
    /// </summary>
    public enum RegistrationStatus
    {
        /// <summary>
        /// Registration is unknown.
        /// </summary>
        Unknown = 0x00000001,
        /// <summary>
        /// Registration denied.
        /// </summary>
        Denied = 0x00000002,
        /// <summary>
        /// Not registered.
        /// </summary>
        Unregistered = 0x00000003,
        /// <summary>
        /// Attempting to register.
        /// </summary>
        Attempting = 0x00000004,
        /// <summary>
        /// Registered at home.
        /// </summary>
        Home = 0x00000005,
        /// <summary>
        /// Registered for roam.
        /// </summary>
        Roam = 0x00000006,
        /// <summary>
        /// Registered for digital service.
        /// </summary>
        Digital = 0x00000007,
        /// <summary>
        /// Registered for analog service.
        /// </summary>
        Analog = 0x00000008,
    }

    /// <summary>
    /// Caller-ID states
    /// </summary>
    public enum CallerIDState
    {
        /// <summary>
        /// Send caller ID information.
        /// </summary>
        Enabled = 0x00000001,
        /// <summary>
        /// Do not send caller ID information.
        /// </summary>
        Disabled = 0x00000002,
    }

    /// <summary>
    /// These ordinal constants specify which event notification mechanism to use when initializing a session. 
    /// </summary>
    public enum InitializeParameterOptions
    {
        /// <summary>
        /// The application desires to use the Hidden Window event notification mechanism. This flag is exposed only to applications that negotiate a TAPI version of 2.0 or higher. 
        /// </summary>
        UseHiddenWindow = 01,
        /// <summary>
        /// The application desires to use the Event Handle event notification mechanism. This flag is exposed only to applications that negotiate a TAPI version of 2.0 or higher. 
        /// </summary>
        UseEvent = 0x02,
        /// <summary>
        /// The application desires to use the Completion Port event notification mechanism. This flag is exposed only to applications that negotiate a TAPI version of 2.0 or higher.
        /// </summary>
        UseCompletionPort = 0x03
    }
    /// <summary>
    /// These bit-flag constants describe a collection of Boolean line device status items. 
    /// </summary>
    [Flags]
    public enum StatusFlags
    {
        /// <summary>
        /// Specifies whether the line is connected to TAPI. If TRUE, the line is connected and TAPI is able to operate on the line device. If FALSE, the line is disconnected and the application is unable to control the line device through TAPI. 
        /// </summary>
        Connected = 0x01,
        /// <summary>
        /// Indicates whether the line has a message waiting. If TRUE, a message is waiting. If FALSE, no message is waiting. 
        /// </summary>
        MessageWaiting = 0x02,
        /// <summary>
        /// Indicates whether the line is in service. If TRUE, the line is in service. If FALSE, the line is out of service. 
        /// </summary>
        InService = 0x04,
        /// <summary>
        /// Indicates whether the line is locked or unlocked. This bit is most often used with line devices associated with cellular phones. Many cellular phones have a security mechanism that requires the entry of a password to enable the phone to place calls. This bit can be used to indicate to applications that the phone is locked and cannot place calls until the password is entered on the user interface of the phone so that the application can present an appropriate alert to the user. 
        /// </summary>
        Locked
    }
}
 

 