// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using System.IO;
using System.Text;

namespace OpenNETCF.Telephony
{
	/// <summary>
	/// Wrapper class for TAPI line
    /// </summary>
	public class Line: IDisposable
	{
		internal Telephony telephonyObject;
		private IntPtr m_hLine;
		private bool bSyncMakeCall = false;

        /// <summary>
        /// New in v1.1.  Gets the current <see cref="Telephony"/> object.
        /// </summary>
        internal Telephony Telephony
        {
            get
            {
                return telephonyObject;
            }
        }
		
		/// <summary>
		/// TAPI HLINE accessor
		/// </summary>
		public IntPtr hLine { get { return m_hLine; } }

	    /// <summary>
	    /// New Call delegate
	    /// </summary>
	    /// <param name="call"></param>
		public delegate void NewCallHandler(Call call);
	    
	    /// <summary>
	    /// New Call Event
	    /// </summary>
		public event NewCallHandler NewCall;
		
		internal Line(Telephony t, IntPtr hLine)
		{
			telephonyObject = t;
			m_hLine = hLine;
		}

		/// <summary>
		/// Asynchronous function to place a call
		/// </summary>
		/// <param name="Destination">String: phone number</param>
		/// <param name="CountryCode">int: country code, e.g. 1 for US</param>
		/// <param name="Params"><c>LINECALLPARAMS</c> structure data</param>
		/// <param name="Callback">Asynchronous callback. Will be invoked when the call is complete</param>
		/// <param name="State">User-defined state object. Can be null</param>
		/// <returns>IAsyncResult</returns>
		public IAsyncResult BeginMakeCall(string Destination, int CountryCode, /*LINECALLPARAMS*/ byte[] Params,  AsyncCallback Callback, object State)
		{
			bSyncMakeCall = false;
			MakeCallAsyncResult result = new MakeCallAsyncResult(0, Callback, State);

			lock(telephonyObject.PendingRequests.SyncRoot)
			{
				int ret = NativeMethods.lineMakeCall(m_hLine, out result.m_hCall, Destination, CountryCode, Params);
				result.ReplyID = ret;
				if ( ret > 0 )
					telephonyObject.PendingRequests.Add(ret, result);
			}
			return result;
		}

		/// <summary>
		/// BeginMakeCall counterpart
		/// </summary>
		/// <param name="ar">IAsyncResult returned by <c>BeginMakeCall</c></param>
		/// <returns>New Call object, or null if failed</returns>
		public Call EndMakeCall(IAsyncResult ar)
		{
			MakeCallAsyncResult mc = ar as MakeCallAsyncResult;
			if ( mc == null )
				throw new ArgumentException("Invalid parameter", "ar");
			if ( !mc.IsCompleted )
				mc.AsyncWaitHandle.WaitOne();
			Call call = telephonyObject.Calls[mc.hCall] as Call;
			if ( call == null )
			{
				call = new Call( mc.m_hCall );
				telephonyObject.Calls.Add(mc.m_hCall, call);
				call.LoadCallInfo();
				call.LoadCallState();
				//				call.OnLineCallInfo(
				//				LINECALLINFO lci = new LINECALLINFO(1024);
				//				lci.Store();
				//				int ret = NativeTapi.lineGetCallInfo(mc.m_hCall, lci.Data);
				//				if (ret >= 0)
				//				{
				//					lci.Load();
				//					call.m_line = tapiObject.Lines[lci.LinePtr] as Line;
				//				}
			}
			return call;
		}

	    /// <summary>
	    /// Make a call on the line
	    /// </summary>
	    /// <param name="Destination"></param>
	    /// <param name="CountryCode"></param>
	    /// <param name="SupressCallerID"></param>
	    /// <returns></returns>
		public Call MakeCall(string Destination, int CountryCode, bool SupressCallerID)
		{
			byte[] Params = null;
			if ( SupressCallerID )
			{
				// If we want to supress caller ID, we need to prepare LINECALLPARAMS
				CallParameters lp = new CallParameters(Marshal.SizeOf(typeof(CallParameters)) + Marshal.SizeOf(typeof(DeviceSpecificParams)));
				lp.DeviceSpecificOffset = Marshal.SizeOf(typeof(CallParameters));
                lp.DeviceSpecificSize = Marshal.SizeOf(typeof(DeviceSpecificParams));
				lp.Store();
                DeviceSpecificParams lcpds = new DeviceSpecificParams();
				lcpds.CallerIdOptions = CallerIDOptions.Block;
                int index = lp.DeviceSpecificOffset;
				ByteCopy.StructToByteArray(lcpds, ref index, lp.Data);
				Params = lp.Data;
			}
			return MakeCall(Destination, CountryCode, Params);
		}

	    /// <summary>
        /// Make a call on the line
	    /// </summary>
	    /// <param name="Destination"></param>
	    /// <param name="CountryCode"></param>
	    /// <param name="Params"></param>
	    /// <returns></returns>
		public Call MakeCall(string Destination, int CountryCode, /*LINECALLPARAMS*/ byte[] Params)
		{
			IAsyncResult ar = BeginMakeCall(Destination, CountryCode, Params, null, null);
			bSyncMakeCall = true;
			MakeCallAsyncResult mc = ar as MakeCallAsyncResult;
			bool bDone = false;
			while ( !bDone )
			{
				Monitor.Enter(mc);
				if ( mc.IsCompleted )
					bDone = true;
				Monitor.Exit(mc);
				Application.DoEvents();
			}
			//			if ( !(ar as MakeCallAsyncResult).IsCompleted )
			//				ar.AsyncWaitHandle.WaitOne();
			Call call = EndMakeCall(ar);
			return call;
		}

     
		internal void MakeCallCallback(IAsyncResult ar)
		{
		}
	    
        /// <summary>
        /// New Call event
        /// </summary>
        /// <param name="call"></param>
		internal protected void OnNewCall(Call call)
		{
			if ( NewCall != null && !bSyncMakeCall )
				NewCall(call);
		}

		#region IDisposable Members

		/// <summary>
		/// Cleanup
		/// </summary>
		public virtual void Dispose()
		{
			NativeMethods.lineClose(hLine);
			if ( telephonyObject.Lines.Contains(hLine) )
				telephonyObject.Lines.Remove(hLine);
		}

		#endregion

   
	}
}
