// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;

namespace OpenNETCF.Telephony
{
    #region TAPI Structures

    /// <summary>
    /// The LineMessage structure contains parameter values specifying a change in status of the line the application currently has open. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct LineMessage
    {
        /// <summary>
        /// Handle to either a line device or a call. The nature of this handle (line handle or call handle) can be determined by the context provided by MessageID
        /// </summary>
        public IntPtr hDevice;
        /// <bugs>
        /// <bug id="26">Cannot access the field MessageID from struct LineMessage</bug>
        /// <bug id="42">MessageID on LineMessage should be made public</bug>
        /// </bugs>
        public LineMessages MessageID; 
        /// <summary>
        /// Instance data passed back to the application, which was specified by the application in the CallBackInstance parameter of lineInitializeEx
        /// </summary>
        public IntPtr CallbackInstance;
        /// <summary>
        /// Parameter for the message.
        /// </summary>
        public IntPtr Param1;
        /// <summary>
        /// Parameter for the message.
        /// </summary>
        public IntPtr Param2;
        /// <summary>
        /// Parameter for the message.
        /// </summary>
        public IntPtr Param3;

        /// <summary>
        /// Returns the line message as a string
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            switch (MessageID)
            {
                case LineMessages.LINE_CALLINFO:
                    return string.Format("{0}:{1}", MessageID, (CallInfoState)Param1.ToInt32());

                case LineMessages.LINE_CALLSTATE:
                    switch ((CallState)Param1.ToInt32())
                    {
                        case CallState.Disconnected:
                            return string.Format("{0}:{1}:{2}", MessageID, (CallState)Param1.ToInt32(), (DisconnectMode)Param2.ToInt32());
                        default:
                            return string.Format("{0}:{1}", MessageID, (CallState)Param1.ToInt32());
                    }
                case LineMessages.LINE_REPLY:
                    return string.Format("{0}:{1}", MessageID, Param1);

                case LineMessages.LINE_DEVSPECIFIC:
                    switch ((LineDeviceSpecific)Param1.ToInt32())
                    {
                        case LineDeviceSpecific.RegisterState:
                            return string.Format("devspec:{0}:{1}", (LineDeviceSpecific)Param1.ToInt32(), (RegistrationStatus)Param2.ToInt32());
                        case LineDeviceSpecific.EquipmentStateChange:
                            return string.Format("devspec:{0}:{1}", (LineDeviceSpecific)Param1.ToInt32(), (EquipmentState)Param2.ToInt32());
                        default:
                            return string.Format("devspec:{0}", (LineDeviceSpecific)Param1.ToInt32());
                    }

                default:
                    return MessageID.ToString();
            }
        }

    }

    /// <summary>
    /// This structure describes the device-specific parameters
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct DeviceSpecificParams
    {
        /// <summary>
        /// Caller ID options
        /// </summary>
        public CallerIDOptions CallerIdOptions;
    }

    /// <summary>
    /// This structure describes an extension identifier. Extension identifiers are used to identify service provider-specific extensions for line devices. Multiple functions use this structure, including the lineNegotiateAPIVersion function and the TSPI_lineGetExtensionID function.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct ExtensionID
    {
        /// <summary>
        /// First part of the extension identifier.
        /// </summary>
        public IntPtr ExtensionID0;
        /// <summary>
        /// Second part of the extension identifier.
        /// </summary>
        public IntPtr ExtensionID1;
        /// <summary>
        /// Third part of the extension identifier.
        /// </summary>
        public IntPtr ExtensionID2;
        /// <summary>
        /// Fourth part of the extension identifier.
        /// </summary>
        public IntPtr ExtensionID3;
    }

    /// <summary>
    /// This structure describes the dialling parameters
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct DialParameters
    {
        /// <summary>
        /// Duration of a comma in the dialable address, in milliseconds.
        /// </summary>
        public int DialPause;
        /// <summary>
        /// Interdigit time period between successive digits, in milliseconds.
        /// </summary>
        public int DialSpeed;
        /// <summary>
        /// Duration of a digit, in milliseconds.
        /// </summary>
        public int DigitDuration;
        /// <summary>
        /// Maximum amount of time to wait for a dial tone when a 'W' is used in the dialable address, in milliseconds.
        /// </summary>
        public int WaitForDialtone;
    }

    /// <summary>
    /// The CallList structure describes a list of call handles.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal class CallList : PhoneBase
    {
        public CallList(int size)
            : base(size)
        {
            TotalSize = Data.Length; ;
        }
        /// <summary>
        /// 
        /// </summary>
        protected override int MinimumSize { get { return 1024; } }
        /// <summary>
        /// Maximum amount of time to wait for a dial tone when a 'W' is used in the dialable address, in milliseconds.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// Size for this data structure that is needed to hold all the returned information, in bytes.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// Size of the portion of this data structure that contains useful information, in bytes.
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Number of handles in the hCalls array.
        /// </summary>
        public int CallsNumEntries;
        /// <summary>
        /// Size of the array of call handles, in bytes.
        /// </summary>
        public int CallsSize;
        /// <summary>
        /// Offset from the beginning of the structure to the variably sized array of HCALL handles. The size of the array is specified by CallsSize.
        /// </summary>
        public int CallsOffset;
    }

    /// <summary>
    /// The AddressStatus structure describes the current status of an address.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class AddressStatus : PhoneBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="size"></param>
        public AddressStatus(int size)
            : base(size)
        {
            TotalSize = Data.Length; ;
        }
        /// <summary>
        /// 
        /// </summary>
        protected override int MinimumSize { get { return 1024; } }
        /// <summary>
        /// Total size allocated to this data structure, in bytes.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// Size for this data structure that is needed to hold all the returned information, in bytes.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// Size of the portion of this data structure that contains useful information, in bytes.
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Number of stations that are currently using the address.
        /// </summary>
        public int NumInUse;
        /// <summary>
        /// Number of calls on the address that are in call states other than idle, onhold, onholdpendingtransfer, and onholdpendingconference.
        /// </summary>
        public int NumActiveCalls;
        /// <summary>
        /// Number of calls on the address in the onhold state.
        /// </summary>
        public int NumOnHoldCalls;
        /// <summary>
        /// Number of calls on the address in the onholdpendingtransfer or onholdpendingconference state.
        /// </summary>
        public int NumOnHoldPendCalls;
        /// <summary>
        /// Address-related functions that can be invoked on the address in its current state.
        /// </summary>
        public int AddressFeatures;
        /// <summary>
        /// Number of rings set for this address before an unanswered call is considered as no answer.
        /// </summary>
        public int NumRingsNoAnswer;
        /// <summary>
        /// Number of entries in the array referred to by ForwardSize and ForwardOffset.
        /// </summary>
        public int ForwardNumEntries;
        /// <summary>
        /// Size of the forwarding information array, in bytes.
        /// </summary>
        public int ForwardSize;
        /// <summary>
        /// Offset from the beginning of the structure to the variably sized field that describes the address's forwarding information. 
        /// </summary>
        public int ForwardOffset;
        /// <summary>
        /// Size of the terminal modes array, in bytes.
        /// </summary>
        public int TerminalModesSize;
        /// <summary>
        /// Offset from the beginning of the structure to the variably sized device field containing an array with DWORD-sized entries
        /// </summary>
        public int TerminalModesOffset;
        /// <summary>
        /// Size of the device-specific field, in bytes.
        /// </summary>
        public int DeviceSpecificSize;
        /// <summary>
        /// Offset from the beginning of this structure to the variably sized device-specific field. 
        /// </summary>
        public int DeviceSpecificOffset;
    }

    /// <summary>
    /// The CallStatus structure describes the current status of a call.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class CallStatus : PhoneBase
    {
        /// <summary>
        /// Creates an instance of the CallStatus structure
        /// </summary>
        /// <param name="size"></param>
        public CallStatus(int size)
            : base(size)
        {
            TotalSize = Data.Length; ;
        }
        /// <summary>
        /// 
        /// </summary>
        protected override int MinimumSize { get { return 1024; } }
        /// <summary>
        /// Total size allocated to this data structure, in bytes.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// Size for this data structure that is needed to hold all the returned information, in bytes.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// Size of the portion of this data structure that contains useful information, in bytes.
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Current state of the call
        /// </summary>
        public CallState CallState;
        /// <summary>
        /// nterpretation of the CallStateMode member is call-state-dependent. In many cases, the value will be zero. 
        /// </summary>
        public int CallStateMode;
        /// <summary>
        /// Application's privilege for this call. 
        /// </summary>
        public CallPrivilege CallPrivilege;
        /// <summary>
        /// Flags that indicate the Telephony API functions that can be invoked on the call, given the availability of the feature in the device capabilities, the current call state, and call ownership of the invoking application. A zero indicates the corresponding feature cannot be invoked by the application on the call in its current state; a one indicates the feature can be invoked.
        /// </summary>
        public CallFeature CallFeatures;
        /// <summary>
        /// Size of the device-specific field, in bytes.
        /// </summary>
        public int DeviceSpecificSize;
        /// <summary>
        /// Offset from the beginning of the structure to the variably sized device-specific field. 
        /// </summary>
        public int DeviceSpecificOffset;
        /// <summary>
        /// Indicates additional functions can be invoked on the call, given the availability of the feature in the device capabilities, the current call state, and call ownership of the invoking application.
        /// </summary>
        public CallFeature2 CallFeatures2;
        /// <summary>
        /// Coordinated Universal Time at which the current call state was entered.
        /// </summary>
        public int StateEntryTimeLow;
        /// <summary>
        /// Coordinated Universal Time at which the current call state was entered.
        /// </summary>
        public int StateEntryTimeHigh;
    }

    /// <summary>
    /// The LineForward structure describes an entry of the forwarding instructions. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct LineForward
    {
        /// <summary>
        /// Creates an instance of LineForward from a byte array
        /// </summary>
        /// <param name="data"></param>
        /// <param name="offset"></param>
        /// <returns></returns>
        public static LineForward FromByteArray(byte[] data, int offset)
        {
            LineForward lf = new LineForward();
            lf.ForwardMode = (ForwardMode)BitConverter.ToInt32(data, offset);
            lf.CallerAddressSize = BitConverter.ToInt32(data, offset + 4);
            lf.CallerAddressOffset = BitConverter.ToInt32(data, offset + 8);
            lf.DestCountryCode = BitConverter.ToInt32(data, offset + 12);
            lf.DestAddressSize = BitConverter.ToInt32(data, offset + 16);
            lf.DestAddressOffset = BitConverter.ToInt32(data, offset + 20);
            return lf;
        }
        /// <summary>
        /// Types of forwarding. 
        /// </summary>
        public ForwardMode ForwardMode;
        /// <summary>
        /// Size of the variably sized field containing the address of a caller to be forwarded, in bytes.
        /// </summary>
        public int CallerAddressSize;
        /// <summary>
        /// Offset from the beginning of this structure to the variably sized field containing the address of a caller to be forwarded. 
        /// </summary>
        public int CallerAddressOffset;
        /// <summary>
        /// Country code of the destination address to which the call is to be forwarded.
        /// </summary>
        public int DestCountryCode;
        /// <summary>
        /// Size of the variably sized field containing the address of the address where calls are to be forwarded, in bytes.
        /// </summary>
        public int DestAddressSize;
        /// <summary>
        /// Offset from the beginning of this structure to the variably sized field containing the address of the address where calls are to be forwarded.
        /// </summary>
        public int DestAddressOffset;

    }

    // Artifically limited to one list entry
    /// <summary>
    /// The ForwardList structure describes a list of forwarding instructions. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class ForwardList
    {
        [Internal]
        byte[] m_data;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="size"></param>
        public ForwardList(int size)
        {
            m_data = new byte[size];
            TotalSize = size;
        }

        /// <summary>
        /// 
        /// </summary>
        public byte[] Data { get { return m_data; } }

        /// <summary>
        /// Total size of the data structure, in bytes.
        /// </summary>
        public int TotalSize
        {
            get { return BitConverter.ToInt32(m_data, 0); }
            set { Buffer.BlockCopy(BitConverter.GetBytes(value), 0, m_data, 0, 4); }
        }
        
        /// <summary>
        /// Number of entries in the array specified as LineForward[]
        /// </summary>
        public int NumEntries
        {
            get { return BitConverter.ToInt32(m_data, 4); }
            set { Buffer.BlockCopy(BitConverter.GetBytes(value), 0, m_data, 4, 4); }
        }
        
        /// <summary>
        /// Array of forwarding instructions.
        /// </summary>
        public LineForward[] List
        {
            get
            {
                LineForward[] lf = new LineForward[NumEntries];
                for (int i = 0; i < NumEntries; i++)
                    lf[i] = LineForward.FromByteArray(m_data, 8 + Marshal.SizeOf(typeof(LineForward)) * i);
                return lf;
            }
        }
    }

    /// <summary>
    /// The VarString structure is used for returning variably sized strings. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal class VarString : PhoneBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="size"></param>
        public VarString(int size)
            : base(size)
        {
            TotalSize = Data.Length; ;
        }

        /// <summary>
        /// Total size allocated to this data structure, in bytes.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// Size for this data structure that is needed to hold all the returned information, in bytes.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// Size of the portion of this data structure that contains useful information, in bytes.
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Format of the string. 
        /// </summary>
        public int StringFormat;
        /// <summary>
        /// Size of the string information, including the null terminator, in bytes.
        /// </summary>
        public int StringSize;
        /// <summary>
        /// Offset from the beginning of the structure to the variably sized device field containing the string information. 
        /// </summary>
        public int StringOffset;
    }

    #endregion
}
