// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Collections;
using System.Threading;

namespace OpenNETCF.Telephony

{
	/// <summary>
	/// General TAPI wrapper class
    /// </summary>
	public class Telephony
	{
		#region Internal data
		private IntPtr m_hLineApp;
		private int m_dwNumDev;
		private IntPtr hEvent = IntPtr.Zero; // Not currently used
		private Thread thTapi;

		/// <summary>
		/// Collection of open TAPI lines
		/// </summary>
		internal Hashtable Lines;
		/// <summary>
		/// Collection of active TAPI calls
		/// </summary>
		internal Hashtable Calls;
		/// <summary>
		/// Collection of pending TAPI asynchronous requests
		/// </summary>
		internal Hashtable PendingRequests;

		#endregion

		#region Public data
		/// <summary>
		/// Minimum suported API version. Set this before call to Initialize if needed
		/// </summary>
		public int APIVersionLow = 0x10004;
		/// <summary>
		/// Maximum suported API version. Set this before call to Initialize if needed
		/// </summary>
		public int APIVersionHigh = 0x20000;
		/// <summary>
		/// Number of devices registered with TAPI
		/// </summary>
		/// <value>int</value>
		public int NumberOfDevices { get { return m_dwNumDev; } }
		/// <summary>
		/// TAPI HLINEAPP
		/// </summary>
		/// <value></value>
		public IntPtr hLineApp { get { return m_hLineApp; } }
		#endregion

		#region Delegate definitions
		/// <summary>
		/// </summary>
		public delegate void ReinitializeHandler();
		/// <summary>
		/// </summary>
		public delegate void MessageHandler(LineMessage msg);
		#endregion

		#region Events
		/// <summary>
		/// Invoked when LINE_REINIT is received
		/// </summary>
		public event ReinitializeHandler Reinitialize;
		/// <summary>
		/// Invoked when TAPI message is received, but before it is processed
		/// </summary>
		public event MessageHandler LineMessage;
		#endregion

		/// <summary>
		/// Constructor. Creates an instance of Tapi object
		/// </summary>
		public Telephony()
		{
			Lines = new Hashtable();
			Calls = new Hashtable();

			PendingRequests = new Hashtable();
		}

		/// <summary>
		/// Intialize TAPI data. Throws a TelephonyException if unsuccessful
		/// </summary>
		/// <returns>returns result of the call to lineInitializeEx</returns>
		public int Initialize()
		{
			// We support API version up to 2.0
			int dwApiVersion = 0x20000;
			InitializeParameters2 initParams = new InitializeParameters2(hEvent);
			
			// Intialize TAPI app and throw an exception if failed
			int ret = NativeMethods.lineInitializeEx(out m_hLineApp, IntPtr.Zero, IntPtr.Zero, "MyApp", out m_dwNumDev, ref dwApiVersion, initParams);
			if ( ret != 0 )
				throw new TelephonyException(ret);

			// Start listen for events
			thTapi = new Thread(new ThreadStart(TapiThreadProc));
			thTapi.Start();

			//TODO: build device list

			return ret;
		}


		/// <summary>
		/// Stop all TAPI related activity. Perform cleanup
		/// </summary>
		public void Shutdown()
		{
			// Destroy active calls if any
			object[] arr = new object[Calls.Values.Count];
			Calls.Values.CopyTo(arr, 0);
			foreach( Call call in arr )
				call.Dispose();

			// Close lines
			arr = new object[Lines.Values.Count];
			Lines.Values.CopyTo(arr, 0);
			foreach( Line line in arr )
				line.Dispose();

			// Even though this flag is set to false, event thread will keep waiting indefinitely inside the call
			// to lineGetMessage. Calling lineShutdown will result in LINE_CLOSE being sent and thread successfully 
			// terminated
			stop = true;
			NativeMethods.lineShutdown(m_hLineApp);
		}

		/// <summary>
		/// Opens TAPI line
		/// </summary>
		/// <param name="deviceID">ID of the device to open. Must be less than <c>NumberOfDevices</c></param>
		/// <param name="mode">combination of <c>LINEMEDIAMODE</c> flags</param>
		/// <param name="priv">combination of <c>LINECALLPRIVILEGE</c> flags</param>
		/// <returns>Newly created Line object</returns>
		public Line CreateLine(int deviceID, MediaMode mode, CallPrivilege priv)
		{
			IntPtr hLine;
			int ret = NativeMethods.lineOpen(m_hLineApp, deviceID, out hLine, NegotiateVersion(deviceID), 0, IntPtr.Zero, priv, mode, IntPtr.Zero);
			if ( ret != 0 )
				throw new TelephonyException(ret);
			Line line = new Line(this, hLine);
			Lines.Add(hLine, line);
			return line;
		}

        /// <summary>
        /// Opens a <see cref="CellularLine"/>
        /// </summary>
        /// <param name="deviceID">ID of the device to open. Must be less than <c>NumberOfDevices</c></param>
        /// <param name="mode">combination of <c>LINEMEDIAMODE</c> flags</param>
        /// <param name="priv">combination of <c>LINECALLPRIVILEGE</c> flags</param>
        /// <returns>Returns the created <see cref="CellularLine"/> object</returns>
        /// <exception cref="TelephonyException">Throws a TelephonyException if the cellular line cannot be openned.</exception>
        public CellularLine CreateCellularLine(int deviceID, MediaMode mode, CallPrivilege priv)
        {
            IntPtr hLine;
            int ret = NativeMethods.lineOpen(m_hLineApp, deviceID, out hLine, NegotiateVersion(deviceID), 0, IntPtr.Zero, priv, mode, IntPtr.Zero);
            if (ret != 0)
                throw new TelephonyException(ret);
            CellularLine line = new CellularLine(this, hLine);
            Lines.Add(hLine, line);
            return line;
        }

		/// <summary>
		/// Retrieves device caps
		/// </summary>
		/// <param name="deviceID">ID of the device. Must be less than <c>NumberOfDevices</c></param>
		/// <param name="dc"></param>
		/// <returns>Filled-in LINEDEVCAPS structure</returns>
		public ErrorCode GetDeviceCapabilities(int deviceID, out DeviceCapabilities dc)
		{
			dc = new DeviceCapabilities(1024);
			dc.Store();
			int ret = NativeMethods.lineGetDevCaps(m_hLineApp, deviceID, NegotiateVersion(deviceID), 0, dc.Data);
			dc.Load();
			if ( (ErrorCode)ret == ErrorCode.StructureTooSmall )
			{
				dc = new DeviceCapabilities(dc.NeededSize);
				ret = NativeMethods.lineGetDevCaps(m_hLineApp, deviceID, NegotiateVersion(deviceID), 0, dc.Data);
				dc.Load();
			}
			return (ErrorCode)ret;
		}

		/// <summary>
		/// Negotiates API version with TAPI for a given device
		/// </summary>
		/// <param name="deviceID">ID of the device. Must be less than <c>NumberOfDevices</c></param>
		/// <returns>Negotiated API version</returns>
		public int NegotiateVersion(int deviceID)
		{
			int dwApiVersion;
			ExtensionID le;
			int ret = NativeMethods.lineNegotiateAPIVersion(m_hLineApp, deviceID, APIVersionLow, APIVersionHigh, out dwApiVersion, out le);
			if ( ret != 0 )
				throw new TelephonyException(ret);
			return dwApiVersion;
		}

	    /// <summary>
	    /// Event for receiving line messages
	    /// </summary>
	    /// <param name="msg"></param>
		protected void OnMessage(LineMessage msg)
		{
			if ( LineMessage != null )
				LineMessage(msg);
		}

	    /// <summary>
	    /// Event for when the line is reinitialized
	    /// </summary>
		protected void OnLineReinit()
		{
			Shutdown();
			Lines.Clear();
			Calls.Clear();
			PendingRequests.Clear();
			Initialize();
			if ( Reinitialize != null )
				Reinitialize();
		}

	    /// <summary>
	    /// Line closed event
	    /// </summary>
	    /// <param name="msg"></param>
		protected void OnLineClose(LineMessage msg)
		{
			IntPtr hLine = msg.hDevice;
			Line line = Lines[hLine] as Line;
			if ( line != null )
				line.Dispose();
		}

	    /// <summary>
	    /// Event when the line receives a reply
	    /// </summary>
	    /// <param name="msg"></param>
		protected void OnLineReply(LineMessage msg)
		{
			lock(PendingRequests.SyncRoot)
			{
				object ar = PendingRequests[(int)msg.Param1];
				if ( ar != null )
				{
					if ( ar is MakeCallAsyncResult )
					{
						((ar as MakeCallAsyncResult).AsyncWaitHandle as ManualResetEvent).Set();
						if ( (ar as MakeCallAsyncResult).Callback != null )
							(ar as MakeCallAsyncResult).Callback(ar as IAsyncResult);
					}
                    else if (ar is TapiAsyncResult)
                    {
                        //Added this for other async responses fro CellCore
                        ((TapiAsyncResult)ar).Response = msg.Param2.ToInt32();
                        ((ManualResetEvent)((TapiAsyncResult)ar).AsyncWaitHandle).Set();
                        if (((TapiAsyncResult)ar).Callback != null)
                            ((TapiAsyncResult)ar).Callback((IAsyncResult)ar);

                    }
					PendingRequests.Remove((int)msg.Param1);
				}
			}
		}


		/// <summary>
		/// Main event-listening loop
		/// </summary>
		private bool stop = false;
		private void TapiThreadProc()
		{
			while( !stop )
			{
				LineMessage msg;
				if ( NativeMethods.lineGetMessage(m_hLineApp, out msg, -1) == 0 )
				{
					OnMessage(msg);

					switch(msg.MessageID)
					{
						case LineMessages.LINE_CLOSE:
							OnLineClose(msg);
							break;

						case LineMessages.LINE_CALLINFO:
							OnLineCallInfo(msg);
							break;
						case LineMessages.LINE_CALLSTATE:
							OnLineCallState(msg);
							break;
						case LineMessages.LINE_REPLY:
							OnLineReply(msg);
							break;
						case LineMessages.LINE_APPNEWCALL:
							OnNewCall(msg);
							break;
					}
				}
			}
		}

		private void OnNewCall(LineMessage msg)
		{
			bool bNewCall = false;

			if (!Calls.Contains(msg.Param2))
			{
				Calls.Add(msg.Param2, new Call(msg.Param2));
				bNewCall = true;
			}

			if (Calls.Contains(msg.Param2))
			{
				Call call = Calls[msg.Param2] as Call;
                //new in v1.1 Added line message
                call.LineMessage = msg;
				call.m_addressID = msg.Param1.ToInt32();

				if (call.m_line == null)
				{
					Line line = Lines[msg.hDevice] as Line;

					call.m_line = line;
					if (bNewCall && line != null)
						line.OnNewCall(call);
				}
			}
		}

		private void OnLineCallInfo(LineMessage msg)
		{
			bool bNewCall = false;

			if (!Calls.Contains(msg.hDevice))
			{
				Calls.Add(msg.hDevice, new Call(msg.hDevice));
				bNewCall = true;
			}

			if (Calls.Contains(msg.hDevice))
			{
				Call call = Calls[msg.hDevice] as Call;
                //new in v1.1 Added line message
                call.LineMessage = msg;
				
				call.OnLineCallInfo(msg);
				if (call.m_line == null)
				{
					Line line = Lines[call.Info.LinePtr] as Line;

					call.m_line = line;
					if (bNewCall && line != null)
						line.OnNewCall(call);
				}
			}
		}
		
		private void OnLineCallState(LineMessage msg)
		{
			bool bNewCall = false;

			if (!Calls.Contains(msg.hDevice))
			{
				Calls.Add(msg.hDevice, new Call(msg.hDevice));
				bNewCall = true;
			}

			if (Calls.Contains(msg.hDevice))
			{
				Call call = Calls[msg.hDevice] as Call;
                //new in v1.1 Added line message
                call.LineMessage = msg;
				
				call.OnLineCallState(msg);
				if (bNewCall)
				{
					call.LoadCallInfo();

					Line line = Lines[call.Info.LinePtr] as Line;

					call.m_line = line;
					if (line != null)
						line.OnNewCall(call);
				}
			}
		}

	    /// <summary>
	    /// Gets the cellular line 
	    /// </summary>
	    /// <param name="mode"></param>
	    /// <param name="privilege"></param>
	    /// <returns></returns>
		[Obsolete("Use CellularLine instead.",true)]
        public Line GetCellularLine(MediaMode mode, CallPrivilege privilege)
		{
			for ( int i = 0; i < m_dwNumDev; i++ )
			{
				DeviceCapabilities dc = new DeviceCapabilities(1024);
				dc.Store();
				int dwVersion = NegotiateVersion(i);
				int ret = NativeMethods.lineGetDevCaps(hLineApp, i, dwVersion, 0, dc.Data);
				if ( ret < 0 )
					return null;
                if ((ErrorCode)ret == ErrorCode.StructureTooSmall)
				{
					dc.Data = new byte[dc.NeededSize];
					NativeMethods.lineGetDevCaps(hLineApp, i, dwVersion, 0, dc.Data);
				}
				dc.Load();
				if ( dc.ProviderName == NativeMethods.CELLTSP_PROVIDERINFO_STRING )
				{
					return CreateLine(i, mode, privilege);
				}
			}
			return null;
		}

        /// <summary>
        /// New for v1.1. Retreives the current cellular line available.
        /// </summary>
        /// <param name="mode">the <see cref="MediaMode">mode</see> to retreive the cellular line</param>
        /// <param name="privilege">the <see cref="CallPrivilege">privilege</see> to use when openning the line</param>
        /// <returns>Mehtods returns null if no appropriate cellular line is found else a <see cref="CellularLine"/></returns>
        public CellularLine CellularLine(MediaMode mode, CallPrivilege privilege)
        {
            for (int i = 0; i < m_dwNumDev; i++)
            {
                DeviceCapabilities dc = new DeviceCapabilities(1024);
                dc.Store();
                int dwVersion = NegotiateVersion(i);
                int ret = NativeMethods.lineGetDevCaps(hLineApp, i, dwVersion, 0, dc.Data);
                if (ret < 0)
                    return null;
                if ((ErrorCode)ret == ErrorCode.StructureTooSmall)
                {
                    dc.Data = new byte[dc.NeededSize];
                    NativeMethods.lineGetDevCaps(hLineApp, i, dwVersion, 0, dc.Data);
                }
                dc.Load();
                if (dc.ProviderName == NativeMethods.CELLTSP_PROVIDERINFO_STRING)
                {
                    return CreateCellularLine(i, mode, privilege);
                }
            }
            return null;
        }
	}
}
