// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// Base class for telephony functionality.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public abstract class PhoneBase
    {
        [Internal]
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 0)]
        byte[] m_data;
        
        /// <summary>
        /// Creates an instance of PhoneBase
        /// </summary>
        /// <param name="nSize"></param>
        public PhoneBase(int nSize)
        {
            m_data = new byte[Math.Min(nSize, MinimumSize)];
            Initialize(this);
        }
        
        /// <summary>
        /// Load the internal data from a byte array
        /// </summary>
        public void Load()
        {
            ByteCopy.ByteArrayToStruct(m_data, this);
        }
        
        /// <summary>
        /// Store the internal data in a byte array
        /// </summary>
        public void Store()
        {
            ByteCopy.StructToByteArray(this, m_data);
        }
        
        /// <summary>
        /// Get or set the buffer for holding internal data.
        /// </summary>
        public byte[] Data { get { return m_data; } set { m_data = value; } }

        /// <summary>
        /// Retrieve a string from the internal data
        /// </summary>
        public string GetString(int offset, int size)
        {
            if (offset + size > m_data.Length)
                return "";
            return Encoding.Unicode.GetString(m_data, offset, size).TrimEnd(' ', '\0');
        }

        /// <summary>
        /// Initialize the internal data structures
        /// </summary>
        protected object Initialize(object obj)
        {
            BindingFlags flags = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance;
            FieldInfo[] fields = obj.GetType().GetFields((BindingFlags) flags);
            
            foreach (FieldInfo fi in fields)
            {
                Type t = fi.FieldType;

                if (t.IsArray)
                {
                    object[] colAttr = fi.GetCustomAttributes(typeof(ArrayAttribute), true);

                    if (colAttr.Length != 1)
                        throw new NotSupportedException(string.Format("Field {0}: Must have Array attribute with size specified", fi.Name));

                    int size = (colAttr[0] as ArrayAttribute).Size;

                    fi.SetValue(obj, Array.CreateInstance(fi.FieldType.GetElementType(), size));
                }
                else if (t.IsValueType && !t.IsPrimitive)
                {
                    fi.SetValue(obj, Initialize(fi.GetValue(obj)));
                }
            }

            return obj;
        }

        // Override this in child objects. 
        // No point in allocating too little memory, because we do not create too many of these
        /// <summary>
        /// Minimum size of internal data storage.
        /// </summary>
        protected virtual int MinimumSize { get { return 256; } }
    }
}
